<?php

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Admin_Notices
 */
class Prodigy_Admin_Notices {

    const MYSQL_MINIMAL_VERSION = 5.6;

    public $notices = [
        'check_mysql_notice',
        'check_migration'
    ];

    /**
     * Prodigy_Admin_Notices constructor.
     */
    public function __construct() {
        add_action( 'admin_print_styles', array( $this, 'add_notices' ) );
    }

    /**
     * Set notice action
     */
    public function add_notices() {
        foreach ( $this->notices as $notice ) {
            add_action( 'admin_notices', array( $this, $notice ) );
        }
    }

    /**
     * Set check version notice
     */
    public function check_mysql_notice() {
        global $pagenow, $wpdb;

        if ( 'index.php' === $pagenow || ( isset($_GET['post_type']) && $_GET['post_type'] == Prodigy::get_prodigy_product_type() ) ) {
            $current_version = $wpdb->db_version();

            if ( (float) $current_version < self::MYSQL_MINIMAL_VERSION ) {
                self::update_notice( 'check_mysql_notice', true );
            } else {
                self::update_notice( 'check_mysql_notice', false );
            }

            if ( get_option( 'prodigy_admin_notice_check_mysql_notice' ) ) {
                include dirname( __FILE__ ) . '/notice_views/check-db-version-notice.php';
            }
        }
    }
    
    /**
     * Set check version notice
     */
    public function check_migration() {
        global $pagenow;
        
        if ( isset($_GET['post_type']) && $_GET['post_type'] == Prodigy::get_prodigy_product_type()  ) {

            if ( ! get_option(ProdigyUpdaterMigration::getCurrentOptionNameMigration()) ) {
                self::update_notice( 'check_migration', true );
            } else {
                self::update_notice( 'check_migration', false );
            }
            
            if ( get_option( 'prodigy_admin_notice_check_migration' ) ) {
                include dirname( __FILE__ ) . '/notice_views/check-db-migrate-notice.php';
            }
        }
    }

    /**
     * @param string $name
     * @param string $is_enable
     */
    public static function update_notice( $name, $is_enable ) {
        $option = get_option( 'prodigy_admin_notice_' . $name );
        if ( isset( $option ) ) {
            update_option( 'prodigy_admin_notice_' . $name, $is_enable );
        }
    }

}

new Prodigy_Admin_Notices();