<?php

/**
 * Prodigy admin notifications class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Admin_Notation
 */
class Prodigy_Admin_Notification {
    
    const NOTICE_FIELD = 'my_admin_notice_message';
    
    /**
     *  Display errors
     */
    public function displayAdminNotice() {
        $option      = get_option( self::NOTICE_FIELD );
        $message     = isset( $option['message'] ) ? $option['message'] : false;
        $noticeLevel = ! empty( $option['notice-level'] ) ? $option['notice-level'] : 'notice-error';
        
        if ( $message ) {
            echo "<div class='notice {$noticeLevel} is-dismissible'><p>" . esc_attr( $message ) . "</p></div>";
            delete_option( self::NOTICE_FIELD );
        }
    }
    
    /**
     * @param string $message
     */
    public static function displayError( $message ) {
        self::updateOption( $message, 'notice-error' );
    }
    
    /**
     * @param string $message
     */
    public static function displayWarning( $message ) {
        self::updateOption( $message, 'notice-warning' );
    }
    
    /**
     * @param string $message
     */
    public static function displayInfo( $message ) {
        self::updateOption( $message, 'notice-info' );
    }
    
    /**
     * @param string $message
     */
    public static function displaySuccess( $message ) {
        self::updateOption( $message, 'notice-success' );
    }
    
    /**
     * @param string $message
     * @param string $noticeLevel
     */
    protected static function updateOption( $message, $noticeLevel ) {
        update_option( self::NOTICE_FIELD, [
            'message'      => $message,
            'notice-level' => $noticeLevel
        ] );
    }
}
