<?php

/**
 * Prodigy admin product bulk edit class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Admin_Metaboxes
 */
class Prodigy_Admin_Meta_Boxes extends Prodigy_Product_Meta_Box {

    const GALLERY_PREFIX = 'prodigy_';
    const ID_THUMBNAIL_META = '_thumbnail_id';
    const GALLERY_PRODIGY = 'prodigy_gallery';

    /**
     * @var array $gallery_fields
     */
    public static $gallery_fields = [
        [
            'desc' => 'This is the gallery images on the single item page.',
            'id'   => self::GALLERY_PREFIX . 'gallery',
            'type' => 'gallery'
        ],
    ];

    /**
     * Prodigy_Admin_Metaboxes constructor.
     */
    public function __construct() {
        parent::__construct();
        add_action( 'add_meta_boxes', [ $this, 'prodigy_gallery_meta_box' ], 10 );
        add_action( 'save_post', [ $this, 'prodigy_save_gallery_meta' ], 10, 1 );
        add_action( 'save_post', [ $this, 'prodigy_set_default_feature_image' ], 11, 1 );
    }

    /**
     *  Register gallery meta box
     */
    public function prodigy_gallery_meta_box() {
        add_meta_box(
            'prodigy-product-images',
            __( 'Product Gallery', 'prodigy' ),
            [ $this, 'prodigy_show_gallery_meta_box' ],
            Prodigy::get_prodigy_product_type(),
            'side',
            'low'
        );
    }

    /**
     * @param $object
     */
    public function prodigy_show_gallery_meta_box( $object ) {
        global $post;

        // Use nonce for verification
        echo '<input type="hidden" name="gallery_meta_box_nonce" value="' . wp_create_nonce( basename( __FILE__ ) ) . '" />';

        // Begin the field table and loop
        echo '<table class="form-table">';

        foreach ( self::$gallery_fields as $field ) {
            // get value of this field if it exists for this post
            $meta = get_post_meta( $post->ID, $field['id'], true );
            // begin a table row with
            echo '<tr>
               <td>';
            switch ( $field['type'] ) {
                case 'media':
                    $close_button = null;
                    if ( $meta ) {
                        $close_button = '<span class="prodigy-close"></span>';
                    }
                    echo '<input id="prodigy_image" type="hidden" name="prodigy_image" value="' . esc_attr( $meta ) . '" />
                       <div class="prodigy-image-container">' . $close_button . '<img id="prodigy_image_src" src="' . wp_get_attachment_thumb_url( $this->prodigy_get_image_id( $meta ) ) . '"></div>
                       <input id="prodigy-image__button" type="button" value="Add Image" />';
                    break;
                case 'gallery':
                    $meta_html = null;
                    if ( $meta ) {
                        $meta_html  .= '<ul class="prodigy_gallery_list">';
                        $meta_array = explode( ',', $meta );
                        foreach ( $meta_array as $meta_gall_item ) {
                            $meta_html .= '<li>
                                                <div class="prodigy-gallery-container">
                                                <span class="prodigy-gallery__button-close">
                                                <img id="' . esc_attr( $meta_gall_item ) . '" src="' . wp_get_attachment_thumb_url( $meta_gall_item ) . '">
                                                </span>
                                                </div>
                                           </li>';
                        }
                        $meta_html .= '</ul>';
                        $meta_html .= '</ul>';
                    }
                    echo '<input id="prodigy_gallery" type="hidden" name="prodigy_gallery" value="' . esc_attr( $meta ) . '" />
                       <span id="prodigy_gallery_src">' . $meta_html . '</span>
                       <div class="prodigy-gallery__footer">
                       <input id="prodigy-gallery__button" class="button button-primary button-large" type="button" value="Add Gallery" />
                       </div>';
                    break;
            } //end switch
            echo '</td></tr>';
        } // end foreach
        echo '</table>'; // end table
    }

    /**
     * @param $image_url
     *
     * @return mixed
     */
    public function prodigy_get_image_id( $image_url ) {
        global $wpdb;
        $attachment = $wpdb->get_col( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE guid='%s';", $image_url ) );

        return $attachment[0];
    }

    /**
     * @param $post_id
     *
     * @return mixed
     */
    public function prodigy_save_gallery_meta( $post_id ) {
        if ( isset( $_POST['gallery_meta_box_nonce'] ) ) {
            if ( ! wp_verify_nonce( $_POST['gallery_meta_box_nonce'], basename( __FILE__ ) ) ) {
                return $post_id;
            }
        }

        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return $post_id;
        }

        // Check permissions
        if ( isset( $_POST['post_type'] ) ) {
            if ( 'page' == sanitize_text_field($_POST['post_type']) ) {
                if ( ! current_user_can( 'edit_page', $post_id ) ) {
                    return $post_id;
                }
            } elseif ( ! current_user_can( 'edit_post', $post_id ) ) {
                return $post_id;
            }
        }


        // Loop through meta fields
        foreach ( self::$gallery_fields as $field ) {
            if ( isset( $_POST[ $field['id'] ] ) ) {
                $new_meta_value = esc_attr( $_POST[ $field['id'] ] );
            }
            $meta_key   = $field['id'];
            $meta_value = get_post_meta( $post_id, $meta_key, true );
            // If theres a new meta value and the existing meta value is empty

            if ( isset( $new_meta_value ) ) {
                if ( $new_meta_value && empty($meta_value) ) {
                    update_post_meta( $post_id, $meta_key, $new_meta_value );
                    // If theres a new meta value and the existing meta value is different
                } elseif ( $new_meta_value && $new_meta_value != $meta_value ) {
                    update_post_meta( $post_id, $meta_key, $new_meta_value );
                } elseif ( $new_meta_value == null && $meta_value ) {
                    delete_post_meta( $post_id, $meta_key, $meta_value );
                }
            }
        }
    }


    /**
     * @param $post_id
     *
     * @return bool
     */
    public function prodigy_set_default_feature_image( $post_id ) {

        if ( has_post_thumbnail( $post_id ) ) {
            return true;
        }

        $prodigy_gallery = get_post_meta( $post_id, self::GALLERY_PRODIGY, true );
        $prodigy_gallery = explode( ',', $prodigy_gallery );

        if ( ! is_array( $prodigy_gallery ) ) {
            return true;
        }

        /*
         * set feature first valid img from gallery
         */
        foreach ( $prodigy_gallery as $value ) {
            if ( ! empty( (int) $value ) ) {

                set_post_thumbnail( $post_id, $value );
                unset( $prodigy_gallery[ array_search( $value, $prodigy_gallery ) ] );
                update_post_meta( $post_id, self::GALLERY_PRODIGY, implode( ",", $prodigy_gallery ) );

                break;
            }
        }

        return true;
    }

}

new Prodigy_Admin_Meta_Boxes();
