<?php

/**
 * Prodigy admin product inventory class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_Inventory_Box
 */
class Prodigy_Product_Inventory_Box extends Prodigy_Product_Meta_Box {

    /**
     * Prodigy_Admin_Metaboxes constructor.
     */
    public function __construct()
    {
        parent::__construct();
        add_action( 'add_meta_boxes', [ $this, 'prodigy_inventory_meta_box' ] );
        add_action( 'save_post', [ $this, 'prodigy_save_inventory_meta' ] );
        add_action( 'wp_ajax_prodigy-validate-sku', [ $this, 'validate_unique_sku' ] );
        add_action( 'wp_ajax_prodigy-variant-validate-sku', [ $this, 'validate_variant_unique_sku' ] );
        add_action( 'pre_post_update', [$this, 'is_validation'], 99, 1);
    }

    /**
     * @param int $post_id
     */
    public function is_validation( int $post_id ) {

        $error = false;
        if ( !isset( $_POST[ '_inline_edit' ] ) ) {

            if (!empty($_POST['inventory_sku'])) {
                if (empty(filter_var($_POST['inventory_sku']))) {
                    Prodigy_Admin_Notification::displayError(__('Field inventory_sku is not valid', 'prodigy'));
                    $error = true;
                }
            }

            if (isset($_POST['inventory_stock_status'])) {

                if (!filter_var($_POST['inventory_stock_status'], FILTER_VALIDATE_INT) && $_POST['inventory_stock_status'] !== '0') {
                    Prodigy_Admin_Notification::displayError(__('Field inventory_stock_status is not valid', 'prodigy'));
                    $error = true;
                }
            }

            if (!empty($_POST['check_product_manage_stock']) && $_POST['check_product_manage_stock'] == 'on') {

                if (!filter_var($_POST['inventory_stock_quantity'], FILTER_VALIDATE_INT) && $_POST['inventory_stock_quantity'] !== '0') {
                    Prodigy_Admin_Notification::displayError(__('Field inventory_stock_quantity is not valid', 'prodigy'));
                    $error = true;
                }

                if (!filter_var($_POST['check_allow_backorders'], FILTER_VALIDATE_INT) && $_POST['check_allow_backorders'] !== '0') {
                    Prodigy_Admin_Notification::displayError(__('Field check_allow_backorders is not valid', 'prodigy'));
                    $error = true;
                }
            }
        }

        if ( $error ) {
            $url = admin_url( 'post.php?post=' . $post_id ) . '&action=edit';
            wp_redirect( $url );
            exit;
        }
    }

    /**
     * Registration inventory meta-box
     */
    public function prodigy_inventory_meta_box() {
        add_meta_box(
            'prodigy-product-inventory',
            __( 'Product inventory', 'prodigy' ),
            [ $this, 'prodigy_show_inventory_meta_box'],
            Prodigy::get_prodigy_product_type(),
            'advanced',
            'high'
        );
    }

    /**
     * Render meta-box
     *
     * @param $post
     */
    public function prodigy_show_inventory_meta_box( $post ) {
        wp_nonce_field( 'prodigy_show_inventory_meta_box', 'prodigy_inventory_nonce' );

        $sku            = get_post_meta( $post->ID, 'product_sku', true );
        $stock_status   = get_post_meta( $post->ID, 'product_stock_status', true ) ? get_post_meta( $post->ID, 'product_stock_status', true ) : 0;
        $stock_quantity   = get_post_meta( $post->ID, 'product_stock_quantity', true );
        $allow_backorders = get_post_meta( $post->ID, 'product_allow_backorders', true );
        $check_product_manage_stock = get_post_meta( $post->ID, 'check_product_manage_stock', true );
        $check_allow_backorders = get_post_meta( $post->ID, 'check_allow_backorders', true );

        include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-inventory-meta-box.php';
    }

    /**
     * Post sku validate
     */
    public function validate_unique_sku() {
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $parse = parse_url( wp_get_referer() );
        parse_str( $parse['query'], $params );
        $sku  = $_POST['inventory_sku'] ?? $_POST['product_sku'];
        $post_id = $_POST['post_id'];
        $result = $this->product->is_check_unique_sku( sanitize_text_field($post_id), sanitize_text_field($sku) );

        echo $result = isset($result) ? 'false' : 'true';
        wp_die();
    }


    /**
     * variant sku validate
     */
    public function validate_variant_unique_sku() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $parse = parse_url( wp_get_referer() );
        parse_str( $parse['query'], $params );
        $sku  = $_POST['inventory_sku'] ?? $_POST['product_sku'];
        $post_id = $_POST['post_id'];
        $result = $this->product->is_check_unique_sku( sanitize_text_field($post_id), sanitize_text_field($sku) );

        echo $result = isset($result) ? 'false' : 'true';
        wp_die();
    }

    /**
     * Save data for meta-box
     *
     * @param $post_id
     *
     * @return mixed
     */
    public function prodigy_save_inventory_meta( $post_id ) {
        if ( !isset( $_POST['prodigy_inventory_nonce'] ) || !wp_verify_nonce( $_POST['prodigy_inventory_nonce'],'prodigy_show_inventory_meta_box' ) )
            return;

        if ( !current_user_can( 'edit_post', $post_id ))
            return;

        if ( isset( $_POST['inventory_sku'] ) ) {
            update_post_meta( $post_id, 'product_sku', sanitize_text_field( $_POST['inventory_sku'] ) );
        }

        if ( !empty( $_POST['check_product_manage_stock'] ) && $_POST['check_product_manage_stock'] == 'on' ) {

            update_post_meta( $post_id, 'check_product_manage_stock', sanitize_text_field( $_POST['check_product_manage_stock'] ) );

            if ( isset( $_POST['inventory_stock_quantity'] ) ) {
                update_post_meta( $post_id, 'product_stock_quantity', filter_var( $_POST['inventory_stock_quantity'], FILTER_SANITIZE_NUMBER_INT ) );
            }

            if ( isset( $_POST['check_allow_backorders'] ) ) {
                update_post_meta( $post_id, 'check_allow_backorders', filter_var( $_POST['check_allow_backorders'], FILTER_SANITIZE_NUMBER_INT ) );
            } else {
                delete_post_meta( $post_id, 'check_allow_backorders' );
            }

        } else {
            delete_post_meta( $post_id, 'check_product_manage_stock' );

            if ( isset( $_POST['inventory_stock_status'] ) ) {
                update_post_meta( $post_id, 'product_stock_status', filter_var( $_POST['inventory_stock_status'], FILTER_SANITIZE_NUMBER_INT ) );
            }

        }

        if ( isset( $_POST['inventory_allow_backorders'] ) ) {
            update_post_meta( $post_id, 'product_allow_backorders', sanitize_text_field( $_POST['inventory_allow_backorders'] ) );
        }
    }

}

new Prodigy_Product_inventory_Box();
