<?php

/**
 * Prodigy admin product list columns class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Product_List_Columns
 */
class Prodigy_Product_List_Columns
{
    /**
     * Prodigy_Product_List_Columns constructor.
     */
    public function __construct()
    {
        $type = get_option('pg_slug_product_type');

        add_filter( 'manage_'.$type.'_posts_columns', [ $this, 'prodigy_filter_posts_columns' ] );
        add_action( 'manage_'.$type.'_posts_custom_column', [ $this, 'prodigy_filter_posts_column' ], 10, 2);
    }

    /**
     * @param array $columns
     *
     * @return array
     */
    public function prodigy_filter_posts_columns( $columns )
    {
        $new_columns = [];

        if ( isset( $columns['cb'] ) ) {
            $new_columns['cb'] = $columns['cb'];
            unset( $columns['cb'] );
        }

        if ( isset( $columns['comments'] ) ) {
            unset( $columns['comments'] );
        }

        $new_columns['thumb'] = 'Image';
        $new_columns['sku'] = 'SKU';
        $new_columns['stock_status'] = 'Stock Status';
        $new_columns['regular_price'] = 'Regular Price';
        $new_columns['sale_price'] = 'Sale Price';
        $new_columns['status'] = 'Status';
        $columns = array_merge( $new_columns, $columns );

        $n_columns = [];
        $move = 'title';
        $before = 'thumb';
        foreach ( $columns as $key => $value ) {
            if ( $key === $before) {
                $n_columns[$move] = $move;
            }
            $n_columns[$key] = $value;
        }

        return $n_columns;
    }

    /**
     * @param array $column
     * @param int $post_id
     */
     public function prodigy_filter_posts_column( $column, $post_id ) {

         switch ( $column ) {
             case 'thumb':
                 $image = get_the_post_thumbnail( $post_id, [80, 80] );
                 if ( !empty($image) ) {
                     echo $image;
                 } else {
                    $image = plugin_dir_url( PRODIGY_PLUGIN_PATH . 'web/admin/images/placeholder.png' ) . 'placeholder.png';
                    echo '<img src="' . $image . '" alt="Thumb' . '" class="wp-post-image" height="48" width="48" />';
                 }
                 break;
             case 'sku':
                 $meta_sku = get_post_meta( $post_id, 'product_sku' );
                 $s_meta_sku = isset( $meta_sku[0] ) ? $meta_sku[0] : '';
                 echo esc_attr($s_meta_sku);
                 break;
             case 'stock_status':
                 $meta_status_sku = get_post_meta( $post_id, 'product_stock_status' );
                 $s_meta_status_sku = isset( $meta_status_sku[0] ) ? Prodigy_Product::$stock_status_mapper[$meta_status_sku[0]] : '';
                 echo esc_attr($s_meta_status_sku);
                 break;
             case 'stock_quantity':
                 $meta_stock_quantity = get_post_meta( $post_id, 'product_stock_quantity' );
                 $s_meta_stock_quantity = isset( $meta_stock_quantity[0] ) ? $meta_stock_quantity[0] : '';
                 echo esc_attr($s_meta_stock_quantity);
                 break;
             case 'regular_price':
                 $meta_regular_price = get_post_meta( $post_id, 'product_regular_price' );
                 $s_meta_regular_price = isset( $meta_regular_price[0] ) ? $meta_regular_price[0] : '';
                 echo esc_attr($s_meta_regular_price);
                 break;
             case 'sale_price':
                 $meta_sale_price = get_post_meta( $post_id, 'product_sale_price' );
                 $s_meta_sale_price = isset( $meta_sale_price[0] ) ? $meta_sale_price[0] : '';
                 echo esc_attr($s_meta_sale_price);
                 break;
             case 'status':
                 $meta_status = get_post_meta( $post_id, Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE );
                 if ( isset($meta_status[0]) && $meta_status[0] === Prodigy_Product_Synchronize::SUCCESS_STATUS ) {
                     echo "<span style='color: green;font-weight: bold'>".esc_html__('success', 'prodigy')."</span>";
                 } elseif ( isset($meta_status[0]) && $meta_status[0] === Prodigy_Product_Synchronize::PENDING_STATUS ) {
                     echo "<span style='color: orange;font-weight: bold'>".esc_html__('pending', 'prodigy')."</span>";
                 } else {
                     echo "<span style='color: red;font-weight: bold' class='retry'>".esc_html__('failed', 'prodigy')."</span>
                            <a href='#' id='product_synchronize' data-id=" .esc_attr($post_id) . ">retry</a>";
                 }
                 break;
             default:
                 echo '';
         }
    }

}

new Prodigy_Product_List_Columns();
