<?php

/**
 * Prodigy admin product list filters class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_List_Filters
 */
class Prodigy_Product_List_Filters {
    
    /**
     * Prodigy_Product_List_Filters constructor.
     */
    public function __construct() {
        add_action( 'restrict_manage_posts', [ $this, 'prodigy_add_filter_post_type_by_taxonomy' ] );
        add_action( 'restrict_manage_posts', [ $this, 'prodigy_add_filter_post_type_by_stock_status' ] );
        add_filter( 'parse_query', [ $this, 'prodigy_set_category_query' ] );
        add_filter( 'parse_query', [ $this, 'prodigy_set_tags_query' ] );
        add_filter( 'parse_query', [ $this, 'prodigy_set_stock_status_query' ] );
    }
    
    /**
     * Set stock status filter
     */
    public function prodigy_add_filter_post_type_by_stock_status() {
        global $typenow;
        
        if ( Prodigy::get_prodigy_product_type() === $typenow ) {
            $current_plugin = '';
            if ( isset( $_GET['slug'] ) ) {
                $current_plugin = sanitize_text_field( $_GET['slug'] );
            } ?>
            <select name="slug" id="slug">
                <option value="all" <?php selected( 'all', $current_plugin ); ?>><?php esc_html_e( 'All stock statuses', 'prodigy' );
                    ?></option>
                <?php foreach ( Prodigy_Product::$stock_status_mapper as $key => $value ) { ?>
                    <option value="<?php echo esc_attr( $key ); ?>" <?php selected( $key, $current_plugin ); ?>><?php echo esc_attr( $value ); ?></option>
                <?php } ?>
            </select>
        <?php }
    }
    
    /**
     * @param array $query
     */
    public function prodigy_set_stock_status_query( $query ) {
        global $pagenow;
        
        // Get the post type
        $post_type = isset( $_GET['post_type'] ) ? sanitize_text_field( $_GET['post_type'] ) : null;
        $slug      = isset( $_GET['slug'] ) ? sanitize_text_field( $_GET['slug'] ) : null;
        
        if ( is_admin() && $pagenow === 'edit.php' && $post_type === Prodigy::get_prodigy_product_type() && isset( $slug ) && $slug !== 'all' ) {
            $query->query_vars['meta_key']     = 'product_stock_status';
            $query->query_vars['meta_value']   = $slug;
            $query->query_vars['meta_compare'] = '=';
        }
    }
    
    /**
     * @param array $query
     */
    public function prodigy_set_category_query( $query ) {
        global $pagenow;
        
        $post_type = Prodigy::get_prodigy_product_type();
        $taxonomy  = Prodigy::get_prodigy_category_type();
        $q_vars    = &$query->query_vars;
        if (
            $pagenow === 'edit.php' && isset( $q_vars['post_type'] ) &&
            $q_vars['post_type'] === $post_type &&
            isset( $q_vars[ $taxonomy ] ) && is_numeric( $q_vars[ $taxonomy ] ) && $q_vars[ $taxonomy ] !== 0
        ) {
            $term                = get_term_by( 'id', $q_vars[ $taxonomy ], $taxonomy );
            $q_vars[ $taxonomy ] = $term->slug;
        }
    }
    
    /**
     * @param array $query
     */
    public function prodigy_set_tags_query( $query ) {
        global $pagenow;
        
        $post_type = Prodigy::get_prodigy_product_type();
        $taxonomy  = Prodigy::get_prodigy_tag_type();
        $q_vars    = &$query->query_vars;
        if (
            $pagenow === 'edit.php' && isset( $q_vars['post_type'] ) &&
            $q_vars['post_type'] === $post_type &&
            isset( $q_vars[ $taxonomy ] ) && is_numeric( $q_vars[ $taxonomy ] ) && $q_vars[ $taxonomy ] !== 0
        ) {
            $term                = get_term_by( 'id', $q_vars[ $taxonomy ], $taxonomy );
            $q_vars[ $taxonomy ] = $term->slug;
        }
    }
    
    /**
     * Set category filter
     */
    public function prodigy_add_filter_post_type_by_taxonomy() {
        global $typenow;
        
        $post_type = Prodigy::get_prodigy_product_type();
        $taxonomy  = Prodigy::get_prodigy_category_type();
        $terms     = get_terms( $taxonomy );
        if ( $typenow === $post_type && ! empty( $terms ) && isset($_GET[ $taxonomy ]) ) {
            $selected      = $_GET[ $taxonomy ] ?? sanitize_text_field( $_GET[ $taxonomy ] ) ?? '';
            $info_taxonomy = get_taxonomy( $taxonomy );
            
            $label_categories = $info_taxonomy->label ?? esc_html_e( 'Categories', 'prodigy' );
            $label_categories = esc_html( 'All ' . $label_categories );
            
            wp_dropdown_categories( [
                'show_option_all' => __( $label_categories, 'prodigy' ),
                'taxonomy'        => $taxonomy,
                'name'            => $taxonomy,
                'orderby'         => 'name',
                'selected'        => $selected,
                'show_count'      => true,
                'hide_empty'      => true,
            ] );
        };
    }
}

new Prodigy_Product_List_Filters();
