<?php

/**
 * Prodigy admin product class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-product-attributes.php';
include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-product.php';
include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-product-variants.php';
include_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-prodigy-admin-notification.php';
include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-api-client.php';

/**
 * Main meta-box class
 *
 * Class Prodigy_Product_Box
 */
class Prodigy_Product_Meta_Box {
    
    protected $product;
    protected $product_attributes;
    protected $product_variants;
    
    /**
     * Prodigy_Product_Meta_Box constructor.
     */
    public function __construct() {
        $this->product            = new Prodigy_Product();
        $this->product_attributes = new Prodigy_Product_Attributes();
        $this->product_variants   = new Prodigy_Product_Variants();

        $type = get_option('pg_slug_product_type');
        
        add_action( 'admin_enqueue_scripts', [ $this, 'prodigy_meta_box_load_style' ] );
        add_action( 'pre_post_update', [ $this, 'prodigy_validate_meta' ], 99, 2 );
        add_action( 'post_updated', [ $this, 'set_product_status' ], 10, 3 );
        add_action( 'admin_notices', [ new Prodigy_Admin_Notification(), 'displayAdminNotice' ] );
        add_action( 'delete_post', [ $this, 'prodigy_delete_product_attribute' ] );
        add_filter( 'post_type_labels_'.$type, [ $this, 'change_featured_image_labels' ], 10, 1 );
        add_action( 'admin_head', [ $this, 'hide_discussion_box' ] );
        add_action( 'wp_insert_post_data', [ $this, 'prepare_post_title' ], 20, 2 );
        
    }
    
    /**
     * @param $data
     * @param $postarr
     *
     * @return mixed
     */
    public function prepare_post_title( $data, $postarr ) {
        
        if( ! isset($_POST['post_type']) || $_POST['post_type'] != Prodigy::get_prodigy_product_type() ) return $data;
        if( get_current_screen()->id != Prodigy::get_prodigy_product_type() ) return $data;
        if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE  ) return $data;
        if ( ! current_user_can('edit_post', $postarr['ID'] ) ) return $data;

        $data['post_title'] = preg_replace('/[^ a-zа-яё\-\d]/ui', '', $data['post_title'] );

        return $data;
    }
    
    /**
     * @param int $post_ID
     * @param array $post_after
     * @param array $post_before
     */
    public function set_product_status( $post_ID, $post_after, $post_before ) {
        
        $stock_status = isset( $_POST['inventory_stock_status'] );
        if ( $stock_status ) {
            if ( isset( $post_after->post_status ) ) {
                $is_trash = ( $post_after->post_status === 'trash' ) ? true : null;
            }
            
            if ( ! isset( $is_trash ) ) {
                $this->update_status_product( $post_ID );
            }
            
        }
    }
    
    public function hide_discussion_box() {
        global $pagenow;
        if ( is_admin() && $pagenow == 'post-new.php' OR $pagenow == 'post.php' ) {
            echo "<script type='text/javascript'>
            jQuery(document).ready(function($) {
                jQuery('#commentstatusdiv').hide();
                jQuery('#commentstatusdiv-hide').attr('checked', false);
            });
            </script>
        ";
        }
    }
    
    
    /**
     * @param int $post_ID
     */
    public function update_status_product( $post_ID ) {
        global $wpdb;
        $is_hide_product = get_option( 'pg_inventory_hide_products' );
        
        if ( $is_hide_product && (int) $_POST['inventory_stock_status'] ) {
            
            $wpdb->get_results(
                $wpdb->prepare(
                    "UPDATE $wpdb->posts SET post_status = %s WHERE ID = %d ",
                    [ Prodigy::PRODIGY_HIDE_STATUS, $post_ID ]
                )
            );
        }
    }
    
    /**
     * @param object $labels
     *
     * @return object
     */
    public function change_featured_image_labels( $labels ) {
        $labels->not_found = 'No products found.';
        
        return $labels;
    }
    
    /**
     * Delete product attribute relation
     *
     * @param $post_id
     */
    public function prodigy_delete_product_attribute( $post_id ) {
    
        /**
         * clear aggregation attributes
         */
        $this->product_attributes->delete_aggregation_products_by_attribute( $post_id );
        
        $this->product_attributes->delete_product_attribute_relation( $post_id );
    }
    
    /**
     *  Load common scripts for meta boxes
     */
    public function prodigy_meta_box_load_style() {
        $screen       = get_current_screen();
        $current_page = $screen ? $screen->id : '';
        
        if ( in_array( $current_page, Prodigy::get_pages_list() ) ) {
            wp_enqueue_media();
            wp_enqueue_script( 'media-upload' );
            wp_enqueue_script( 'jquery.validation.min',
                dirname( plugin_dir_url( __FILE__ ) ) . '/web/admin/js/jquery.validation.min.js',
                [ 'jquery' ]
            );

            wp_enqueue_script(
                'jquery.inputmask.bundle.min',
                dirname( plugin_dir_url(__FILE__) ) . '/web/libraries/inputmask/js/jquery.inputmask.bundle.min.js',
                [ 'jquery' ]
            );

            wp_enqueue_style(
                'inputmask',
                dirname( plugin_dir_url( __FILE__ ) ) . '/web/libraries/inputmask/css/inputmask.css'
            );
        }
    }
    
    
    /**
     * @param int $post_id
     *
     * @return mixed
     */
    public function prodigy_validate_meta( $post_id ) {
        $error = false;
        if ( ! isset( $_POST['_inline_edit'] ) ) {
            if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
                return $post_id;
            }
            
            if ( ! isset( $_POST['post_type'] ) ) {
                return $post_id;
            }
            
            if ( ! current_user_can( 'edit_posts', $post_id ) ) {
                return $post_id;
            }
            
            
            foreach ( $_POST as $key => $item ) {
                switch ( $key ) {
                    case 'post_title':
                        if ( trim( $_POST[ $key ] ) === '' ) {
                            Prodigy_Admin_Notification::displayError( esc_html__( 'Product name is required' ) );
                            $error = true;
                            break;
                        }
//                    TODO - for description field
//                    case 'content':
//                        if ( trim( $_POST[ $key ] ) === '' ) {
//                            Prodigy_Admin_Notification::displayError( esc_html__( 'Product description is required' ) );
//                            $error = true;
//                            break;
//                        }
                    case 'regular_price':
                        if ( $_POST['regular_price'] == 0 ) {
                            break;
                        } elseif ( empty( filter_var( $_POST['regular_price'], FILTER_VALIDATE_FLOAT ) ) ) {
                            $error = true;
                            Prodigy_Admin_Notification::displayError( esc_html__( 'Product regular price is not valid' ) );
                            break;
                        }
                    case 'inventory_sku':
                        if ( trim( $_POST[ $key ] ) === '' ) {
                            Prodigy_Admin_Notification::displayError( esc_html__( 'Invalid product SKU' ) );
                            $error = true;
                            break;
                        } else {
                             $is_valid = $this->product->is_check_unique_sku($post_id, $_POST['inventory_sku']);

                             if ($is_valid) {
                                 Prodigy_Admin_Notification::displayError( esc_html__( 'SKU is not unique, please change to another' ) );
                                 $error = true;
                                 break;
                             }
                        }
                }
                
                if ( $error ) {
                    $url = admin_url( 'post.php?post=' . $post_id ) . '&action=edit';
                    wp_redirect( $url );
                    exit;
                }
            }
        }
        
        return $_POST;
    }
}
