<?php

/**
 * Prodigy admin product price class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_Quick_Edit
 */
class Prodigy_Product_Quick_Edit {

    /**
     * Prodigy_Product_List_Quick_Edit constructor.
     */
    public function __construct() {
        add_action( 'quick_edit_custom_box', [ $this, 'prodigy_update_quick_edit' ], 10, 3 );
        add_action( 'wp_ajax_set-current-post-id', [ $this, 'prodigy_set_post_id' ] );
        add_action( 'pre_post_update', [$this, 'prodigy_quick_edit_validate_price'], 99, 2 );
        add_action( 'save_post', [ $this, 'prodigy_save_quick_edit_data' ], 10, 2 );
        add_action( 'admin_head-edit.php', [ $this, 'custom_display_quick_edit'], 10, 1  );
    }


    public function custom_display_quick_edit()
    {
        global $current_screen;

        if( 'edit-'.Prodigy::get_prodigy_product_type() != $current_screen->id ) {
            return;
        }

        ?>

        <script type="text/javascript">
            jQuery(document).ready( function($) {
                $('span:contains("Password")').each(function (i) {
                    $(this).parent().parent().remove();
                });
                $('span:contains("Date")').each(function (i) {
                    $(this).parent().remove();
                });
                $('.inline-edit-date').each(function (i) {
                    $(this).remove();
                });
            });
        </script>
        <?php
    }

    /**
     * @param int $post_id
     * @param string $sku
     *
     * @return bool
     */
    public function is_check_unique_sku( $post_id, $sku ) {
        global $wpdb;

        $post_type = Prodigy::get_prodigy_product_type();

        $type = 'product_sku';
        $post_exists = $wpdb->get_var( $wpdb->prepare( "SELECT * FROM wp_postmeta as meta LEFT JOIN wp_posts as post ON post.ID = meta.post_id WHERE post.ID<>'%d' AND meta.meta_key = '%s' AND meta.meta_value = '%s' AND post.post_type = '%s' ", [ $post_id, $type, $sku, $post_type ] ) );

        return empty( $post_exists );
    }


    /**
     * @param int $post_id
     */
    public function prodigy_quick_edit_validate_price( $post_id ) {
        $error = false;

        if ( isset( $_POST[ '_inline_edit' ] ) ) {

            $is_valid_sku = $this->is_check_unique_sku( $post_id, sanitize_text_field($_POST['product_sku']) );

            if (!$is_valid_sku) {
                $error = esc_html__('You need enter valid sku', 'prodigy');
            }

            if ( !empty( $_POST['product_quick_regular_price'] ) ) {
                if ( empty( filter_var($_POST['product_quick_regular_price'], FILTER_VALIDATE_FLOAT) ) ) {
                    $error = esc_html__('Field Regular price is not valid', 'prodigy');
                }
            }

            if ( !empty($_POST['product_quick_sale_price']) ) {
                if ( empty( filter_var($_POST['product_quick_sale_price'], FILTER_VALIDATE_FLOAT) ) ) {
                    $error = esc_html__('Field Sale price is not valid', 'prodigy');
                }
            }

            if ( !empty( $_POST['product_stock_quantity'] ) ) {
                if ( empty( filter_var($_POST['product_stock_quantity'], FILTER_VALIDATE_INT) ) ) {
                    $error = esc_html__('Field Stock quantity is not valid', 'prodigy');
                }
            }

            if ( abs( $_POST['product_quick_regular_price'] ) < abs( $_POST['product_quick_sale_price'] ) ) {
                $error = esc_html__('Sales price must be lower than regular price', 'prodigy');
            }
        }

        if ( $error ) {
            echo $error;
            exit;
        }
    }

    /**
     * Get product data
     */
    public function prodigy_set_post_id() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $post_id = $_POST['post_id'];
        $meta_sale_price = get_post_meta( $post_id, 'product_sale_price' );
        $response['sale_price'] = isset( $meta_sale_price ) ? $meta_sale_price[0] : '';
        $meta_regular_price = get_post_meta( $post_id, 'product_regular_price' );
        $response['regular_price'] = isset( $meta_regular_price ) ? $meta_regular_price[0] : '';
        $meta_sku = get_post_meta( $post_id, 'product_sku' );
        $response['sku'] = isset( $meta_sku ) ? $meta_sku[0] : '';
        $meta_stock_status = get_post_meta( $post_id, 'product_stock_status' );
        $response['stock_status'] = isset( $meta_stock_status ) ? $meta_stock_status[0] : '';
        $meta_stock_quantity = get_post_meta( $post_id, 'product_stock_quantity' );
        $response['stock_quantity'] = isset( $meta_stock_quantity ) ? $meta_stock_quantity[0] : '';
        $allow_backorders = get_post_meta( $post_id, 'product_allow_backorders' );
        $response['allow_backorders'] = isset( $allow_backorders ) ? $allow_backorders[0] : '';
        
        foreach ($response as $item => $value) {
            $response[$item] = esc_attr($value);
        }

        echo wp_json_encode( $response );
        wp_die();
    }

    /**
     * @param string $column
     * @param string $post_type
     */
    public function prodigy_update_quick_edit( $column, $post_type ) {
        if ( Prodigy::get_prodigy_product_type() !== $post_type ) {
            return;
        }

        if ( $column === 'regular_price' ) {
            include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-quick-edit.php';
        }
    }

    /**
     * Save data
     *
     * @param string $post_id
     *
     * @return mixed
     */
    public function prodigy_save_quick_edit_data( $post_id ) {
        if ( empty( $_POST ) ) {
            return $post_id;
        }

        // don't save for autosave
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return $post_id;
        }

        if ( !isset( $_POST[ '_inline_edit' ] )) {
            return $post_id;
        }

        if ( isset($_POST['product_quick_regular_price']) ) {
            update_post_meta( $post_id, 'product_regular_price', sanitize_text_field( $_POST['product_quick_regular_price'] ) );
        }

        if ( isset($_POST['product_quick_sale_price']) ) {
            update_post_meta( $post_id, 'product_sale_price', sanitize_text_field( $_POST['product_quick_sale_price']
            ) );
        }

        if ( isset($_POST['product_sku']) ) {
            update_post_meta( $post_id, 'product_sku', sanitize_text_field( $_POST['product_sku'] ) );
        }

        if ( isset($_POST['product_stock_status']) ) {
            update_post_meta( $post_id, 'product_stock_status', sanitize_text_field( $_POST['product_stock_status'] ) );
        }

        if ( isset($_POST['product_stock_quantity']) ) {
            update_post_meta( $post_id, 'product_stock_quantity', sanitize_text_field( $_POST['product_stock_quantity'] ) );
        }

        if ( isset($_POST['product_allow_backorders']) ) {
            update_post_meta( $post_id, 'product_allow_backorders', sanitize_text_field( $_POST['product_allow_backorders'] ) );
        }
    }
}

new Prodigy_Product_Quick_Edit();
