<?php

/**
 * Prodigy admin product related products class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_Related_Box
 */
class Prodigy_Product_Related_Box extends Prodigy_Product_Meta_Box {

    /**
     * Prodigy_Product_UpSell_Box constructor.
     */
    public function __construct()
    {
        parent::__construct();
        add_action( 'add_meta_boxes', [ $this, 'prodigy_related_products_meta_box' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'prodigy_load_related_products_style' ] );
        add_action( 'save_post', [ $this, 'prodigy_save_related_products_meta' ] );
        add_action( 'wp_ajax_prodigy-get-related-products', [ $this, 'prodigy_get_related_products' ] );
        add_action( 'pre_post_update', [$this, 'is_validation'], 99, 1);
    }

    /**
     * Get products from autocomplete
     */
    public function prodigy_get_related_products( ) {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $products = $this->product->get_product_from_autocomplete( sanitize_text_field($_POST['search']), sanitize_text_field($_POST['post_id']) );

        $response = [];
        foreach ( $products as $key=>$product ) {
            $response[$key] = esc_attr($product['post_title']);
        }

        echo wp_json_encode( $response );
        die();
    }

    /**
     * Create meta box
     */
    public function prodigy_related_products_meta_box() {
        add_meta_box(
            'prodigy-product-upsell',
            esc_html__( 'Related products', 'prodigy' ),
            [ $this, 'prodigy_show_related_meta_box'],
            Prodigy::get_prodigy_product_type(),
            'advanced',
            'default'
        );
    }

    /**
     * Render view
     *
     * @param object $post
     */
    public function prodigy_show_related_meta_box( $post ) {
        // Use nonce for verification
        wp_nonce_field( 'prodigy_show_related_meta_box', 'prodigy_related_products_nonce' );

        $upsell_products = get_post_meta( $post->ID, Prodigy_Product::PRODUCT_RELATED_UPSELL_META );
        $crossell_products = get_post_meta( $post->ID, Prodigy_Product::PRODUCT_RELATED_CROSSELL_META );
        $crossell_products_ids = maybe_unserialize( $crossell_products );
        $upsell_products_ids = maybe_unserialize( $upsell_products );


        if ( isset( $upsell_products_ids[0] ) ) {
            $upsell_product_names = $this->product->get_product_names_by_ids( $upsell_products_ids[0] ) ;
        }
        if ( isset($crossell_products_ids[0]) ) {
            $crossell_product_names = $this->product->get_product_names_by_ids( $crossell_products_ids[0] );
        }

        include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-related-meta-box.php';
    }

    /**
     *  Include scripts
     */
    public function prodigy_load_related_products_style() {
        $screen       = get_current_screen();
        $current_page = $screen ? $screen->id : '';

        if ( in_array( $current_page, Prodigy::get_pages_list() ) ) {
            wp_enqueue_script(
                'tag-it.min',
            dirname( plugin_dir_url(__FILE__) ) . '/web/admin/js/tag-it.min.js',
                [ 'jquery' ],
                false,
                true
            );
        }
    }

    /**
     * @param int $post_id
     */
    public function is_validation( int $post_id ) {

        $error = false;

        if ( isset($_POST['prodigy_related_products_nonce']) && wp_verify_nonce($_POST['prodigy_related_products_nonce'], 'prodigy_show_related_meta_box')) {
            if (!current_user_can('edit_post', sanitize_text_field($post_id))) {
                Prodigy_Admin_Notification::displayError(__('You cannot edit this post.', 'prodigy'));
                $error = true;
            }
        }

        if ($error) {
            $url = admin_url('post.php?post=' . $post_id) . '&action=edit';
            wp_redirect($url);
            exit;
        }
    }

    /**
     * @param int $post_id
     */
    public function prodigy_save_related_products_meta( $post_id ) {

        if ( isset($_POST['upsell']) && !empty( $_POST['upsell'] ) ) {

            $related_products = explode( ",", filter_var( $_POST['upsell'], FILTER_SANITIZE_STRING ) );
            $related_product_ids = $this->product->get_product_ids_ny_name( $related_products );

            $upsell_ids = [];
            foreach ( $related_product_ids as $key=>$item ) {
                $upsell_ids[$key] = $item['id'];
            }

            update_post_meta( $post_id, Prodigy_Product::PRODUCT_RELATED_UPSELL_META, $upsell_ids );
        } else {
            delete_post_meta( $post_id, Prodigy_Product::PRODUCT_RELATED_UPSELL_META );
        }

        if ( isset($_POST['crossells']) && !empty( $_POST['crossells'] ) ) {
            $related_products = explode( ",", filter_var( $_POST['crossells'], FILTER_SANITIZE_STRING ) );
            $related_product_ids = $this->product->get_product_ids_ny_name( $related_products );

            $crossell_ids = [];
            foreach ($related_product_ids as $key=>$item) {
                $crossell_ids[$key] = $item['id'];
            }

            update_post_meta( $post_id, Prodigy_Product::PRODUCT_RELATED_CROSSELL_META, $crossell_ids );
        } else {
            delete_post_meta($post_id, Prodigy_Product::PRODUCT_RELATED_CROSSELL_META);
        }
    }
}

new Prodigy_Product_Related_Box();
