<?php

/**
 * Prodigy admin product shipping class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_Shipping_Box
 */
class Prodigy_Product_Shipping_Box extends Prodigy_Product_Meta_Box {

    /**
     * Prodigy_Admin_Metaboxes constructor.
     */
    public function __construct()
    {
        parent::__construct();
        add_action( 'add_meta_boxes', [ $this, 'prodigy_shipping_meta_box' ] );
        add_action( 'save_post', [ $this, 'prodigy_save_shipping_meta' ] );
        add_action( 'pre_post_update', [ $this, 'is_validate_shipping_data' ], 99, 2 );
    }

    /**
     *  Register meta-box
     */
    public function prodigy_shipping_meta_box() {
        add_meta_box(
            'prodigy-product-shipping',
            esc_html__( 'Shipping & Taxes', 'prodigy' ),
            [ $this, 'prodigy_show_shipping_meta_box'],
            Prodigy::get_prodigy_product_type(),
            'advanced',
            'low'
        );
    }

    /**
     * @param int $post_id
     */
    public function is_validate_shipping_data( $post_id ) {

        $error = false;

        if ( ! isset( $_POST['_inline_edit'] ) ) {
            if ( isset( $_POST['product_requires_shipping'] ) ) {

                foreach ( $_POST as $key => $item ) {
                    switch ( $key ) {
                        case 'product_weight_value':
                            if ( (int) $_POST['product_weight_value'] === 0  ) {
                                break;
                            }

                            if ( ! filter_var( (int) $_POST['product_weight_value'], FILTER_VALIDATE_FLOAT )) {
                                Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid dimension weight value!', 'prodigy' ) );
                                $error = true;
                                break;
                            }
                        case 'shipping_dimension_length':
                            if ( (int) $_POST['shipping_dimension_length'] === 0  ) {
                                break;
                            }
                            if ( ! filter_var( (int) $_POST['shipping_dimension_length'], FILTER_VALIDATE_FLOAT ) ) {
                                Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid dimension length value!',
                                    'prodigy' ) );
                                $error = true;
                                break;
                            }
                        case 'shipping_dimension_width':
                            if ( (int) $_POST['shipping_dimension_width'] === 0  ) {
                                break;
                            }
                            if ( ! filter_var( (int) $_POST['shipping_dimension_width'], FILTER_VALIDATE_FLOAT ) ) {
                                Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid dimension width value!',
                                    'prodigy' ) );
                                $error = true;
                                break;
                            }
                        case 'shipping_dimension_height':
                            if ( (int) $_POST['shipping_dimension_height'] === 0  ) {
                                break;
                            }
                            if ( ! filter_var( (int) $_POST['shipping_dimension_height'], FILTER_VALIDATE_FLOAT ) ) {
                                Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid dimension height value!',
                                    'prodigy' ) );
                                $error = true;
                            }
                    }
                }
            }
        }

        if ( $error ) {
            $url = admin_url( 'post.php?post=' . $post_id ) . '&action=edit';
            wp_redirect( $url );
            exit;
        }
    }

    /**
     * Render meta-box template
     *
     * @param $post
     */
    public function prodigy_show_shipping_meta_box( $post ) {

        wp_nonce_field( 'prodigy_show_shipping_meta_box', 'prodigy_shipping_nonce' );

        $product_weight_type = get_post_meta( $post->ID, 'product_weight_type', true );
        $product_weight_value = get_post_meta( $post->ID, 'product_weight_value', true );
        $shipping_dimension = get_post_meta( $post->ID, 'shipping_dimension', true );
        $shipping_dimension_length = get_post_meta( $post->ID, 'shipping_dimension_length', true );
        $shipping_dimension_width = get_post_meta( $post->ID, 'shipping_dimension_width', true );
        $shipping_dimension_height = get_post_meta( $post->ID, 'shipping_dimension_height', true );
        $taxable = get_post_meta( $post->ID, 'taxable', true );
        $product_requires_shipping = get_post_meta( $post->ID, 'product_requires_shipping', true );
        $tax_category_id = get_post_meta( $post->ID, 'tax_category_id', true );
        $tax_categories_list = json_decode( get_option('pg_tax_categories_list'), true );
        $shipping_dimensions = get_post_meta( $post->ID, 'shipping_dimensions', true );

        include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-shipping-meta-box.php';
    }

    /**
     * Save data for meta-box
     *
     * @param $post_id
     */
    public function prodigy_save_shipping_meta( $post_id ) {
        if ( !isset( $_POST['prodigy_shipping_nonce'] ) || !wp_verify_nonce( $_POST['prodigy_shipping_nonce'],'prodigy_show_shipping_meta_box') )
            return;

        if ( !current_user_can( 'edit_post', $post_id ))
            return;

        if ( isset( $_POST['product_weight_type']) ) {
            update_post_meta( $post_id, 'product_weight_type', sanitize_text_field( $_POST['product_weight_type'] ) );
        }

        if ( isset( $_POST['product_weight_value'] ) ) {
            update_post_meta( $post_id, 'product_weight_value', sanitize_text_field( $_POST['product_weight_value']
            ) );
        }

        if ( isset( $_POST['shipping_dimensions'] ) ) {
            update_post_meta( $post_id, 'shipping_dimensions', sanitize_text_field( $_POST['shipping_dimensions'] ) );
        }

        if ( isset( $_POST['shipping_dimension_length']) ) {
            update_post_meta( $post_id, 'shipping_dimension_length', sanitize_text_field( $_POST['shipping_dimension_length'] ) );
        }

        if ( isset( $_POST['shipping_dimension_width']) ) {
            update_post_meta( $post_id, 'shipping_dimension_width', sanitize_text_field( $_POST['shipping_dimension_width'] ) );
        }

        if ( isset( $_POST['shipping_dimension_height']) ) {
            update_post_meta( $post_id, 'shipping_dimension_height', sanitize_text_field( $_POST['shipping_dimension_height'] ) );
        }

        if ( isset( $_POST['tax_category_id']) ) {
            update_post_meta( $post_id, 'tax_category_id', sanitize_text_field( $_POST['tax_category_id'] ) );
        }


        if ( isset($_POST['taxable']) ) {
            update_post_meta($post_id, 'taxable', sanitize_text_field( $_POST['taxable']) );
        } else {
            delete_post_meta($post_id, 'taxable');
        }

        if ( isset($_POST['product_requires_shipping']) ) {
            update_post_meta($post_id, 'product_requires_shipping', sanitize_text_field( $_POST['product_requires_shipping']) );
        } else {
            delete_post_meta($post_id, 'product_requires_shipping');
        }
    }

}

new Prodigy_Product_Shipping_Box();
