<?php

/**
 * Prodigy admin product variants class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-product.php';

/**
 * Class Prodigy_Product_Variants_Box
 */
class Prodigy_Product_Variants_Box extends Prodigy_Product_Meta_Box {

    /**
     * Prodigy_Product_Variants_Box constructor.
     */
    public function __construct()
    {
        parent::__construct();
        add_action( 'add_meta_boxes', [ $this, 'prodigy_variants_meta_box' ] );
        add_action( 'save_post', [ $this, 'prodigy_save_variants_meta' ], 10, 1 );
        add_action( 'wp_ajax_create-variant-meta-box', [ $this, 'get_variants_ajax' ] );
        add_action( 'pre_post_update', [$this, 'prodigy_validate_variants'], 99, 2);
        add_action( 'wp_ajax_prodigy-delete-variant-image', [ $this, 'remove_variant_image' ] );
    }


    /**
     *  Remove image from variants when the same remove in gallery
     */
    public function remove_variant_image() {
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $variants = $this->product_variants->get_variants_from_product( $_POST['post_id'] );
        foreach ( $variants as $key => $variant ) {
            if ( $variant['category-image-id'] == $_POST['remove_image'] ) {
                update_post_meta( $variant['ID'], 'category-image-id', '' );
            }
        }

        echo true;

        wp_die();
    }


    /**
     * Register variant meta-box
     */
    public function prodigy_variants_meta_box() {
        add_meta_box(
            'prodigy-product-variants',
            esc_html__( 'Product Variants', 'prodigy' ),
            [ $this, 'prodigy_show_variants_meta_box' ],
            Prodigy::get_prodigy_product_type(),
            'advanced',
            'low'
        );
    }

    /**
     * Render variants meta-box
     *
     * @param object $post
     */
    public function prodigy_show_variants_meta_box( $post ) {
        wp_nonce_field( 'prodigy_show_variants_meta_box', 'prodigy_variants_nonce' );
        $is_use_option = get_post_meta( $post->ID, '_use_option', true );
        $variants = $this->product_variants->get_variants_from_product( $post->ID );

        $images = [];
        foreach ( $variants as $key => $variant ) {
            $thumbnail_id = $variant['category-image-id'];

            if ( !empty( $thumbnail_id ) ) {
                $image = wp_get_attachment_thumb_url( $thumbnail_id );
            } else {
                $image = plugin_dir_url( PRODIGY_PLUGIN_PATH . 'web/admin/images/placeholder.png' ) . 'placeholder.png';
            }

            $images[$key] = str_replace( ' ', '%20', $image );
        }

        include PRODIGY_PLUGIN_PATH . 'admin/partials/prodigy-product-variants-meta-box.php';
    }

    /**
     * Get enable variants
     *
     * @return string
     */
    public function get_variants_ajax() {
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $meta_attributes = get_post_meta( sanitize_text_field($_POST['post_id']), Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META, true );
        $post_attributes = maybe_unserialize( $meta_attributes );

        $enable_variants = [];
        if ( !empty($_POST['enable_variants']) ) {
            $enable_variants = array_diff( $_POST['enable_variants'], [ '' ] );
        }
        $variant_names = [];

        if ( !empty( $enable_variants ) ) {
            $current_variants = $this->product_variants->prepare_variants( $enable_variants, $post_attributes );

            $current_variant_names = [];
            foreach ( $current_variants as $key=>$variant_ids ) {
                $current_variant_names[$key] = Prodigy_Product_Attributes::get_value_by_ids( $variant_ids );
            }

            foreach ( $current_variant_names as $key=>$names ) {
                $i = 0;
                foreach ( $names as $name ) {
                    $variant_names[$key][$i++] = $name;
                }
            }
        }

        $result_variants = $this->product_variants->generate_variants( $variant_names );

        foreach ( $result_variants as $key => $variant ) {
            if ( !empty( $_POST['sku'] ) ) {
                $result_variants[$key]['sku'] = $this->product_variants->generate_sku( sanitize_text_field($_POST['sku']), $variant['name'] );
                $product_id = Prodigy_Product::get_product_from_sku($result_variants[$key]['sku']);
                if ( !empty($product_id) ) {
                    $result_variants[ $key ]['id'] = $product_id;
                }
            }
        }

        $current_product = array_merge(
            (array) get_post( sanitize_text_field($_POST['post_id']) ),
            (array) $this->product_variants->format_meta_array( get_post_meta( sanitize_text_field($_POST['post_id']) ) )
        );

        if ( !$result_variants ) {
            echo wp_json_encode([
                'error' => 'Can`t create variants',
                'status' => 0,
            ]);
        } else {
            echo wp_json_encode([
                'variants' => $result_variants,
                'current_product' => $current_product,
                'status' => !empty( $result_variants ),
            ]);
        }

        wp_die();
    }

    /**
     * @param int $post_id
     */
    public function prodigy_save_default_images( $post_id ) {
        $main_product_image = get_post_thumbnail_id( $post_id );
        $product_gallery_images = get_post_meta( $post_id, 'prodigy_gallery', true );

        // set default image for variants (in feature image)
        $attachment_ids = [];
        if ( isset( $_POST['variants']['category-image-id'] ) ) {
            foreach ( $_POST['variants']['category-image-id'] as $key => $variant_image ) {
                if ( empty( $variant_image ) ) {
                    $_POST['variants']['category-image-id'][ $key ] = $main_product_image;
                    $attachment_ids[$key] = $main_product_image;
                } else {
                    $attachment_ids[$key] = $variant_image;
                }
            }
        }

        // set images from variants to products gallery
        if ( !empty($attachment_ids) && empty($product_gallery_images) ) {
            $gallery_images_array = explode( ',', $product_gallery_images );
            $images               = !empty( $gallery_images_array[0] ) ? array_merge( $gallery_images_array, $attachment_ids ) : $attachment_ids;

            if ( in_array( $_POST['_thumbnail_id'], $images ) ) {
                $images = array_diff( $images, [$_POST['_thumbnail_id']] );
            }
            update_post_meta( $post_id, 'prodigy_gallery', implode( ",", array_unique( $images ) ) );
        }
    }

    /**
     * @param int $post_id
     *
     * @return bool
     */
    public function prodigy_set_default_variants_images( $post_id ) {
        $prodigy_gallery = get_post_meta( $post_id, 'prodigy_gallery', true );
        $prodigy_gallery = explode( ',', $prodigy_gallery );

        if ( empty( $prodigy_gallery ) ) {
            return true;
        }

        // set default image for variants
        $attachment_ids = [];
        if ( isset( $_POST['variants']['category-image-id'] ) ) {
            foreach ( $_POST['variants']['category-image-id'] as $key => $variant_image ) {
                if ( empty( $variant_image ) ) {
                    $_POST['variants']['category-image-id'][ $key ] = $prodigy_gallery[0];
                    $attachment_ids[$key] = $prodigy_gallery[0];
                } else {
                    $attachment_ids[$key] = $variant_image;
                }
            }
        }
    }

    /**
     * Save attributes data
     *
     * @param int $post_id
     */
    public function prodigy_save_variants_meta( $post_id ) {

        if ( !isset( $_POST['prodigy_variants_nonce'] ) || !wp_verify_nonce( $_POST['prodigy_variants_nonce'], 'prodigy_show_variants_meta_box' ) ) {
            return;
        }

        if ( !current_user_can( 'edit_post', $post_id ) ) {
            return;
        }

        $this->prodigy_save_default_images( $post_id );
        $this->prodigy_set_default_variants_images( $post_id );

        if ( isset( $_POST['variants'] ) ) {
            $current_product = get_post( $post_id );
            $current_variants = $this->product_variants->get_product_variants( $post_id );
            $post_variants = $this->update_post_variants( $_POST );

            if ( empty( $current_variants ) ) {
                $this->product_variants->save( $current_product, $post_variants );
            } else {
                $this->product_variants->update_variants( $current_product, $current_variants, $post_variants );
            }
        } else {
            $this->product_variants->delete_all_variants( $post_id );
        }
    }

    /**
     * @param int $post_id
     *
     * @return array
     */
    public function prodigy_validate_variants( $post_id ) {
        $error = false;

        $variants = [];

        if (isset($_POST['variants'])) {
            $variants = $_POST['variants'];
        }

        if ( ! isset( $_POST[ '_inline_edit' ] ) ) {
            if ( isset( $variants['post_title'] ) && ! empty( $variants['post_title'] ) ) {
                foreach ( $variants['post_title'] as $value ) {
                    if ( ! sanitize_text_field($value) || empty($value) ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You must enter valid title!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['post_attributes']) && ! empty( $variants['post_attributes'] ) ) {
                foreach ( $variants['post_attributes'] as $value ) {
                    if ( ! sanitize_text_field( $value ) || empty( $value ) ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You must enter valid attributes!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['category-image-id']) && ! empty( $variants['category-image-id'] ) ) {
                foreach ( $variants['category-image-id'] as $value ) {
                    if ( ! filter_var($value, FILTER_VALIDATE_INT ) && $value !== '' ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You must enter valid image id!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset( $variants['product_regular_price'] ) ) {
                foreach ( $variants['product_regular_price'] as $value ) {
                    if (
                        ! filter_var( $value, FILTER_VALIDATE_FLOAT ) && $value != 0

                    ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You must enter valid regular price!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['product_sale_price']) && ! empty( $variants['product_sale_price'] ) ) {
                foreach ( $variants['product_sale_price'] as $value ) {
                    if ( ! filter_var($value, FILTER_VALIDATE_FLOAT ) && $value != 0 ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You must enter valid sale price!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['product_from_date']) && ! empty( $variants['product_from_date'] ) ) {
                foreach ( $variants['product_from_date'] as $value ) {
                    if ( ! filter_var(
                            $value,
                            FILTER_VALIDATE_REGEXP,
                            ['options' => ['regexp' => '/([12]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[12]\d|3[01]))/']]) &&
                         $value !== ''
                    ) {
                        Prodigy_Admin_Notification::displayError(esc_html__('You must enter valid date!', 'prodigy'));
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['product_to_date']) && ! empty($variants['product_to_date'] ) ) {
                foreach ( $variants['product_to_date'] as $value ) {
                    if ( ! filter_var(
                            $value,
                            FILTER_VALIDATE_REGEXP,
                            ['options' => ['regexp' => '/([12]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[12]\d|3[01]))/']]) &&
                         $value !== ''
                    ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You must enter valid date!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['product_stock_status']) && ! empty( $variants['product_stock_status'] ) ) {
                foreach ( $variants['product_stock_status'] as $value ) {
                    if ( ! filter_var( $value, FILTER_VALIDATE_INT ) && $value !== '0' ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid stock status!', 'prodigy'));
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['product_stock_quantity']) && ! empty($variants['product_stock_quantity'] ) ) {
                foreach ( $variants['product_stock_quantity'] as $value ) {
                    if ( ! filter_var( $value, FILTER_VALIDATE_INT ) && $value !== '' ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid stock quantity!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['allow']) && !empty( $variants['allow'] ) ) {
                foreach ( $variants['allow'] as $value ) {
                    if ( ! filter_var( $value, FILTER_VALIDATE_INT ) && $value !== '0' ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid allow backorders!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['product_weight_type']) && ! empty( $variants['product_weight_type'] ) ) {
                foreach ( $variants['product_weight_type'] as $value ) {
                    if ( ! filter_var( $value, FILTER_VALIDATE_INT ) && $value !== '0' ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid weight type!', 'prodigy') );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset( $variants['product_weight_value'] ) ) {
                foreach ( $variants['product_weight_value'] as $value ) {

                    if ( (int) $value === 0  ) {
                        break;
                    }

                    if ( ! filter_var( (int) $value, FILTER_VALIDATE_INT ) ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid weight value!', 'prodigy'));
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['dimension']) && ! empty($variants['dimension'] ) ) {
                foreach ( $variants['dimension'] as $value ) {
                    if ( (int) $value === 0  ) {
                        break;
                    }
                    if ( ! filter_var( $value, FILTER_VALIDATE_INT ) && $value !== '0' ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid dimension value!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset($variants['shipping_dimension_length'] ) ) {
                foreach ( $variants['shipping_dimension_length'] as $value ) {
                    if ( (int) $value === 0  ) {
                        break;
                    }
                    if ( ! filter_var( (int) $value, FILTER_VALIDATE_INT ) ) {
                        Prodigy_Admin_Notification::displayError( esc_html__('You need enter valid dimension length value!', 'prodigy'));
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset( $variants['shipping_dimension_width'] ) ) {
                foreach ( $variants['shipping_dimension_width'] as $value ) {
                    if ( (int) $value === 0  ) {
                        break;
                    }
                    if ( ! filter_var( (int) $value, FILTER_VALIDATE_INT ) ) {
                        Prodigy_Admin_Notification::displayError( esc_html__( 'You need enter valid dimension width value!', 'prodigy' ) );
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset( $variants['shipping_dimension_height'] ) ) {
                foreach ( $variants['shipping_dimension_height'] as $value ) {
                    if ( (int) $value === 0  ) {
                        break;
                    }
                    if ( ! filter_var( (int) $value, FILTER_VALIDATE_INT ) ) {
                        Prodigy_Admin_Notification::displayError(esc_html__('You need enter valid dimension height value!', 'prodigy'));
                        $error = true;
                        break;
                    }
                }
            }

            if ( isset( $variants['product_sku'] ) ) {
                foreach ( $variants['product_sku'] as $key => $value ) {
                    if ( $value === ''  ) {
                        Prodigy_Admin_Notification::displayError(esc_html__('Product SKU is empty', 'prodigy'));
                        $error = true;
                        break;
                    } else {
                        $is_valid = $this->product->is_check_unique_sku($variants['post_id'][$key], $value);

                        if ($is_valid) {
                            Prodigy_Admin_Notification::displayError( esc_html__( 'SKU ' .$value. ' is not unique, please change to another' ) );
                            $error = true;
                            break;
                        }
                    }
                }
            }

        }


        if ( $error ) {
            $url = admin_url( 'post.php?post=' . $post_id ) . '&action=edit';
            wp_redirect( $url );
            exit;
        }

        return $_POST;
    }

    /**
     * Copy shipping option to variants
     *
     * @param array $post
     *
     * @return array
     */
    public function update_post_variants( $post ) {
        if (isset($post['product_requires_shipping'])) {
            $count_variants = sanitize_text_field(count($post['variants']['post_title']));

            for ($i = 0; $i <= $count_variants - 1; $i++) {
                $post['variants']['check_requires_shipping'][$i] = $post['product_requires_shipping'];
            }
        }

        return $post['variants'];
    }
}

new Prodigy_Product_Variants_Box();
