<?php

/**
 * Prodigy admin product listener class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

require_once PRODIGY_PLUGIN_PATH . 'includes/wp-queue/wp-async-request.php';
require_once PRODIGY_PLUGIN_PATH . 'includes/wp-queue/wp-background-process.php';
include_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-aggregate-product-data-synchronize.php';

/**
 * Class Prodigy_Product_Listener
 */
class Prodigy_Product_Bulk_Listener extends WP_Background_Process {


    protected $action = 'bulk_edit_process';

    protected $prodigy_synchronize;
    protected $api_client;

    /**
     * @param mixed $id
     *
     * @return bool|mixed
     */
    protected function task( $id ) {
        $this->prodigy_synchronize = new Prodigy_Aggregate_Data_Synchronize();
        $this->api_client          = new Prodigy_Api_Client();
        $show_product_response     = [];
        $exist_remote_data         = $this->prodigy_synchronize->get_remote_product_data( $id );
        if ( isset( $exist_remote_data ) && $exist_remote_data['remote_product_id'] != null ) {
            $show_product_response = $this->api_client->show_product( $exist_remote_data['remote_product_id'] );
        }

        if ( ! empty( $show_product_response ) && $show_product_response['response']['code'] == '200' ) {
            $form_params = $this->prodigy_synchronize->prepare_request_for_retry_product( $id );
            $response    = $this->api_client->update_product( $exist_remote_data['remote_product_id'], json_encode($form_params) );

            $data_response = json_decode( wp_remote_retrieve_body( $response ), 1 );

            if ( ! empty( $data_response ) && isset( $data_response['errors'] ) ) {
                update_post_meta(
                    $id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );

                return false;
            }

            if ( $response['response']['code'] == '200' ) {
                $this->prodigy_synchronize->update_product_status( $id, 'publish' );
                update_post_meta(
                    $id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::SUCCESS_STATUS
                );
            } else {
                $this->prodigy_synchronize->update_product_status( $id, 'unsynchronized' );
                update_post_meta(
                    $id,
                    Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                    Prodigy_Product_Synchronize::FAILED_STATUS
                );
            }
        } else {
            $this->prodigy_synchronize->update_product_status( $id, 'unsynchronized' );
            update_post_meta(
                $id,
                Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                Prodigy_Product_Synchronize::FAILED_STATUS
            );
        }

        return false;
    }

    public function complete() {
        parent::complete();
        header( "Location: " . esc_url($_SERVER['HTTP_REFERER']) );
    }
}