<?php

/**
 * Prodigy admin product listener class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

require_once PRODIGY_PLUGIN_PATH . 'includes/wp-queue/wp-async-request.php';
require_once PRODIGY_PLUGIN_PATH . 'includes/wp-queue/wp-background-process.php';
require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-batch-products-synchronize.php';

/**
 * Class Prodigy_Product_Listener
 */
class Prodigy_Batch_Products_Listener extends WP_Background_Process {

    protected $action = 'send_batch_products_process';

    private $batch_content;
    private $api_client;
    private $batch_products;

    /**
     * @param mixed $id
     *
     * @return bool|mixed
     */
    protected function task( $id ) {
        $this->batch_content  = new Prodigy_Send_Batch_Products();
        $this->api_client     = new Prodigy_Api_Client();
        $this->batch_products = new Prodigy_Batch_Products_Synchronize();

        if ( isset( $id ) ) {
            $products       = $id;
            $batch_products = $this->batch_content->set_product_objects( $products );
            $request_data   = $this->batch_products->prepare_batch_products( $batch_products );
            $response       = $this->api_client->create_batch_products( $request_data );

            if ( ! isset( $response['errors'] ) ) {
                $response = json_decode( $response['body'], true );
                $batch_id = (int) $response['data']['id'];
            } else {
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger', $response['errors'], 'create batch products error' );
                }
            }

            if ( isset( $batch_id ) ) {
                $this->batch_products->update_sending_batch_products( $products, $batch_id );
            }
        }

        return  false;
    }

    public function complete() {
        parent::complete();

        add_option( 'show_batch_errors', true );
    }
}