<?php

/**
 * Prodigy admin product listener class
 *
 * @version 1.0.0
 * @package prodigy/admin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

require_once PRODIGY_PLUGIN_PATH . 'includes/wp-queue/wp-async-request.php';
require_once PRODIGY_PLUGIN_PATH . 'includes/wp-queue/wp-background-process.php';
require_once PRODIGY_PLUGIN_PATH . 'includes/class-prodigy-batch-products-synchronize.php';

/**
 * Class Prodigy_Product_Listener
 */
class Prodigy_Batch_Status_Listener extends WP_Background_Process {

    protected $action = 'send_batch_status_process';

    private $batch_content;
    private $api_client;
    private $batch_products;


    /**
     * @param mixed $local_batch_id
     *
     * @return bool|mixed
     */
    protected function task( $local_batch_id ) {
        $this->batch_content = new Prodigy_Send_Batch_Products();
        $this->api_client = new Prodigy_Api_Client();
        $this->batch_products = new Prodigy_Batch_Products_Synchronize();

        $batch_id = $this->get_remote_batch_id($local_batch_id);

        $need_to_restart = false;
        if (!empty($batch_id)) {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', 'execute task with local id ' . $local_batch_id . ' - ' . $batch_id,
                    'check batch status' );
            }
            $is_update_status = $this->batch_content->set_completed_status( $batch_id );

            if ( $is_update_status ) {
                $results_query = $this->api_client->show_batch_products_result( $batch_id );
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger', $results_query, 'show_batch_products_result' );
                }
                if ( ! isset( $results_query['errors'] ) ) {
                    $result_products = json_decode( $results_query['body'], true );
                    $this->batch_products->update_products_data( $result_products );
                } else {
                    if ( PRODIGY_DEBUG_MODE ) {
                        do_action( 'logger', $results_query['errors'], 'show batch products result error' );
                    }
                }
            } else {
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger',
                        'restart task with local id (not completed) ' . $local_batch_id . ' - ' . $batch_id,
                        'check batch status' );
                }
                $need_to_restart = true;
            }
        } else {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', 'remote batch id not found for local id ' . $local_batch_id,
                    'check batch status' );
            }
            $need_to_restart = true;
        }

        if ($need_to_restart) {
            return $local_batch_id;
        } else {
            return false;
        }
    }


    /**
     * @param int $batch_id
     *
     * @return mixed
     */
    public function get_remote_batch_id($batch_id) {
        global $wpdb;

        $result_terms = $wpdb->get_row(
            $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}prodigy_batches_sent_out_products WHERE local_batch_id = %s",
                [ $batch_id ]
            )
        );

        return $result_terms->remote_batch_id;
    }

    public function complete() {
        parent::complete();
    }
}