<?php

/**
 * Prodigy Batches Categories class
 *
 * @version    1.0.5
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Batches_Categories
 */
class Prodigy_Batches_Categories {
    
    /**
     * @var $prodigy_api_client
     */
    private $prodigy_api_client;
    
    /**
     * Prodigy_Batches_Categories constructor.
     *
     * @param Prodigy_Api_Client $prodigy_api_client
     */
    public function __construct( Prodigy_Api_Client $prodigy_api_client ) {
        $this->prodigy_api_client = $prodigy_api_client;
    }
    
    /**
     * Create categories
     */
    public function send_categories() {
        
        $categories = $this->get_prodigy_categories();
        $result     = $this->prodigy_api_client->create_batches_category( $categories );
        
        if ( array_key_exists( 'error', $result ) ) {
            wp_die( $result['message'] );
        }
        
        $status_code = wp_remote_retrieve_response_code( $result );
        
        if ( $status_code != 200 ) {
            $errors  = $this->prodigy_api_client->get_list_errors( $result );
            $message = wp_remote_retrieve_response_message( $result ) . '. ' . implode( ', ', $errors );
            
            $log_response = [
                'message' => $message,
                'result'  => $result
            ];
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $log_response, 'error' );
            }
        } else {
            
            $result_body = json_decode( wp_remote_retrieve_body( $result ), true );
            $this->processing_sync_categories( $result_body );
        }
    }
    
    /**
     * @param array $data
     */
    protected function processing_sync_categories( array $data ) {
        
        if ( ! empty( $data['data'] ) && is_array( $data['data'] ) ) {
            foreach ( $data['data'] as $key => $value ) {
                if ( ! empty( $value['attributes']['row-id'] ) && $value['attributes']['success'] == 'true' ) {
                    $this->set_relationship_categories(
                        esc_attr( $value['attributes']['row-id'] ),
                        esc_attr( $value['attributes']['data']['id'] )
                    );
                } else {
                    $log_response = [
                        'message' => 'Error updated categories',
                        'value'   => $value
                    ];
                    if ( PRODIGY_DEBUG_MODE ) {
                        do_action( 'logger', $log_response, 'error' );
                    }
                }
            }
        }
    }
    
    /**
     * @param int $term_id
     * @param int $remote_category_id
     */
    protected function set_relationship_categories( $term_id, $remote_category_id ) {
        update_term_meta(
            $term_id,
            Prodigy_Admin_Category::PRODIGY_HOSTED_CATEGORY_RELATION,
            $remote_category_id,
            true
        );
    }
    
    /**
     * @return array|int|WP_Error
     */
    protected function get_prodigy_categories() {
        $prepare_terms = [];
        $terms         = get_terms( [
            'taxonomy'   => Prodigy::get_prodigy_category_type(),
            'hide_empty' => false,
        ] );
        
        if ( ! empty( $terms ) ) {
            
            foreach ( $terms as $key => $value ) {
                $prepare_terms[ $key ]['row_id'] = $value->term_id;
                $prepare_terms[ $key ]['name']   = $value->slug;
                if ( ! empty( $value->parent ) ) {
                    $prepare_terms[ $key ]['parent_id'] = $value->parent;
                }
            }
        }
        
        return $prepare_terms;
    }
}
