<?php

/**
* Prodigy send batch products
*
* @version 1.1.0
* @package prodigy/admin
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}


require_once PRODIGY_PLUGIN_PATH . 'includes/batches-content-synchronization/class-prodigy-batch-products-listener.php';
require_once PRODIGY_PLUGIN_PATH . 'includes/batches-content-synchronization/class-prodigy-batch-status-listener.php';


/**
 * Class Prodigy_Send_Batch_Content
 */
class Prodigy_Send_Batch_Products {
    private $batch_products;
    private $batch_process;
    private $api_client;
    private $product_process;

    /**
     * Prodigy_Send_Batch_Content constructor.
     */
    public function __construct() {
        $this->batch_products = new Prodigy_Batch_Products_Synchronize();
        $this->api_client = new Prodigy_Api_Client();
        $this->product_process = new Prodigy_Batch_Products_Listener();
        $this->batch_process = new Prodigy_Batch_Status_Listener();
    }

    /**
     *  Send batch products action
     */
    public function prodigy_send_batch_products() {
        $this->add_batch_to_queue();
        $this->start_queues();
    }


    /**
     *  Start products queues
     */
    public function start_queues() {
        $is_product_process = $this->check_queue_key('send_batch_products_process_batch');
        $is_status_process = $this->check_queue_key('send_batch_status_process_batch');
        if ( empty($is_product_process) ) {
            $this->product_process->save()->dispatch();
        }

        if ( empty($is_status_process) ) {
            $this->batch_process->save()->dispatch();
        }
    }


    /**
     *  Add products to queue
     */
    public function add_batch_to_queue() {
        $batch_size = 25;
        $page = 1;
        do {
            $offset = ( $page - 1 ) * $batch_size;
            $query_args = array(
                'post_type'  =>  Prodigy::get_prodigy_product_type(),
                'orderby'    =>  'date',
                'order'      =>  'desc',
                'posts_per_page'     =>  $batch_size,
                'offset'     =>  $offset,
                'post_status' => [ 'unsynchronized' ]
            );

            $query = new WP_Query ( $query_args );

            $product_ids = [];
            $count_posts = count($query->posts);
            foreach ( $query->posts as $key => $post ) {
                if (!$this->batch_products->is_sended_product( $post->ID )) {
                    $product_ids[ $key ] = $post->ID;
                }
            }

            if (count($product_ids) > 0) {
                $uuid = generate_uuid();
                $this->batch_products->insert_sending_batch_products( $uuid, $product_ids );
                $this->product_process->push_to_queue( $product_ids );
                $this->batch_process->push_to_queue( $uuid );
            }
            $page++;
        } while ($count_posts > 0);
    }


    /**
     * @param string $process_name
     *
     * @return mixed
     */
    public function check_queue_key( $process_name ) {
        global $wpdb;

        $result = $wpdb->get_results(
            "SELECT option_name
                   FROM $wpdb->options
                   WHERE option_name 
                   LIKE '%{$process_name}%';"
        );

        return $result;
    }


    /**
     * @param array $product_ids
     *
     * @return array
     */
    public function set_product_objects($product_ids) {
        $prepared_products = [];
        foreach ( $product_ids  as $products_id ) {
            $prepared_products[$products_id] = get_prodigy_product($products_id);
        }

        return $prepared_products;
    }

    /**
     * @param int $batch_id
     *
     * @return bool
     */
    public function set_completed_status( $batch_id ) {
        $completed_query = $this->api_client->show_batch_products( $batch_id );
        if ( PRODIGY_DEBUG_MODE ) {
            do_action( 'logger', $completed_query, 'completed status response' );
        }
        if ( ! isset( $completed_query['errors'] ) ) {
            $completed_batches = json_decode( $completed_query['body'], true );
            if ( $completed_batches['data']['attributes']['total'] == $completed_batches['data']['attributes']['completed'] ) {
                return true;
            } else {
                return false;

            }
        } else {
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $completed_query['errors'], 'show batch products error' );
            }
            return false;
        }
    }

}

return new Prodigy_Send_Batch_Products();