<?php

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Activator
 */
class Prodigy_Activator {

    // prodigy admin notification list
    public static $notices = [
        'check_mysql_notice'
    ];
    
    /**
     * Short Description. (use period)
     *
     * Long Description.
     *
     * @since    1.0.0
     */
    public static function activate() {
        $tax_categories_list = include PRODIGY_PLUGIN_PATH . 'includes/settings/tax-categories-list.php';


        
        // set default settings
        update_option( 'pg_tax_categories_list', $tax_categories_list );
        update_option( 'pg_currency_type', '$' );
        update_option( 'pg_currency_position', 'left' );
        update_option( 'pg_thousands_separator', ', ' );
        update_option( 'pg_decimal_separator', '.' );
        update_option( 'pg_number_decimals', 2 );
        update_option( 'pg_low_stock_threshold', 2 );
        update_option( 'pg_out_stock_threshold', 0 );
        update_option( 'pg_hold_stock_time', 1 );
        update_option( 'pg_add_cart_behaviour', 'redirect_to_cart' );
        update_option( 'pg_dimensions', 'in' );
        update_option( 'pg_weight', 'lbs' );
        update_option( 'pg_stock_format', 1 );
        update_option( 'pg_cart_expiration_time', 168 );
        update_option( 'pg_custom_expiration_time', 0 );
        update_option( 'pg_add_cart_behaviour', 'current_page' );
        update_option( 'pg_all_slugs_product_type', ['prodigy_product_type', 'product'] );
        update_option( 'pg_all_slugs_category_type', ['prodigy_product_cat', 'product-category'] );
        update_option( 'pg_all_slugs_tag_type', ['prodigy_product_tag', 'product-tag'] );
        update_option( 'pg_indicator_sync_content', 'yes' );
        update_option( 'pg_set_pages', false );
        
        // disable requires fields in comment form
        update_option( 'require_name_email', 0 );
        update_option( 'show_comments_cookies_opt_in', 0 );

        foreach ( self::$notices as $notice ) {
            update_option( 'prodigy_admin_notice_' . $notice, 0 );
        }

        self::set_roles();
    }
    
    public static function set_roles() {
        global $wp_roles;
        
        if ( ! class_exists( 'WP_Roles' ) ) {
            return;
        }
        
        if ( ! isset( $wp_roles ) ) {
            $wp_roles = new WP_Roles();
        }
        
        add_role(
            'customer',
            'Customer',
            [
                'read' => true,
            ]
        );
    }
    
}
