<?php

/**
 * Setup Wizard Class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Authentication
 */
if ( ! is_user_logged_in() ) {
    auth_redirect();
    exit();
}

/**
 * Prodigy_Admin_Setup_Wizard class.
 */
class Prodigy_Admin_Setup_Wizard {
    
    const LENGTH_CHARACTERS_TOKEN = 30;
    const NAME_TOKEN_HOSTED_SYSTEM = 'pg_token_hosted_system';
    const SYNC_CONTENT_TRUE = 'yes';
    const SYNC_CONTENT_FALSE = 'no';
    
    /**
     * Current step
     *
     * @var int
     */
    private $step = 1;
    
    /**
     * @var int
     *
     */
    private $rstep;
    
    /**
     * Connected to hosted system
     *
     * @var bool
     */
    private $is_connected = false;
    
    /**
     * Steps for the setup wizard
     *
     * @var array
     */
    private $steps = [
        1 => 'connect_wizard_content',
        2 => 'setup_settings'
    ];
    
    /**
     * @var array
     */
    private $redirect_steps = [
        1 => 'save_setup_settings',
        2 => 'save_finish_wizard',
        3 => 'save_steps_wizard'
    ];
    
    /**
     * @var array
     */
    private $page_header_steps = [
        1 => 'Connection',
        2 => 'Store Locale Setup'
    ];
    
    /**
     * @var array
     */
    private $page_header_content = [
        1 => 'Data Synchronization',
        2 => 'Store Locale Setup'
    ];
    
    /**
     * @var array
     */
    private $weights = [
        'lbs',
        'kg',
        'g',
        'oz',
    ];
    
    /**
     * @var array
     */
    private $dimensions = [
        'cm',
        'in',
    ];
    
    /**
     * @var void
     */
    private $url_domain_hosted_system;
    
    /**
     * @var void
     */
    private $domain_hosted_system;
    
    /**
     * @var bool
     */
    private $is_created_products;
    
    /**
     * @var bool
     */
    private $is_created_categories;
    
    /**
     * @var int
     */
    private $saved_step;
    
    /**
     * @var array
     */
    private $ignore_statuses_product = [ 'auto-draft', 'draft' ];
    
    /**
     * Hook in tabs.
     */
    public function __construct() {
        
        /*
         * Get token from hosted system
         */
        if ( ! empty( $_GET['token'] ) ) {
            $this->create_connection();
        }
        
        /*
         * Clear data before to connect to hosted system
         */
        if ( ! empty( $_GET['clear'] ) ) {
            $this->clear_data_before_connect();
        }
    
        if ( ! empty($_GET['refresh_permalinks'])) {
            $this->refresh_permalink();
        }
    
        if ( ! empty( $_GET['redirect_hosted_system'] ) ) {
            $this->redirect_to_hosted_system();
        }
        
        $this->is_connected             = get_option( 'pg_store_key' );
        $this->saved_step               = get_option( 'pg_step_wizard' );
        $this->domain_hosted_system     = get_option( 'pg_domain_hosted_system', 'Store name domain hosted' );
        $this->url_domain_hosted_system = get_option( 'pg_url_domain_hosted_system', 'Store url domain hosted' );
        $this->is_created_products      = $this->is_created_products();
        $this->is_created_categories    = $this->is_created_categories();
        $this->set_step();

        add_action( 'admin_menu', array( $this, 'admin_menus' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'wizard_enqueue_scripts' ) );
        add_action( 'admin_init', array( $this, 'setup_wizard' ) );
    }
    
    /**
     * Add admin menus/screens.
     */
    public function admin_menus() {
        add_dashboard_page( '', '', 'manage_options', 'prodigy-setup', '' );
    }
    
    /**
     *
     */
    private function refresh_permalink() {
        Prodigy::refresh_permalink();
        wp_safe_redirect(admin_url('edit.php?post_type='.Prodigy::get_prodigy_product_type().'&page=prodigy_settings'));
        exit;
    }
    
    /**
     * New connection for wizard
     */
    private function create_connection() {
        global $wp_session;
        
        $token            = filter_var( $_GET['token'], FILTER_SANITIZE_STRIPPED );
        $pg_store_key_old = get_option( 'pg_store_key' );
        
        if ( ! empty( $pg_store_key_old ) && empty( get_option( Prodigy::PRODIGY_FIRST_CONNECTED_PRODUCT ) ) ) {
            $api_client = new Prodigy_Api_Client();
            $info_cat   = get_term_by( 'slug', 'defaultcategory', Prodigy::get_prodigy_category_type() );
            
            if ( ! empty( $info_cat->term_id ) ) {
                $hosted_category_id = get_term_meta( $info_cat->term_id, 'prodigy_hosted_category_relation', true );
                if ( ! empty( $hosted_category_id ) ) {
                    $api_client->delete_category( $hosted_category_id );
                }
            }
            
            $api_client->disconnection( $pg_store_key_old );
            delete_option( 'pg_image_logo' );
        }
    
        update_option( 'pg_store_key', $token, false );
        $api_client_new     = new Prodigy_Api_Client();
        $request_connection = $api_client_new->setup_connection( $token, UrlHelper::getSiteUrl( site_url() ) );
        
        if ( array_key_exists( 'error', $request_connection ) ) {
            wp_send_json_error( $request_connection['message'], $request_connection['code'] );
        }
        
        $response_body = json_decode( wp_remote_retrieve_body( $request_connection ) );
        $status_code   = wp_remote_retrieve_response_code( $request_connection );
        
        if ( $status_code == 200 ) {
            
            update_option( 'pg_domain_hosted_system', $response_body->data->attributes->name );
            update_option( 'pg_url_domain_hosted_system', $response_body->data->attributes->subdomain );
            update_option( 'pg_step_wizard', 1 );
    
            $sync_content = get_option( 'pg_indicator_sync_content' );
    
            if ( $sync_content == self::SYNC_CONTENT_TRUE ) {
                $update_categories_obj = new Prodigy_Batches_Categories($api_client_new);
                $update_products_obj = new Prodigy_Send_Batch_Products();
                
                $update_obj = new Prodigy_Synchronize_Data($update_categories_obj, $update_products_obj);
                $update_obj->synchronize();
            }

        } else {
            update_option( 'pg_store_key', false );
            if ( PRODIGY_DEBUG_MODE ) {
                do_action( 'logger', $response_body, 'error' );
            }
            $wp_session['message_error_connection'] = ucfirst( (string) $response_body->errors[0]->detail );
        }
    }
    
    /**
     * check exist categories
     * @return bool
     */
    public function is_created_categories() {
        
        $count_cats = wp_count_terms( Prodigy::get_prodigy_category_type() );
        
        /*
         * ignore default category
         */
        if ( $count_cats == 1 ) {
            $count_cats = 0;
        }
        
        return (bool) $count_cats;
    }
    
    /**
     * check exist products
     * @return bool
     */
    public function is_created_products() {
        $count = 0;
        
        $count_posts = wp_count_posts( Prodigy::get_prodigy_product_type() );
        
        if ( ! $count_posts ) {
            return (bool) $count;
        }
        
        foreach ( $count_posts as $key => $val ) {
            if ( in_array( $key, $this->ignore_statuses_product ) ) {
                continue;
            }
            
            if ( $val > 0 ) {
                $count = $val;
                break;
            }
        }
        
        return (bool) $count;
    }
    
    /**
     * clear products
     * @return bool
     */
    public function delete_products() {
        
        $loop = new WP_Query( [ 'post_type' => Prodigy::get_prodigy_product_type(), 'post_status' => 'any' ] );
        
        if ( $loop->have_posts() ) {
            while ( $loop->have_posts() ): $loop->the_post();
                
                if ( ! empty( get_the_ID() ) ) {
                    wp_delete_post( get_the_ID(), true );
                }
            
            endwhile;
        }
        
        return true;
    }
    
    /**
     * clear categories
     *
     * @param bool $default_category
     */
    public function delete_categories( $default_category = false ) {
        $terms = get_terms( [
            'taxonomy'   => Prodigy::get_prodigy_category_type(),
            'hide_empty' => false,
        ] );
        
        if ( ! empty( $terms ) ) {
            foreach ( $terms as $term ) {
                if ( empty( $default_category ) && $term->name == Prodigy::PRODIGY_DEFAULT_NAME_CATEGORY ) {
                    continue;
                }
                wp_delete_term( $term->term_id, $term->taxonomy );
            }
        }
    }
    
    /**
     * clear attributes
     */
    public function delete_attributes() {
        global $wpdb;
        $obj_attr = Prodigy_Product_Attributes::get_attribute_taxonomies_aggregation();
        
        if ( is_array( $obj_attr ) ) {
            foreach ( $obj_attr as $attr ) {
                wp_delete_term( $attr->term_id, $attr->slug );
            }
        }
        $wpdb->query( "SET FOREIGN_KEY_CHECKS=0" );
        Prodigy_Product_Attributes::truncate_product_attrs();
        Prodigy_Product_Attributes::truncate_attrs_aggregation();
        Prodigy_Product_Attributes::truncate_attrs_taxonomy();
        $wpdb->query( "SET FOREIGN_KEY_CHECKS=1" );
    }
    
    /**
     * get url for wizard
     * @return string|void
     */
    public function get_url_clear_data() {
        if ( get_option( 'pg_story_key' ) ) {
            return admin_url( 'index.php?page=prodigy-setup' );
        }
        
        return admin_url( 'index.php?page=prodigy-setup&clear=true' );
    }
    
    /**
     * clear products, categories and attributes and redirect to dashboard
     * @return string|void
     */
    public function clear_data_before_connect() {
        
        if ( get_option( 'pg_story_key' ) ) {
            return admin_url( 'index.php?page=prodigy-setup' );
        }
        
        $this->clear_pg_content();
        
        return admin_url( 'index.php?page=prodigy-setup' );
    }
    
    /**
     * clear products, categories and attributes
     * @return string|void
     */
    public function clear_pg_content() {
        
        if ( get_option( 'pg_story_key' ) ) {
            return admin_url( 'index.php?page=prodigy-setup' );
        }
        
        $this->delete_products();
        $this->delete_categories();
        $this->delete_attributes();
    }
    
    /**
     * Show the setup wizard.
     */
    public function setup_wizard() {
        if ( empty( $_GET['page'] ) || 'prodigy-setup' !== $_GET['page'] ) {
            return;
        }
        
        $step_template  = $this->get_template_content_wizard();

        ob_start();
        $this->setup_wizard_header();
        $this->$step_template();
        $this->setup_wizard_footer();
        
        exit;
    }
    
    /**
     * clear step
     */
    public function clear_steps() {
        delete_option( 'pg_step_wizard' );
    }
    
    /**
     * set step for wizard
     */
    public function set_step() {
        
        if ( ! empty( $_GET['step'] ) && key_exists( $_GET['step'], $this->steps ) && $this->is_connected ) {
            $this->step = sanitize_text_field($_GET['step']);
        }
    }
    
    /**
     * @return string
     */
    public function get_url_login_hosted_system() {
        return admin_url( 'index.php?page=prodigy-setup&redirect_hosted_system=1' );
    }
    
    public function redirect_to_hosted_system() {
    
        $sync_content = get_option( 'pg_indicator_sync_content' );
    
        if ( $sync_content == self::SYNC_CONTENT_FALSE ) {
            $this->clear_pg_content();
        }
        
        $this->set_token_for_hosted_system();
        $params = array(
            'redirect_url' => admin_url( 'index.php?page=prodigy-setup' ),
        );
        
        $url = PRODIGY_PROTOCOL_DOMAIN . PRODIGY_MAIN_DOMAIN . '/plugin_connection_stores?' . http_build_query( $params );
    
        wp_redirect( $url );
        
        exit;
    }
    
    /**
     * @return bool
     */
    public function set_token_for_hosted_system() {
        return update_option( self::NAME_TOKEN_HOSTED_SYSTEM, substr( sha1( rand() ), 0, self::LENGTH_CHARACTERS_TOKEN ), false );
    }
    
    /**
     * @return mixed|void
     */
    public function get_token_for_hosted_system() {
        return get_option( self::NAME_TOKEN_HOSTED_SYSTEM );
    }
    
    /**
     * @return string
     */
    public function get_template_content_wizard(): string {
        
        $step = $this->steps[1];
        
        if ( $this->is_connected ) {
            $step = $this->steps[ $this->step ];
        }
        
        return $step;
    }
    
    
    /**
     * @return string/bool
     */
    public function get_rstep_wizard() {
        
        if ( $this->is_connected ) {
            $this->rstep = $this->redirect_steps[ $this->rstep ];
        }
        
        return $this->rstep;
    }
    
    
    /**
     * Setup Wizard Header.
     */
    public function setup_wizard_header() {
        set_current_screen();
        require_once PRODIGY_PLUGIN_PATH . 'admin/partials/wizard/prodigy-wizard-header-page.php';
    }
    
    /**
     * Setup Wizard Footer.
     */
    public function setup_wizard_footer() {
        
        require_once PRODIGY_PLUGIN_PATH . 'admin/partials/wizard/prodigy-wizard-footer-page.php';
    }
    
    
    /**
     * Register/enqueue scripts and styles for the Setup Wizard.
     *
     * Hooked onto 'admin_enqueue_scripts'.
     */
    public function wizard_enqueue_scripts() {
        
        if ( ! did_action( 'wp_enqueue_media' ) ) {
            wp_enqueue_media();
        }
        
        wp_enqueue_script( 'wizard', plugin_dir_url( dirname( __FILE__ ) ) . 'admin/partials/wizard/assets/js/wizard.js', [ 'jquery' ], false, true );
        
        wp_enqueue_style( 'wizard', plugin_dir_url( dirname( __FILE__ ) ) . 'admin/partials/wizard/assets/css/wizard.css', [], false );
    }
    
    /**
     * STEP 1
     * Output the content for the current step.
     */
    public function connect_wizard_content() {
        update_option( 'pg_step_wizard', 1 );
        require_once PRODIGY_PLUGIN_PATH . 'admin/partials/wizard/prodigy-wizard-step1-page.php';
    }
    
    /**
     * STEP 2
     * Output the content for the current step.
     */
    public function setup_settings() {
        update_option( 'pg_step_wizard', 2 );
        require_once PRODIGY_PLUGIN_PATH . 'admin/partials/wizard/prodigy-wizard-step2-page.php';
    }
    
    /**
     * safe redirect
     */
    public function save_setup_settings() {
        
        $this->save_settings();
        wp_safe_redirect( admin_url() . 'index.php?page=prodigy-setup&step=3' );
    }
    
    /**
     * Save settings on wizard
     */
    private function save_settings() {
        $weights    = ( in_array( $_POST['weights'], $this->weights ) ) ? sanitize_text_field($_POST['weights']) : '';
        $dimensions = ( in_array( $_POST['dimensions'], $this->dimensions ) ) ? sanitize_text_field($_POST['dimensions']) : '';
        $logo       = ( $image_logo = filter_var( $_POST['image_logo'], FILTER_VALIDATE_INT ) ) ? $image_logo : '';
        
        if ( ! empty( $weights ) ) {
            update_option( 'pg_weight', $weights );
        }
        if ( ! empty( $dimensions ) ) {
            update_option( 'pg_dimensions', $dimensions );
        }
        if ( ! empty( $logo ) ) {
            update_option( 'pg_image_logo', $logo );
        }
    }
    
    /**
     * STEP 3
     * Output the content for the current step.
     */
    public function upload_logo() {
        update_option( 'pg_step_wizard', 3 );
        require_once PRODIGY_PLUGIN_PATH . 'admin/partials/wizard/prodigy-wizard-step3-page.php';
    }
    
    /**
     * save setting by click finish
     */
    public function save_finish_wizard() {
        update_option( 'pg_image_logo', filter_var( $_POST['image_logo'], FILTER_VALIDATE_INT ) );
        Prodigy::refresh_permalink();
        wp_safe_redirect( admin_url() );
    }
    
    /**
     * save step by click button "back to dashboard
     */
    public function save_steps_wizard() {
        update_option( 'pg_step_wizard', $this->step );
        $this->save_settings();
        wp_safe_redirect( admin_url() );
    }
}

new Prodigy_Admin_Setup_Wizard();
