<?php

/**
 * Aggregate data class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_Synchronize
 */
class Prodigy_Aggregate_Data_Synchronize {

    const RETRY_SYNCHRONIZE_ACTION = 'retry-synchronize';
    const PRODUCT_STATUS_SYNCHRONIZE = 'product_status_synchronize';
    const REMOTE_RELATIONSHIP_META = 'remote_relationship';
    const BASIC_USERNAME = 'ProdigyWP1admin';
    const BASIC_PASSWORD = 'RHuFw0Dvg0G6HXJC';


    /**
     * @param int $post_id
     *
     * @return mixed
     */
    public function get_remote_product_data( $post_id ) {
        $is_exist_product_data = get_post_meta( $post_id, self::REMOTE_RELATIONSHIP_META );

        return isset( $is_exist_product_data[0] ) ? $is_exist_product_data[0] : 0;
    }

    /**
     * @param int   $post_id
     * @param array $remote_data
     *
     * @return bool|int
     */
    public function update_remote_product_data( $post_id, $remote_data ) {
        return update_post_meta( $post_id, self::REMOTE_RELATIONSHIP_META, $remote_data );
    }

    /**
     * @param array $response
     *
     * @return array
     */
    public function parse_variants_from_response( $response ) {
        $data_response = $response;
        $response      = [];
        if ( isset( $data_response['included'] ) ) {
            $key = 0;
            foreach ( $data_response['included'] as $item ) {
                if ( $item['type'] === 'variants' && !$item['attributes']['master']) {
                    $response['variants'][ $key ] = $item;
                    $key ++;
                }

                if (isset($item['attributes']['master'])) {
                    $response['master_variant_id'] = $item['id'];
                }
            }
        }

        if (isset($data_response)) {
            $response['remote_product_id'] = $data_response['data']['id'];
        }

        return $response;
    }

    /**
     * @param array $form_variants
     * @param array $remote_variants
     *
     * @return array
     */
    public function update_variants_for_remote_product( $form_variants, $remote_variants ) {
        $local  = [];
        $remote = [];

        if ( !empty($form_variants) ) {
            foreach ($form_variants as $form_variant) {
                $local[$form_variant['sku']] = $form_variant;
            }
        }

        if ( !empty($remote_variants) ) {
            foreach ($remote_variants as $remote_variant) {
                $remote[$remote_variant['attributes']['sku']] = $remote_variant;
            }
        }

        $variants_to_update = array_intersect_key( $local, $remote );
        $variants_to_delete = array_diff_key( $remote, $local );
        $variants_to_create = array_diff_key( $local, $remote );

        $variants = [];
        foreach ( $variants_to_update as $sku => $variant ) {
            $current_variant       = $local[ $sku ];
            $current_variant['id'] = $remote[ $sku ]['id'];
            $variants[]            = $current_variant;
        }

        foreach ( $variants_to_create as $sku => $variant ) {
            $current_variant        = $local[ $sku ];
            $variants[]             = $current_variant;
        }

        foreach ( $variants_to_delete as $sku => $variant ) {
            $current_variant = [
                'sku'     => $sku,
                'delete' => true,
            ];
            $variants[]      = $current_variant;
        }

        return $variants;
    }

    /**
     * @param int $product_id
     */
    public function delete_remote_product_data( $product_id ) {
        delete_post_meta( $product_id, self::REMOTE_RELATIONSHIP_META );
    }

    /**
     * @param int $product_id
     *
     * @return array
     */
    public function prepare_request_for_retry_product( $product_id ) {
        $product = get_prodigy_product( $product_id );

        $taxable      = $product->get_meta_field( 'taxable' );
        if ($taxable) {
            $tax_category_code = $product->get_meta_field( 'tax_category_id' ) ?? '';
            if ( ! empty( $tax_category_code ) ) {
                $tax_category_name = $this->get_category_name_by_code( $tax_category_code );
            } else {
                $tax_category_name = '';
            }
        }

        $local_categories = $product->get_categories();

        $categories = [];
        $key        = 0;

        if ( is_array($local_categories) ) {
            foreach ( $local_categories as $category ) {
                $category_id = get_term_meta( $category->term_id,
                    Prodigy_Admin_Category::PRODIGY_HOSTED_CATEGORY_RELATION, true );
                if ( ! empty( $category_id ) ) {
                    $categories[ $key ] = $category_id;
                    $key ++;
                }
            }
        }

        $image = $this->prepare_image_for_send($product->get_meta_field( '_thumbnail_id' ));

        $post_url = parse_url(get_permalink( $product_id ));

        $tags = [];
        if ( ! empty( $product->get_tags() ) ) {
            $tags = $product->get_tags();
        }

        $request = [
            'image_base64'    => $image['src'],
            'tag_list'        => $tags,
            'include'         => "categories,tax-information,master-variant.dimension,master-variant.inventory,variants.dimension,variants.inventory",
            'name'            => $product->get_field( 'post_title' ),
            'description'     => $product->get_field( 'post_content' ),
            'category_ids'    => $categories,
            'sku'             => $product->get_meta_field( 'product_sku' ),
            'url'             => $post_url['path'],
            'price'           => $product->get_meta_field( 'product_regular_price' ),
            'sale_price'      => $product->get_meta_field( 'product_sale_price' ) ?? '',
            'variants'        => ! empty( $product->get_variants() ) ?
                $this->prepare_variants_for_retry( $product->get_variants(), get_permalink( $product_id ), $image, $product->get_meta_field( 'product_requires_shipping' ) ) :
                []
        ];

        if ( isset( $image['name'] ) ) {
            $request['image_filename'] = $image['name'];
        }

        if ($taxable) {
            $request['tax_information'] = [
                'category_name' => $tax_category_name,
                'category_code' => $tax_category_code,
                'taxable'       => isset( $taxable )
            ];
        } else {
            $request['tax_information'] = [
                'category_name' => '',
                'category_code' => '',
                'taxable'       => false
            ];
        }

        $this->get_dimensions_for_request( $request, $product, true );

        return $request;
    }


    /**
     * @param string $img_url
     *
     * @return array
     */
    public function prepare_image_for_send( $img_url ) {
        $image_url = ! empty( wp_get_attachment_image_url( $img_url ) ) ? wp_get_attachment_image_url( $img_url ) : '';
        $file = pathinfo( $image_url );
        $image['src'] = '';

        $image_str = '';
        if (!empty($image_url)) {
            if ( PRODIGY_ENVIRONMENT === 'staging' || PRODIGY_ENVIRONMENT === 'develop' ) {

                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger',
                        $_SERVER['SERVER_NAME'] . (string) self::BASIC_USERNAME . ':' . self::BASIC_PASSWORD,
                        "is dev server" );
                }
                $context = stream_context_create( [
                    'http' => [
                        'header' => "Authorization: Basic " . base64_encode( (string) self::BASIC_USERNAME . ':' . self::BASIC_PASSWORD )
                    ]
                ] );

                $image_str = file_get_contents( $image_url, false, $context );
            } else {
                $image_src = wp_remote_get( $image_url );

                if ( ! empty( $image_src ) ) {
                    $image_str = $image_src['body'];
                }
            }
        }

        if ( PRODIGY_DEBUG_MODE ) {
            do_action( 'logger', $image_str, "get image binary" );
        }

        if ( !empty($image_url) ) {
            $image['src'] = 'data:image/' . $file['extension'] . ';base64,' . base64_encode( $image_str );
            $image['name'] = $file['basename'];
        }

        return $image;
    }

    /**
     * @param int $post_id
     * @param array $post
     *
     * @return array
     */
    public function prepare_request( $post_id, $post ) {
        
        $taxable = false;
        if (isset($post['taxable'])) {
            $taxable = $post['taxable'];
        }
        
        if ( $taxable ) {
            $tax_category_code = $post['tax_category_id'] ?? '';
            if ( ! empty( $tax_category_code ) ) {
                $tax_category_name = $this->get_category_name_by_code( $tax_category_code );
            } else {
                $tax_category_name = '';
            }
        }

        $local_categories = $post['tax_input'][ Prodigy::get_prodigy_category_type() ];

        $default_category = get_term_by('name', 'DefaultCategory', Prodigy::get_prodigy_category_type());

        if (!empty($default_category) && !empty($local_categories)) {
            foreach ( $local_categories as $key=>$value ) {
                if ( empty((int)$value) || $value == $default_category->term_id ) {
                    unset( $local_categories[ $key ] );
                }
            }
        }

        $categories = [];
        $key = 0;

        if ( !empty($local_categories) ) {
            foreach ( $local_categories as $category ) {
                $category_id = get_term_meta( $category, Prodigy_Admin_Category::PRODIGY_HOSTED_CATEGORY_RELATION,
                    true );
                if ( ! empty( $category_id ) ) {
                    $categories[ $key ] = $category_id;
                    $key ++;
                }
            }
        }

        $image = $this->prepare_image_for_send($post['_thumbnail_id']);

        $post_url      = parse_url( get_permalink( $post_id ) );
        $path_url      = explode( '/', $post_url['path'] );
        $info_path_url = array_filter( $path_url );
    
        if ( empty($info_path_url) && ! array_key_exists( Prodigy::get_prodigy_product_type(), $info_path_url ) ) {
            $product_link = '/' . $post['post_type'] . '/' . sanitize_title( $post['post_title'] );
        } else {
            $product_link = $post_url['path'];
        }

        $tags = [];
        if ( ! empty( $post['tax_input'][Prodigy::get_prodigy_tag_type()] ) ) {
            $tags = explode( ",", $post['tax_input'][Prodigy::get_prodigy_tag_type()] );
        }

        if ( PRODIGY_DEBUG_MODE ) {
            do_action( 'logger', $categories, 'categories' );
        }

        $post_fields = [
            'image_base64'    => $image['src'],
            'tag_list'        => $tags,
            'include'         => "categories,tax-information,master-variant.dimension,master-variant.inventory,variants.dimension,variants.inventory",
            'name'            => $post['post_title'],
            'description'     => $post['content'],
            'category_ids'    => $categories,
            'sku'             => $post['inventory_sku'],
            'url'             => $product_link,
            'price'           => $post['regular_price'],
            'sale_price'      => empty($post['sale_price']) ? '' : $post['sale_price'],
            'variants'        => ! empty( $post['variants'] ) ?
                $this->prepare_variants($post['variants'], $product_link, $image, $post['product_requires_shipping']) :
                [],
        ];

        if (isset($image['name'])) {
            $post_fields['image_filename'] = $image['name'];
        }

        if ($taxable) {
            $post_fields['tax_information'] = [
                'category_name' => $tax_category_name,
                'category_code' => $tax_category_code,
                'taxable'       => isset( $taxable )
            ];
        } else {
            $post_fields['tax_information'] = [
                'category_name' => '',
                'category_code' => '',
                'taxable'       => false
            ];
        }

        $this->get_dimensions_for_request( $post_fields, $post );

        return $post_fields;
    }

    /**
     * @param array $post_fields
     * @param bool  $is_update
     *
     * @return mixed
     */
    public function set_payload_for_request( $post_fields, $is_update = false ) {
        $query        = urldecode( http_build_query( $post_fields ) );
        $params       = explode( '&', $query );
        $query_params = [];

        foreach ( $params as $param ) {
            $param                = explode( '=', $param );
            $key                  = $param[0];
            $query_params[ $key ] = $param[1];
        }

        if ( isset( $post_fields['variants'] ) && empty( $post_fields['variants'] ) ) {
            $query_params['variants[]'] = null;
        }

        if ( $is_update ) {
            unset( $query_params['variants[]'] );
        }

        $boundary = '----' . wp_generate_password( 24, false );
        $headers  = array(
            'Content-type' => 'multipart/form-data; boundary=' . $boundary
        );

        $payload = '';

        // First, add the standard POST fields
        foreach ( $query_params as $name => $value ) {
            $name = preg_replace( '/\[\d+\]/', '[]', $name );
            if ( preg_match( '/image/', $name ) ) {
                $payload .= '--' . $boundary;
                $payload .= "\r\n";
                $payload .= 'Content-Disposition: form-data; name="' . $name .
                            '"; filename="' . basename( $value ) . '"' . "\r\n";
                $payload .= "\r\n";
                $payload .= !empty( $value ) ? file_get_contents( $value ) : '';
                $payload .= "\r\n";
            } else {
                $payload .= '--' . $boundary;
                $payload .= "\r\n";
                $payload .= 'Content-Disposition: form-data; name="' . $name .
                            '"' . "\r\n\r\n";
                $payload .= isset( $value ) ? $value : '';
                $payload .= "\r\n";
            }
        }

        $payload .= '--' . $boundary . '--';

        $request['body']    = $payload;
        $request['headers'] = [
            'Content-type' => 'multipart/form-data; boundary=' . $boundary,
        ];

        return $request;
    }


    /**
     * @param string $category_code
     *
     * @return mixed
     */
    public function get_category_name_by_code( $category_code ) {
        $tax_categories_list = json_decode( get_option('pg_tax_categories_list'), true );

        foreach ( $tax_categories_list['categories'] as $category ) {
            if ($category['product_tax_code'] == $category_code) {
                return $category['name'];
            }
        }
    }

    /**
     * @param array $variants
     * @param string $product_url
     * @param array $main_image
     * @param bool $shippable
     * @return array
     */
    public function prepare_variants_for_retry( array $variants, string $product_url, $main_image, $shippable ) {
        $response = [];

        foreach ( $variants as $key => $variant ) {

            $post_url = parse_url($product_url);

            $image  = empty($variants['category-image-id'][ $key ]) ? $main_image : $this->prepare_image_for_send($variant['category-image-id']);

            $response[ $key ]['image_base64'] = !empty($image) ? $image['src'] : '';
            $response[ $key ]['sku']   = $variant['product_sku'];
            $response[ $key ]['url']   = $post_url['path'];
            $response[ $key ]['price'] = $variant['product_regular_price'];
            $response[ $key ]['sale_price'] = empty($variant['product_sale_price']) ? '' : $variant['product_sale_price'];

            if ( !empty($image['src'] ) ) {
                $response[ $key ]['image_filename'] = $image['name'];
            }

            $options = json_decode( str_replace( "'", '"', stripslashes( $variant['post_attributes'] ) ), 1 );

            $i = 0;
            foreach ( $options as $name => $option ) {
                $response[ $key ]['options'][ $i ]['name']  = $name;
                $response[ $key ]['options'][ $i ]['value'] = $option;
                $i ++;
            }

            $response[ $key ]['dimension']['weight_value'] = $variant['product_weight_value'];
            $response[ $key ]['dimension']['height_value'] = $variant['shipping_dimension_height'];
            $response[ $key ]['dimension']['width_value']  = $variant['shipping_dimension_width'];
            $response[ $key ]['dimension']['depth_value']  = $variant['shipping_dimension_length'];
            $response[ $key ]['dimension']['size_unit']    = Prodigy_Product::$dimension_type_mapper[ $variant['dimension'] ];
            $response[ $key ]['dimension']['weight_unit']  = Prodigy_Product::$weight_type_mapper[ $variant['product_weight_type'] ];
            $response[ $key ]['dimension']['shippable']    = !empty( $shippable );

            $response[ $key ]['inventory']['stock'] = Prodigy_Product::$stock_status_mapper_for_api[ $variant['product_stock_status'] ];
        }

        return $response;
    }

    /**
     * @param array $variants
     * @param string $product_url
     * @param array $main_image
     * @param bool $shippable
     * @return array
     */
    public function prepare_variants( array $variants, string $product_url, $main_image, $shippable ) {
        $response = [];

        $count_variants = count( $variants['post_title'] );

        for ( $key = 0; $key <= $count_variants - 1; $key ++ ) {

            $image  = empty($variants['category-image-id'][ $key ]) ? $main_image : $this->prepare_image_for_send($variants['category-image-id'][ $key ]);

            $response[ $key ]['sku']   = $variants['product_sku'][ $key ];
            $response[ $key ]['image_base64'] = !empty($image) ? $image['src'] : '';
            $response[ $key ]['url']   = $product_url;
            $response[ $key ]['price'] = $variants['product_regular_price'][ $key ];
            $response[ $key ]['sale_price'] = empty($variants['product_sale_price'][ $key ]) ? '' : $variants['product_sale_price'][ $key ];

            if (!empty($image['src'])) {
                $response[ $key ]['image_filename'] = $image['name'];
            }

            $options = json_decode( str_replace( "'", '"', stripslashes( $variants['post_attributes'][ $key ] ) ), 1 );

            $i = 0;
            foreach ( $options as $name => $option ) {
                $response[ $key ]['options'][ $i ]['name']  = $name;
                $response[ $key ]['options'][ $i ]['value'] = $option;
                $i ++;
            }

            $response[ $key ]['dimension']['weight_value'] = $variants['product_weight_value'][ $key ];
            $response[ $key ]['dimension']['height_value'] = $variants['shipping_dimension_height'][ $key ];
            $response[ $key ]['dimension']['width_value']  = $variants['shipping_dimension_width'][ $key ];
            $response[ $key ]['dimension']['depth_value']  = $variants['shipping_dimension_length'][ $key ];
            $response[ $key ]['dimension']['size_unit']    = Prodigy_Product::$dimension_type_mapper[ $variants['dimension'][ $key ] ];
            $response[ $key ]['dimension']['weight_unit']  = Prodigy_Product::$weight_type_mapper[ $variants['product_weight_type'][ $key ] ];
            $response[ $key ]['dimension']['shippable']    = !empty( $shippable );

            $response[ $key ]['inventory']['stock'] = Prodigy_Product::$stock_status_mapper_for_api[ $variants['product_stock_status'][ $key ] ];
        }

        return $response;
    }

    /**
     * @param array $request
     * @param array $item
     * @param bool  $is_update
     *
     * @return mixed
     */
    public function get_dimensions_for_request( &$request, $item, $is_update = false ) {
            if ( $is_update ) {
                $product_weight_value = $item->get_meta_field( 'product_weight_value' );
                if ( isset($product_weight_value) ) {
                    $request['dimension']['weight_value'] = $product_weight_value;
                }
                $shipping_dimension_height = $item->get_meta_field( 'shipping_dimension_height' );
                if ( isset( $shipping_dimension_height ) ) {
                    $request['dimension']['height_value'] = $shipping_dimension_height;
                }
                $shipping_dimension_width = $item->get_meta_field( 'shipping_dimension_width' );
                if ( isset( $shipping_dimension_width ) ) {
                    $request['dimension']['width_value'] = $shipping_dimension_width;
                }
                $shipping_dimension_length = $item->get_meta_field( 'shipping_dimension_length' );
                if ( isset( $shipping_dimension_length ) ) {
                    $request['dimension']['depth_value'] = $item->get_meta_field( 'shipping_dimension_length' );
                    $request['dimension']['size_unit']   = Prodigy_Product::$dimension_type_mapper[ $item->get_meta_field( 'shipping_dimensions' ) ];
                    $request['dimension']['weight_unit'] = Prodigy_Product::$weight_type_mapper[ $item->get_meta_field( 'product_weight_type' ) ];
                    $check_shipping                      = $item->get_meta_field( 'check_requires_shipping' );
                    $request['dimension']['shippable']   = !empty( $check_shipping );
                    $request['inventory']['stock']       = Prodigy_Product::$stock_status_mapper_for_api[ $item->get_meta_field( 'product_stock_status' ) ];
                }
            } else {
                if ( isset( $item['product_requires_shipping'] ) ) {
                    $request['dimension']['weight_value'] = $item['product_weight_value'];
                    $request['dimension']['height_value'] = $item['shipping_dimension_height'];
                    $request['dimension']['width_value']  = $item['shipping_dimension_width'];
                    $request['dimension']['depth_value']  = $item['shipping_dimension_length'];
                    $request['dimension']['size_unit']    = Prodigy_Product::$dimension_type_mapper[ $item['shipping_dimensions'] ];
                    $request['dimension']['weight_unit']  = Prodigy_Product::$weight_type_mapper[ $item['product_weight_type'] ];
                    $request['dimension']['shippable']    = isset( $item['product_requires_shipping'] );
                    $request['inventory']['stock']        = Prodigy_Product::$stock_status_mapper_for_api[ $item['inventory_stock_status'] ];
                } else {
                    $request['dimension']['weight_value'] = "";
                    $request['dimension']['height_value'] = "";
                    $request['dimension']['width_value']  = "";
                    $request['dimension']['depth_value']  = "";
                    $request['dimension']['size_unit']    = get_option( 'pg_dimensions');
                    $request['dimension']['weight_unit']  = get_option( 'pg_weight');
                    $request['dimension']['shippable']    = isset( $item['product_requires_shipping'] );
                    $request['inventory']['stock']        = 'in_stock';
                }
            }

        return $request;
    }


    /**
     * @param int    $post_id
     * @param string $status
     *
     * @return bool|int
     */
    public function update_product_status( $post_id, $status ) {
        global $wpdb;

        return $wpdb->query(
            $wpdb->prepare(
                "UPDATE $wpdb->posts SET post_status = %s WHERE ID = %d",
                [ $status, $post_id ]
            )
        );
    }

}