<?php

/**
 * Preparing for send batch products
 *
 * @version    1.1.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once 'class-prodigy-aggregate-product-data-synchronize.php';
include_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prodigy-aggregate-product-data-synchronize.php';

/**
 * Class Prodigy_Batch_Products
 */
class Prodigy_Batch_Products_Synchronize extends Prodigy_Aggregate_Data_Synchronize {

    const BATCHES_PRODUCTS_TABLE = 'prodigy_batches_sent_out_products';

    /**
     * @param array $products
     *
     * @return array
     */
    public function prepare_batch_products( array $products ) {
        $prepared_request = [];
        $product_ids = [];
        $i = 0;
        foreach ( $products as $product ) {
            $key = $i++;
            $product_ids[$key] = $product->get_row_id();
            $prepared_request['products'][$key] = $this->prepare_request_for_retry_product( $product->get_row_id() );
            $prepared_request['products'][$key]['row_id'] = $product->get_row_id();
        }

        return $prepared_request;
    }

    /**
     * @param $products_response
     */
    public function update_products_data( $products_response ) {
        global $wpdb;

        $table = $wpdb->prefix . self::BATCHES_PRODUCTS_TABLE;

        $prodigy_synchronize = new Prodigy_Aggregate_Data_Synchronize();

        foreach ( (array) $products_response['data'] as $product ) {
            if ( $product['attributes']['success'] ) {
                $wpdb->get_results(
                    $wpdb->prepare(
                        "UPDATE $table SET completed = %d, remote_product_id=%d WHERE product_id = %d",
                        [
                            true,
                            (int) $product['attributes']['data']['id'],
                            (int) $product['attributes']['row-id'],
                        ]
                    )
                );

                if ( $wpdb->result ) {
                    $wpdb->get_results(
                        $wpdb->prepare(
                            "UPDATE $wpdb->posts SET post_status = %s WHERE post_type = %s and ID = %d",
                            [
                                Prodigy::PRODIGY_PUBLISH_POST_STATUS,
                                Prodigy::get_prodigy_product_type(),
                                $product['attributes']['row-id']
                            ]
                        )
                    );

                    update_post_meta(
                        $product['attributes']['row-id'],
                        Prodigy_Aggregate_Data_Synchronize::PRODUCT_STATUS_SYNCHRONIZE,
                        'success'
                    );

                    $remote_data = $prodigy_synchronize->parse_variants_from_response( $product['attributes'] );
                    $prodigy_synchronize->update_remote_product_data( (int) $product['attributes']['row-id'], $remote_data );

                }
            } else {
                $wpdb->get_results(
                    $wpdb->prepare(
                        "UPDATE $table SET error = %s WHERE product_id = %d",
                        [
                            json_encode( $product['attributes']['errors'] ),
                            $product['attributes']['row-id'],
                        ]
                    )
                );
                if ( PRODIGY_DEBUG_MODE ) {
                    do_action( 'logger', $product['attributes']['errors'], 'error send product for batch process' );
                }
            }
        }
    }

    /**
     * @param array $products
     */
    public function insert_sending_batch_products($local_batch_id,  array $productsIds ) {
        global $wpdb;


        foreach ( $productsIds as $product_id ) {
            $table     = $wpdb->prefix . self::BATCHES_PRODUCTS_TABLE;
            $data['local_batch_id'] = $local_batch_id;
            $data['product_id'] = $product_id;
            $data['created_at'] = date( 'Y-m-d H:i:s' );
            $wpdb->insert( $table, $data, [ '%s', '%s', '%s' ] );
        }
    }

    /**
     * @param array $products
     * @param int   $batch_id
     *
     */
    public function update_sending_batch_products( array $products, int $batch_id ) {
        global $wpdb;

        foreach ( $products as $key => $product_id ) {
            $table     = $wpdb->prefix . self::BATCHES_PRODUCTS_TABLE;
            $data['remote_batch_id'] = $batch_id;
            $data['updated_at'] = date( 'Y-m-d H:i:s' );
            $wpdb->update( $table, $data, ["product_id" => $product_id], [ '%d', '%s'],[ '%d' ] );
        }
    }


    /**
     * @param int $product_id
     *
     * @return bool
     */
    public function is_sended_product( $product_id ) {
        global $wpdb;

        $table = $wpdb->prefix . self::BATCHES_PRODUCTS_TABLE;

        $result = $wpdb->get_results(
            $wpdb->prepare( "SELECT * FROM $table WHERE product_id = %d",
                [ $product_id ]
            )
        );

        return !empty($result);
    }


}

return new Prodigy_Batch_Products_Synchronize();