<?php

/**
 * Fired during plugin activation
 *
 * @version 1.0.0
 * @package prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

require_once ABSPATH . 'wp-admin/includes/upgrade.php';

/**
 * Class Prodigy_Install_Wizard
 */
class Prodigy_Install_Wizard {

    /**
     * Set table after plugin install
     * if run unit-tests this func off or return of first
     */
    public static function set_tables() {
        global $wpdb;

        $collate = self::get_charset_collate();

        $tables = "
            CREATE TABLE {$wpdb->prefix}prodigy_attribute_taxonomy (
                 id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                 slug  varchar(200),
                 name varchar(200),
                 order_by tinyint(1) default 0,
                 public tinyint(1) default 1,
                 constraint wp_prodigy_attribute_taxonomy_slug_uindex unique (slug),
                 constraint wp_prodigy_attribute_taxonomy_name_uindex unique (name)
            ) $collate;
            
            CREATE TABLE {$wpdb->prefix}prodigy_product_attributes (
                 product_id int(11) NOT NULL,
                 attribute_id int(11) NOT NULL,
                 term_id      int(11) NOT NULL
            ) $collate; 
            
            CREATE TABLE {$wpdb->prefix}prodigy_attributes_aggregation (
                 id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                 attribute_id  int(11) NOT NULL,
                 attribute_value char(100)     null,
                 count_products  int default 0 null,
                 term_id         int(11)           null,
                 constraint attributes_id_value_aggregation__uindex unique (attribute_id, attribute_value)
            ) $collate;
            
            CREATE TABLE {$wpdb->prefix}prodigy_remote_cart_items (
                 id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                 local_product_id  int(11) NOT NULL,
                 local_user_hash varchar(255) NOT NULL,
                 remote_product_id int(11) NOT NULL,
                 line_item_id int(11) NOT NULL,
                 count_items int(11) default 1,
                 order_number int(11) NOT NULL,
                 token varchar(255) NOT NULL,
                 api_response text,
                 remote_order_item_id int(11) not null,
                 INDEX remote_product_id_index (remote_product_id)
            ) $collate;

            CREATE TABLE {$wpdb->prefix}prodigy_cart_items (
                 id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                 local_product_id  int(11) NOT NULL,
                 remote_product_id int(11) NOT NULL,
                 count_items int(11) default 1,
                 hash varchar(255) NOT NULL,
                 order_token varchar(255) NOT NULL,
                 expiration_date int(11) NOT NULL,
                 attributes text,
                 INDEX cart_items_hash_index (hash),
                 INDEX cart_items_expiration_date_index (expiration_date)
            ) $collate;
            
            CREATE TABLE {$wpdb->prefix}prodigy_orders (
                 id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                 local_user_hash varchar(255) NOT NULL,
                 order_number int(11) NOT NULL,
                 token varchar(255) NOT NULL,
                 status TINYINT(1) default 0,
                 includes text,
                 deleted_at timestamp null default NULL,
                 created_at timestamp default CURRENT_TIMESTAMP,
                 updated_at timestamp default CURRENT_TIMESTAMP
            ) $collate;
            
            CREATE TABLE {$wpdb->prefix}prodigy_user_info (
                 id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                 user_session_hash varchar(255) NOT NULL,
                 name varchar(255)
            ) $collate;
            
            CREATE TABLE {$wpdb->prefix}prodigy_batches_sent_out_products (
                 id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                 local_batch_id varchar(55) NOT NULL,
                 remote_batch_id int(11) NOT NULL,
                 product_id int(11) NOT NULL,
                 remote_product_id int(11) NOT NULL,
                 completed tinyint(1) DEFAULT 0,
                 error text,
                 created_at timestamp DEFAULT CURRENT_TIMESTAMP,
                 updated_at timestamp DEFAULT CURRENT_TIMESTAMP
            ) $collate;
            ";

        dbDelta($tables);
    }

    /**
     * Set collate for custom tables
     *
     * @return string
     */
    public static function get_charset_collate() {
        return "ENGINE=InnoDB DEFAULT CHARSET=utf8";
    }


    /**
     * Retrieve page ids - used for shop, cart. returns -1 if no page is found.
     *
     * @param string $page Page slug.
     * @return int
     */
    public static function prodigy_get_page_id( $page ) {
        $page = apply_filters( 'prodigy_get_' . $page . '_page_id', get_option( 'prodigy_' . $page . '_page_id' ) );

        return $page ? absint( $page ) : -1;
    }
    
    /**
     * clear prodigy pages
     */
    public static function publish_pages () {
        
        $prodigy_pages = [
            'cart',
            'thank-you',
            'shop'
        ];
        
        foreach ($prodigy_pages as $prodigy_slug ) {
            $args = [
                'name'        => $prodigy_slug,
                'post_type'   => 'page',
                'numberposts' => 1
            ];
            $info_post = get_posts($args);
            wp_publish_post($info_post[0]->ID);
        }
        
    }
    
}

return new Prodigy_Install_Wizard();