<?php

/**
 * Prodigy product attributes class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Product_attributes
 */
class Prodigy_Product_Attributes extends Prodigy_Product {
    
    const PRODUCT_ATTRIBUTE_META = '_pg_attribute';
    
    const PRODUCT_ATTRIBUTE_TABLE = 'prodigy_attribute_taxonomy';
    const PRODUCT_ATTRIBUTES = 'prodigy_product_attributes';
    const AGGREGATION_ATTRIBUTES = 'prodigy_attributes_aggregation';
    
    const DELETE_AGGREGATION_ATTRIBUTES = true;
    
    const INCREMENT = 1;
    const DECREMENT = - 1;
    
    /**
     * for product before update
     * @var $old_attributes
     */
    public $old_attributes;
    
    /**
     * Get attribute names
     *
     * @param string $search_query
     *
     * @return array
     */
    public function get_attributes( $search_query = '' ) {
        $attributes = self::get_attribute_taxonomies( sanitize_text_field( $search_query ) );
        
        $name_attributes = [];
        foreach ( $attributes as $key => $attribute ) {
            $name_attributes[ $key ] = $attribute;
        }
        
        return $name_attributes;
    }
    
    /**
     * @param string $search_query
     *
     * @return array
     */
    public function get_all_attributes_with_values_list( $search_query = '' ) {
        $all_attributes = $this->get_attributes( sanitize_text_field($search_query) );
        
        $all_attributes_values = [];
        foreach ( $all_attributes as $attr ) {
            $all_attributes_values[ $attr->id ]['attribute'] = $attr;
            $all_attributes_values[ $attr->id ]['values']    = $this->get_values_for_attribute( $attr->slug );
        }
        
        return $all_attributes_values;
    }


    /**
     * @param string $slug
     *
     * @return array
     */
    public function get_attribute_with_values_by_slug( $slug ) {
        $attribute_values['attribute'] = Prodigy_Product_Attributes::get_attribute_taxonomies_by_slug( $slug );
        $attribute_values['values']    = $this->get_values_for_attribute( $slug );

        return $attribute_values;
    }

    /**
     * @param $id
     *
     * @return array
     */
    public function get_attribute_with_values_by_id( $id ) {
        $attribute_values['attribute'] = Prodigy_Product_Attributes::get_attribute_taxonomies_by_id( $id );
        $attribute_values['values']    = get_terms( [ 'taxonomy' => $attribute_values['attribute']->slug ], [
            'hide_empty' => false,
        ] );
        
        return $attribute_values;
    }
    
    /**
     * Get all attributes
     *
     * @param string $search_query
     *
     * @return mixed
     */
    public static function get_attribute_taxonomies( $search_query = '' ) {
        global $wpdb;
        $table_name = $wpdb->prefix . self::PRODUCT_ATTRIBUTE_TABLE;
        $attribute_taxonomies = false;
        if ( $wpdb->get_var( "SHOW TABLES LIKE '{$table_name}' " ) == $table_name ) {
            $sql = "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE;
            $search_result = esc_sql($search_query);
            if ( ! empty( $search_query ) ) {
                $sql .= " where name like '%{$search_result}%' OR slug LIKE '%{$search_result}%' ";
            }
            $attribute_taxonomies = $wpdb->get_results( $sql );
        }
        
        return $attribute_taxonomies;
    }

    /**
     * Get all attributes
     *
     * @return mixed
     */
    public static function get_attribute_taxonomies_aggregation() {
        global $wpdb;
        
        $sql = "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE . " as t1
         LEFT JOIN {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES . " as t2 on t1.id = t2.attribute_id
         ";
        
        $attribute_taxonomies = $wpdb->get_results( $sql );
        
        return $attribute_taxonomies;
    }
    
    public static function truncate_attrs_aggregation() {
        global $wpdb;
        
        $wpdb->query( "TRUNCATE TABLE {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES );
        
        return true;
    }
    
    public static function truncate_attrs_taxonomy() {
        global $wpdb;
        
        $wpdb->query( "TRUNCATE TABLE {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE );
        
        return true;
    }
    
    public static function truncate_product_attrs() {
        global $wpdb;
        
        $wpdb->query( "TRUNCATE TABLE {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES );
        
        return true;
    }

    /**
     * @param string $taxonomy_name
     *
     * @return array|null
     */
    public function delete_item_from_attribute_taxonomy( $taxonomy_name ) {
        global $wpdb;

        $attribute_taxonomies = $wpdb->get_results(
            "DELETE FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE . " where slug = {$taxonomy_name}"
        );

        return $attribute_taxonomies;
    }

    /**
     * @param string $slug
     *
     * @return object|null
     */
    public function get_attribute_taxonomies_by_name( $slug ) {
        global $wpdb;

        $attribute_taxonomies = $wpdb->get_results( $wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE . " where slug = '%s'",
            [ $slug ]
        ));

        return $attribute_taxonomies;
    }

    /**
     * @param int $id
     *
     * @return object|null
     */
    public static function get_attribute_taxonomies_by_id( int $id ) {
        global $wpdb;

        $attribute_taxonomies = $wpdb->get_row( $wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE . " where id = %d",
            [ $id ]
        ));

        return $attribute_taxonomies;
    }


    /**
     * @param string $slug
     *
     * @return object|null
     */
    public static function get_info_prodigy_attr_taxonomy_by_slug( $slug ) {
        global $wpdb;

        $attribute_taxonomies = $wpdb->get_row( $wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE . " where slug = '%s'",
            [ $slug ]
        ));

        return $attribute_taxonomies;
    }

    /**
     * @param string $slug
     *
     * @return object|null
     */
    public static function get_attribute_name_by_slug( $slug ) {
        global $wpdb;

        $attribute_name = $wpdb->get_row( $wpdb->prepare(
            "SELECT name FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE . " where slug = '%s'",
            [ $slug ]
            ), ARRAY_A
        );

        return $attribute_name['name'];
    }

    /**
     * @param string $slug
     *
     * @return array|WP_Error
     */
    public function get_values_for_attribute( $slug ) {
        global $wpdb;

        $attribute_taxonomies = $wpdb->get_results( $wpdb->prepare(
            "SELECT * FROM {$wpdb->term_taxonomy} WHERE taxonomy = '%s'",
            [ $slug ]
        ));

        $attributes_ids = [];
        foreach ( $attribute_taxonomies as $key => $attribute ) {
            $attributes_ids[ $key ] = $attribute->term_id;
        }

        $placeholders     = where_in_sanitize( $attributes_ids );
        $sql              = "SELECT * FROM {$wpdb->terms} WHERE term_id IN ($placeholders)";
        $attribute_values = $wpdb->get_results($wpdb->prepare($sql, $attributes_ids));

        $array_values = [];
        foreach ( $attribute_values as $key => $value ) {
            $array_values[ $key ]['id']   = $value->term_id;
            $array_values[ $key ]['text'] = $value->name;
            $array_values[ $key ]['slug'] = $value->slug;
        }

        return $array_values;
    }

    /**
     * @param array $value_ids
     *
     * @return array
     */
    public static function get_value_by_ids( $value_ids ) {
        global $wpdb;

        if ( isset( $value_ids ) ) {
            $placeholders = where_in_sanitize($value_ids);
            $sql = "SELECT * FROM {$wpdb->terms} WHERE term_id IN ({$placeholders})";
            $values = $wpdb->get_results($wpdb->prepare($sql, $value_ids));

            $value_names = [];
            foreach ( $values as $value ) {
                $value_names[ $value->term_id ] = $value->name;
            }

            return $value_names;
        }
    }

    /**
     * @param int $post_id
     * @param array $post_attributes
     * @param array $post_values
     * @param array $post_is_use
     *
     * @return bool
     */
    public function update_attributes_from_product( $post_id, $post_attributes, $post_values, $post_is_use = [] ) {

        $attributes = [];

        /*
         * take old attr values
         */
        $this->old_attributes = $this->get_attributes_value_by_product( $post_id );
        
        if (is_array($this->old_attributes) && is_array($post_values)) {
            $terms_to_delete = $this->get_deleting_value_attributes($this->old_attributes, $post_values);
        }
    
        if (isset($terms_to_delete) && is_array($terms_to_delete)) {
            $this->remove_relationship_taxonomies_from_product($post_id, $terms_to_delete);
        }
        
        $this->delete_all_attributes_from_product( $post_id );
        $this->delete_relations_taxonomies_with_product( $post_id, maybe_unserialize( get_post_meta(
            $post_id,
            Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META,
            true
        ) ) );

        if ( empty( $post_attributes ) || empty( $post_values ) ) {
            return false;
        }

        foreach ( $post_attributes as $key => $attribute ) {

            $attr = explode( ",", $attribute );

            $values_attribute_one = array_values( $post_values[ $key ] );

            $this->set_relation_product_with_terms( $post_id, $values_attribute_one );

            $str_values_attribute_one = implode( ',', $values_attribute_one );

            if ( filter_var( $attribute, FILTER_SANITIZE_STRIPPED ) !== $attribute ) {
                Prodigy_Admin_Notification::displayWarning( __( "Field attribute '{$attr[0]}' is not valid", 'prodigy' ) );
                continue;
            }

            if ( filter_var( $str_values_attribute_one, FILTER_SANITIZE_STRIPPED ) !== $str_values_attribute_one ) {
                Prodigy_Admin_Notification::displayWarning( __( "Field attribute`s value '{$attr[0]}' is not valid", 'prodigy' ) );
                continue;
            }

            $attributes[ $key ]['attribute'][ $attr[1] ] = $attr[0];
            $attributes[ $key ]['values']                = $post_values[ $key ];
            if ( isset( $post_is_use[ $attr[1] ] ) ) {
                $attributes[ $key ]['is_use'] = $post_is_use[ $attr[1] ];
            }

            $result = $this->assign_product_to_attribute( (int) $post_id, (int) $attr[1], $values_attribute_one );

            if ( empty( $result ) ) {
                return false;
            }
        }

        if ( empty( $attribute ) ) {
            return false;
        }

        return update_post_meta( $post_id, Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META, serialize( $attributes ) );
    }
    
    public function remove_relationship_taxonomies_from_product(int $post_id, array $terms_to_delete) {
        foreach ($terms_to_delete as $item) {
            $term_info = get_term($item);
            
            if ( !empty($term_info)) {
                wp_remove_object_terms($post_id, $term_info->slug, $term_info->taxonomy);
            }
        }
    }
    
    /**
     * @param array $old_values
     * @param array $new_values
     *
     * @return array
     */
    public function get_deleting_value_attributes(array $old_values, array $new_values) {
        
        $list_old_values = [];
        $list_new_values = [];
        
        foreach ($old_values as $key => $value) {
            $list_old_values = array_merge($list_old_values, wp_list_pluck($value, 'term_id'));
        }
    
        foreach ($new_values as $key => $value) {
            $list_new_values = array_merge($list_new_values, array_values($value));
        }
        
        $diff_attrs = array_diff($list_old_values, $list_new_values);
        
        return $diff_attrs;
    }
    
    /**
     * @param int $post_id
     * @param array $terms
     */
    public function set_relation_product_with_terms( $post_id, $terms ) {
        global $wpdb;
        
        foreach ( $terms as $term_id ) {
            $data = [
                'object_id'        => $post_id,
                'term_taxonomy_id' => $term_id,
                'term_order'       => 0
            ];
            
            $wpdb->replace( 'wp_term_relationships', $data, [ '%d', '%d', '%d' ] );
        }
        
    }
    
    /**
     * @param int $id
     * @param array $params
     *
     * @return bool
     */
    public function update_attribute_with_values_by_id( int $id, $params ) {
        global $wpdb;
        
        $new_values = [];
        $result     = false;
        
        $id = filter_var( $id, FILTER_VALIDATE_INT );
        
        $term_info = self::get_attribute_taxonomies_by_id( $id );
        $slug      = $term_info->slug;
        
        $attribute_slug       = filter_var( $params['attribute_slug'], FILTER_SANITIZE_STRIPPED );
        $attribute_name       = filter_var( $params['attribute_name'], FILTER_SANITIZE_STRIPPED );
        $attribute_orderby    = filter_var( $params['attribute_orderby'], FILTER_SANITIZE_STRIPPED );
        $edit_attribute_value = filter_var( $params['edit_attribute_value'], FILTER_SANITIZE_STRING );
        
        $list_term_current      = get_terms( [ 'taxonomy' => $slug, 'hide_empty' => false ] );
        $list_term_current_slug = wp_list_pluck( $list_term_current, 'slug', 'term_id' );
        $info_attr_taxonomy_old = $this->get_info_prodigy_attr_taxonomy_by_slug( $slug );
        
        if ( $attribute_slug != $slug ) {
            $data = [
                'slug' => $attribute_slug,
            ];

            $wpdb->update( $wpdb->prefix . self::PRODUCT_ATTRIBUTE_TABLE, $data, [ 'slug' => $slug ], ['%s'], ['%s'] );
            $terms = get_terms( [ 'taxonomy' => $slug, 'hide_empty' => false ] );
            foreach ( $terms as $term ) {
                wp_delete_term( $term->term_id, $slug );
            }
            
            $args = [
                'hierarchical'      => false,
                'show_ui'           => false,
                'show_admin_column' => false,
                'query_var'         => true,
            ];
            
            register_taxonomy( $attribute_slug, [ Prodigy::get_prodigy_product_type() ], $args );
            
            $slug = $attribute_slug;
            
            $result = true;
        }
        
        if ( $attribute_name != $term_info->name || $attribute_orderby != $term_info->order_by ) {
            $data = [
                'name'     => $attribute_name,
                'order_by' => $attribute_orderby
            ];
            
            $wpdb->update( $wpdb->prefix . self::PRODUCT_ATTRIBUTE_TABLE, $data, [ 'slug' => $slug ], [ '%s','%s' ], ['%s'] );
            
            $result = true;
        }
        
        // rewrite terms
        $request_values = explode( ",", $edit_attribute_value );
        
        if ( empty( $request_values ) && ! is_array( $request_values ) ) {
            return false;
        }
        
        foreach ( $request_values as $val ) {
            if ( empty( $val ) ) {
                continue;
            }
            $new_values[] = $val;
        }
        
        $old_term_aggregation = array_diff( $list_term_current_slug, $new_values );
        
        if ( is_array( $old_term_aggregation ) ) {
            foreach ( $old_term_aggregation as $old_term_aggregation_key => $old_term_aggregation_val ) {
                if ( wp_delete_term( $old_term_aggregation_key, $attribute_slug ) ) {
                    $wpdb->delete( $wpdb->prefix . Prodigy_Product_Attributes::AGGREGATION_ATTRIBUTES, [ 'term_id' => $old_term_aggregation_key ], ['%d'] );
                }
            }
        }
        
        $new_values = array_unique( $new_values );
        
        foreach ( $new_values as $value ) {
            
            if ( empty( $value = filter_var( $value, FILTER_SANITIZE_STRING ) ) ) {
                continue;
            }
            
            $info_term_value = get_term_by( 'slug', $value, $attribute_slug );
            
            if ( ! empty( $info_term_value ) ) {
                continue;
            }
            
            $result_insert = wp_insert_term( $value, $attribute_slug );
            
            if ( is_array( $result_insert ) ) {
                
                $value = sanitize_title( $value );
                
                if ( ! in_array( $value, $list_term_current_slug ) ) {
                    
                    $wpdb->insert(
                        $wpdb->prefix . Prodigy_Product_Attributes::AGGREGATION_ATTRIBUTES,
                        [
                            'attribute_id'    => $info_attr_taxonomy_old->id,
                            'term_id'         => $result_insert['term_id'],
                            'attribute_value' => $value
                        ],
                        [
                            '%d',
                            '%d',
                            '%s'
                        ]
                    );
                } else {
                    $wpdb->update(
                        $wpdb->prefix . Prodigy_Product_Attributes::AGGREGATION_ATTRIBUTES,
                        [
                            'term_id' => $result_insert['term_id'],
                        ],
                        [
                            'attribute_id'    => $info_attr_taxonomy_old->id,
                            'attribute_value' => $value
                        ],
                        [ '%d' ],
                        [ '%d', '%s']
                    );
                }
                
            }
            
            $result = ! empty( $result_insert );
        }
        
        return $result;
    }
    
    /**
     * @param int $attr_id
     * @param int $term_id
     *
     * @return object|null
     */
    public function get_aggregation_products_by_attributes( int $attr_id, int $term_id ) {
        
        global $wpdb;
        
        $result = $wpdb->get_row(
            $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES . " WHERE attribute_id = %d AND term_id = %d",
                [ $attr_id, $term_id ]
            )
        );
        
        return $result;
    }
    

    /**
     * @param int $attr_id
     *
     * @return object|null
     */
    public function get_aggregation_values_by_attribute( int $attr_id ) {

        global $wpdb;

        $result = $wpdb->get_results(
            $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES . " WHERE attribute_id = %d",
                [ $attr_id ]
            )
        );

        return $result;
    }

    /**
     * @param array $ids (products)
     *
     * @return bool|object
     */
    public function get_aggregation_attr_values( array $ids ) {
        
        global $wpdb;
        
        $list_ids = implode( ',', $ids );
        
        if ( empty( $list_ids ) ) {
            return $this->get_all_aggregation_products_by_attributes();
        }
        
        $attributes     = $this->get_attribute_taxonomies();
        $current_object = get_queried_object();
        
        foreach ( $attributes as &$attribute ) {
            $attributes_value = $wpdb->get_results( $wpdb->prepare(
                "SELECT t2.term_id, t1.attribute_value, count(*) as count_products, count_products as real_count_products
                        FROM {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES . " as t1
                        LEFT JOIN {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES . " as t2 ON t2.term_id = t1.term_id
                        WHERE t2.product_id IN ({$list_ids}) and t1.attribute_id = %d
                        GROUP BY t1.attribute_value, t2.term_id",
                [
                    $attribute->id
                ]
            ), OBJECT_K );

            if ( isset($current_object->taxonomy) && $current_object->taxonomy == Prodigy::get_prodigy_category_type() ) {
                $attributes_value_all = $wpdb->get_results( $wpdb->prepare(
                    "SELECT t1.term_id, t1.attribute_value, 0 as count_products, t1.count_products as real_count_products
                        FROM {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES . " as t1
                        LEFT JOIN {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES . " as t2 ON t2.term_id = t1.term_id
                        LEFT JOIN wp_term_relationships as t3 on t3.object_id = t2.product_id
                        WHERE t1.attribute_id = %d AND t1.count_products > 0 AND t3.term_taxonomy_id = %d",
                    [
                        $attribute->id,
                        $current_object->term_id
                    ]
                ), OBJECT_K );
            } else {
                $attributes_value_all = $wpdb->get_results( $wpdb->prepare(
                    "SELECT term_id, attribute_value, 0 as count_products, count_products as real_count_products  FROM {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES . " WHERE attribute_id = %d AND count_products > 0", [ $attribute->id ] )
                    , OBJECT_K );
            }
            
            $all_attrs = $attributes_value + $attributes_value_all;
            
            /*
             * Amount value in attribute
             */
            $attribute->attributes_value_count = count( $all_attrs );
            $count_v = array_column( $all_attrs, 'count_products' );
            /*
             * Amount value in attribute
             */
            $attribute->attributes_value_count_real = count( array_filter( $count_v ) );
            ///
            
            /*
             * List value in attribute
             */
            $attribute->attributes_value      = array_sum( wp_list_pluck( $all_attrs, 'count_products' ) ) ? $all_attrs : [];
            $attribute->attributes_value_real = array_sum( wp_list_pluck( $all_attrs, 'real_count_products' ) ) ? $all_attrs : [];
        }
        
        return $attributes;
    }
    
    
    /**
     * @return object
     */
    public function get_all_aggregation_products_by_attributes() {
        global $wpdb;
        $attributes = $this->get_attribute_taxonomies();
        foreach ( $attributes as &$attribute ) {
            $attributes_value = $wpdb->get_results( $wpdb->prepare(
                "SELECT *, count_products as real_count_products FROM {$wpdb->prefix}" . self::AGGREGATION_ATTRIBUTES . " WHERE attribute_id = %d AND count_products > 0", [ $attribute->id ] )
            );
            
            $count_v = array_column( $attributes_value, 'count_products' );
            /*
             * Amount value in attribute
             */
            $attribute->attributes_value_count      = count( array_filter( $count_v ) );
            $attribute->attributes_value_count_real = count( array_filter( $count_v ) );
            /*
             * List value in attribute
             */
            $attribute->attributes_value      = array_sum( wp_list_pluck( $attributes_value, 'count_products' ) ) ? $attributes_value : [];
            $attribute->attributes_value_real = array_sum( wp_list_pluck( $attributes_value, 'real_count_products' ) ) ? $attributes_value : [];
        }
        
        return $attributes;
    }
    
    /**
     * @param int $attr_id
     * @param int $term_id
     * @param string $attr_value
     *
     * @return bool|int
     */
    public function set_aggregation_products_by_attributes( int $attr_id, int $term_id, $attr_value = '' ) {
        
        global $wpdb;
        
        $info_aggregation = $this->get_aggregation_products_by_attributes( $attr_id, $term_id );
        
        if ( ! empty( $info_aggregation ) ) {
            return true;
        }
        
        $data = [
            'attribute_id' => $attr_id,
            'term_id'      => $term_id,
        ];
        
        if ( ! empty( $attr_value ) ) {
            $data['attribute_value'] = $attr_value;
        }
        
        return $wpdb->insert( $wpdb->prefix . Prodigy_Product_Attributes::AGGREGATION_ATTRIBUTES, $data, [
            '%d',
            '%d',
            '%s'
        ] );
    }
    
    /**
     * @param int $post_id
     *
     * @return bool
     */
    public function delete_aggregation_products_by_attribute( int $post_id ) {
        
        $all_product_attributes = $this->get_product_attributes_ids( $post_id );
        
        if ( empty( $all_product_attributes ) ) {
            return false;
        }
        
        foreach ( $all_product_attributes as $product_attribute ) {
            $this->increase_aggregation_products_by_attributes( $product_attribute['attribute_id'], $product_attribute['term_id'], self::DECREMENT );
        }
        
        return true;
    }
    
    
    /**
     * @param int $attr_id
     * @param int $term_id
     *
     * @param int $count_products
     *
     * @return bool|false|int
     */
    public function update_aggregation_products_by_attributes( int $attr_id, int $term_id, int $count_products ) {
        
        global $wpdb;
        
        $info_aggregation = $this->get_aggregation_products_by_attributes( $attr_id, $term_id );
        
        if ( empty( $info_aggregation ) ) {
            return false;
        }
        
        $data = [
            'count_products' => $count_products,
        ];
        
        $where = [
            'id' => $info_aggregation->id
        ];
        
        return $wpdb->update( $wpdb->prefix . Prodigy_Product_Attributes::AGGREGATION_ATTRIBUTES, $data, $where, [ '%d' ], [ '%d' ] );
    }
    
    /**
     * @param int $product_id
     *
     * @return object|bool
     */
    public function get_attributes_value_by_product( int $product_id ) {
        global $wpdb;
        
        $result = $wpdb->get_results(
            $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES . " WHERE product_id = %d ",
                [ $product_id ]
            )
        );
        $list   = [];
        
        if ( empty( $result ) ) {
            return false;
        }
        
        foreach ( $result as $res ) {
            $list[ $res->attribute_id ][] = $res;
        }
        
        return $list;
    }
    
    /**
     * @param int $attr_id
     * @param int $term_id
     * @param int $increase (1 / -1 and etc)
     *
     * @return bool|false|int
     */
    public function increase_aggregation_products_by_attributes( $attr_id, $term_id, $increase = self::INCREMENT ) {
        
        $info_aggregation = $this->get_aggregation_products_by_attributes( $attr_id, $term_id );
        
        if ( empty( $info_aggregation ) ) {
            $this->set_aggregation_products_by_attributes( $attr_id, $term_id );
            $info_aggregation = $this->get_aggregation_products_by_attributes( $attr_id, $term_id );
        }
        
        $update = false;
        if ( ! empty( $info_aggregation ) ) {
            
            if ( $info_aggregation->count_products <= 0 && $increase < 0 ) {
                return $update;
            }
            
            $sum    = $info_aggregation->count_products + $increase;
            $update = $this->update_aggregation_products_by_attributes( $attr_id, $term_id, $sum );
        }
        
        return $update;
    }
    
    /**
     * @param string $name
     * @param string $slug
     * @param int $order
     * @param array $values
     *
     * @return array
     */
    public function save( $name, $slug, array $values, $order = null ) {
        
        global $wpdb;
        
        $name = filter_var( trim( $name ), FILTER_SANITIZE_STRIPPED );

        $data = [
            'slug'     => $slug,
            'name'     => $name,
            'public'   => true,
            'order_by' => empty( $order ) ? 0 : $order
        ];
    
        $current_attribute = $this->get_attribute_taxonomies_by_name( $slug );
        
        if ( ! empty( $current_attribute ) ) {
            return [
                'result' => false,
                'error'  => 'Duplicate entry'
            ];
        }
        
        $result = $wpdb->insert( $wpdb->prefix . Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_TABLE, $data, [
            '%s',
            '%s',
            '%d',
            '%d'
        ] );
        
        if ( ! empty( $wpdb->last_error ) ) {
            return $response = [
                'result' => $result,
                'error'  => $wpdb->last_error
            ];
        }
        
        $current_attribute = $this->get_attribute_taxonomies_by_name( $slug );
    
        /*
         * set option widgets show attributes
         */
        $info_widget = get_option( 'widget_filters_prodigy_widget' );
    
        if (
            isset( $info_widget )
            && is_array( $info_widget )
            && empty( get_option( 'prodigy_widget_change_default_state_filter' ) )
            && ! empty( is_active_widget( false, false, 'filters_prodigy_widget', false ) )
        ) {
            foreach ( $info_widget as $key_widget => &$value_widget ) {
                if (
                    isset( $value_widget['list_attributes'] )
                    && is_array( $value_widget['list_attributes'] )
                ) {
                    array_push( $value_widget['list_attributes'], $current_attribute[0]->id );
                }
            }
            update_option( 'widget_filters_prodigy_widget', $info_widget, false );
        }
        
        $args = [
            'hierarchical'      => false,
            'show_ui'           => false,
            'show_admin_column' => false,
            'query_var'         => true,
        ];
        register_taxonomy( $slug, [ Prodigy::get_prodigy_product_type() ], $args );
        
        $values = array_unique( $values );
        
        foreach ( $values as $term ) {
            
            $term = filter_var( trim( $term ), FILTER_SANITIZE_STRIPPED );
            
            $result = wp_insert_term( $term, $slug, [ 'slug' => $term ] );
            
            /*
             * adding attribute relation to aggregation attribute-products
             */
            $this->set_aggregation_products_by_attributes( $current_attribute[0]->id, $result['term_id'], $term );
        }
        
        return $response = [
            'result'            => (bool) $result,
            'current_attribute' => $current_attribute[0],
            'error'             => $wpdb->last_error
        ];
    }
    
    /**
     * @param int $post_id
     * @param int $attribute_id
     *
     * @param array $values_attribute
     *
     * @return boolean
     */
    public function assign_product_to_attribute( int $post_id, int $attribute_id, array $values_attribute ) {
        
        global $wpdb;
        
        $data = [
            'product_id'   => $post_id,
            'attribute_id' => $attribute_id
        ];

        $old_attrs = [];
        if ( !empty($this->old_attributes) && isset($this->old_attributes[ $attribute_id ]) ) {
            $old_attrs = wp_list_pluck( $this->old_attributes[ $attribute_id ], 'term_id' );
        }


        $result = false;
        
        foreach ( $values_attribute as $value_attr ) {
            $data['term_id'] = $value_attr;
            
            /*
             * increment new attr values in wp_prodigy_attributes_aggregation
             * if there is not relation that add 1 to aggregation
             */
            if ( ! in_array( $value_attr, $old_attrs ) ) {
                $this->increase_aggregation_products_by_attributes( $attribute_id, $value_attr, self::INCREMENT );
            }
            
            $result = $wpdb->insert( $wpdb->prefix . self::PRODUCT_ATTRIBUTES, $data, [ '%d', '%d', '%d' ] );
        }
        
        /*
         * decrement old attr values in wp_prodigy_attributes_aggregation
         */
        foreach ( $old_attrs as $old_attr ) {
            if ( ! in_array( $old_attr, $values_attribute ) ) {
                $this->increase_aggregation_products_by_attributes( $attribute_id, $old_attr, self::DECREMENT );
            }
        }
        
        return $result;
    }
    
    /**
     * @param int $product_id
     *
     * @return bool
     */
    public function delete_all_attributes_from_product( $product_id ) {
        global $wpdb;
        
        $result = $wpdb->get_results( $wpdb->prepare(
            "DELETE FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES . " WHERE product_id = %d",
            [ $product_id ]
        ));
        
        return ! empty( $result );
    }
    
    /**
     * @param int $post_id
     * @param array $attribute_values
     */
    public function delete_relations_taxonomies_with_product( $post_id, $attribute_values ) {
        global $wpdb;
        
        if ( ! empty( $attribute_values ) ) {
            foreach ( $attribute_values as $key => $attribute ) {
                wp_delete_object_term_relationships( $post_id, $attribute['attribute'][ key( $attribute['attribute'] ) ] );
            }
        }
    }
    
    /**
     * @param int $product_id
     *
     * @return array
     */
    public function get_product_attributes_ids( $product_id ) {
        global $wpdb;
        
        $attributes = $wpdb->get_results( $wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES . " WHERE product_id = %d",
            [ $product_id ]
        ), ARRAY_A );
        
        return $attributes;
    }
    
    /**
     * @param int $attribute_id
     *
     * @return object|null
     */
    public static function get_count_products_from_attribute_value( $attribute_id ) {
        
        global $wpdb;
        
        $result = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT product_id, attribute_id, count(*) AS count_value 
                       FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES . " 
                       WHERE attribute_id = %d 
                       GROUP BY product_id",
                [ $attribute_id ]
            )
        );
        
        return $result;
    }
    
    /**
     * @param int $attribute_id
     *
     * @return object|null
     */
    public static function get_id_products_from_attribute_value( int $attribute_id ) {
        
        global $wpdb;
        
        $result = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT product_id 
                       FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTES . " 
                       WHERE attribute_id = %d 
                       GROUP BY product_id",
                [ $attribute_id ]
            ),
            ARRAY_A
        );
        
        return $result;
    }
    
    /**
     * @param $attribute_id
     *
     * @return int
     */
    public static function get_count_variants_from_attribute( int $attribute_id ) {
        $products = self::get_id_products_from_attribute_value( $attribute_id );
        $counts   = 0;
        
        if ( is_array( $products ) ) {
            foreach ( $products as $product ) {
                $is_use_option = get_post_meta( $product['product_id'], '_use_option', true );
                if ( ! empty( $is_use_option ) ) {
                    $counts += 1;
                }
            }
        }
        
        return $counts;
    }
    
    /**
     * @param int $attribute_id
     *
     * @return int
     */
    public static function get_count_products_from_attribute( $attribute_id ) {
        
        $result = self::get_count_products_from_attribute_value( $attribute_id );
        
        $count = 0;
        if ( ! empty( $result ) ) {
            $count = count( $result );
        }
        
        return $count;
    }
    
    /**
     * @param $product_id
     *
     * @return boolean
     */
    public function delete_product_attribute_relation( $product_id ) {
        global $wpdb;
        
        return $wpdb->delete(
            $wpdb->prefix . self::PRODUCT_ATTRIBUTES,
            [ 'product_id' => $product_id ],
            [ '%d' ]
        );
    }
    
    /**
     * @param string $slug
     *
     * @return bool
     */
    public function delete_attribute_by_slug( $slug ) {
        global $wpdb;
        $is_delete           = $wpdb->delete( $wpdb->prefix . self::PRODUCT_ATTRIBUTE_TABLE, [ 'slug' => $slug ], ['%s'] );
        $is_delete_attribute = false;
        if ( $is_delete ) {
            $is_delete_attribute = $this->delete_terms_attribute( $slug, self::DELETE_AGGREGATION_ATTRIBUTES );
        }
        
        return $is_delete_attribute;
    }
    
    
    /**
     * @param string $slug
     *
     * @return array
     */
    public function delete_relation_attr_by_slug_for_products( string $slug ) {
        
        $result = [];
        
        $info = $this->get_info_prodigy_attr_taxonomy_by_slug( $slug );
        $ids  = $this->get_id_products_from_attribute_value( $info->id );
        
        if ( is_array( $ids ) ) {
            $ids    = wp_list_pluck( $ids, 'product_id' );
            $result = $this->delete_post_meta_attributes( $ids );
        }
        
        return $result;
    }
    
    /**
     * @param array $posts_id
     *
     * @return array
     */
    public function delete_post_meta_attributes( array $posts_id ) {
        $result = [];
        
        foreach ( $posts_id as $post_id ) {
            $result[] = delete_post_meta( $post_id, Prodigy_Product_Attributes::PRODUCT_ATTRIBUTE_META );
        }
        
        return $result;
    }
    
    /**
     * @param string $slug
     *
     * @param bool $delete_aggregation
     *
     * @return bool
     */
    public function delete_terms_attribute( $slug, $delete_aggregation = false ) {
        global $wpdb;
        
        $result = true;
        
        $current_terms = $wpdb->get_results( $wpdb->prepare(
            "select * from wp_term_taxonomy where taxonomy='%s'",
            [$slug]
        ));
        
        if ( is_array( $current_terms ) ) {
            $result = false;
            foreach ( $current_terms as $term ) {
                wp_delete_term( $term->term_id, $slug );
                
                if ( $delete_aggregation ) {
                    $wpdb->delete( $wpdb->prefix . Prodigy_Product_Attributes::AGGREGATION_ATTRIBUTES, [ 'term_id' => $term->term_id ], ['%d'] );
                }
                
            }
        }
        
        return $result;
    }
    
    /**
     * Check attribute for existence
     *
     * @param string $slug
     *
     * @return bool
     */
    public static function is_empty_attribute( $slug ) {
        global $wpdb;
        
        $attribute_taxonomies = $wpdb->get_results( $wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}" . self::PRODUCT_ATTRIBUTE_TABLE . " where slug = '%s'",
            [ $slug ]
        ));
        
        return empty( $attribute_taxonomies );
    }
    
}
