<?php

/**
 * Prodigy product class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Product
 */
class Prodigy_Product {

    // stock statuses
    const STOCK_STATUS_IN = 'In stock';
    const STOCK_STATUS_OUT = 'Out of Stock';

    const STOCK_STATUS_IN_API = 'in_stock';
    const STOCK_STATUS_OUT_API = 'out_of_stock';

    const STOCK_STATUS_ON = 'On backorder';

    // Allow Backorders status
    const BACKORDER_DO_NOT_ALLOW = 'Do not Allow';
    const BACKORDER_ALLOW = 'Allow';

    // Weight type
    const WEIGHT_LBS = 'lb';
    const WEIGHT_OZ = 'oz';
    const WEIGHT_KG = 'kg';
    const WEIGHT_G = 'g';

    // Product shipping dimensions types
    const DIMENSION_IN = 'in';
    const DIMENSION_CM = 'cm';

    const PRODUCT_ATTRIBUTE_PREFIX = 'pg_';

    const PRODUCT_RELATED_UPSELL_META = '_pg_upsell_products';
    const PRODUCT_RELATED_CROSSELL_META = '_pg_crossell_products';

    public static $weight_type_mapper = [
        0 => self::WEIGHT_LBS,
        1 => self::WEIGHT_G,
        2 => self::WEIGHT_OZ ,
        3 => self::WEIGHT_KG ,
    ];

    public static $dimension_type_mapper = [
        0 => self::DIMENSION_CM,
        1 => self::DIMENSION_IN
    ];

    public static $backorder_type_mapper = [
        0 => self::BACKORDER_DO_NOT_ALLOW,
        1 => self::BACKORDER_ALLOW
    ];

    public static $stock_status_mapper = [
        0 => self::STOCK_STATUS_IN,
        1 => self::STOCK_STATUS_OUT
    ];
    public static $stock_status_mapper_for_api = [
        0 => self::STOCK_STATUS_IN_API,
        1 => self::STOCK_STATUS_OUT_API
    ];

    /**
     * Prodigy_Base constructor.
     */
    public function __construct() {
        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @param string $search
     * @param int $current_product_id
     *
     * @return array
     */
    public function get_product_from_autocomplete( $search, $current_product_id = 0 ) {
        $search = esc_sql($search);
        $type_product = Prodigy::get_prodigy_product_type();
        $sql = "SELECT product.post_title from {$this->wpdb->posts} as product 
                WHERE post_title like '%{$search}%' and post_type='{$type_product}'
                and post_status='publish'
               ";

        if ( !empty($current_product_id) ) {
            $sql .= " and ID<>{$current_product_id}";
        }

        $products = $this->wpdb->get_results( $sql, ARRAY_A );

        return $products;
    }

    /**
     * @param array $names
     *
     * @return array|null
     */
    public function get_product_ids_ny_name( $names ) {
        $product_type = Prodigy::get_prodigy_product_type();
        $result = [];
        if ( isset($names) ) {
            $names = esc_sql($names);
            $sql = "SELECT product.id FROM {$this->wpdb->posts} as product
                WHERE post_title IN ('" . implode("','", $names) . "')
                AND post_type='{$product_type}' AND post_status='publish'
               ";

            $result = $this->wpdb->get_results($sql, ARRAY_A);
        }

        return $result;
    }

    /**
     * @param array $ids
     *
     * @return array
     */
    public function get_product_names_by_ids( $ids ) {
        $product_type = Prodigy::get_prodigy_product_type();
        $result = [];
        if ( !empty($ids) ) {
            $ids = esc_sql($ids);
            $sql = "SELECT product.post_title FROM {$this->wpdb->posts} as product 
                WHERE product.ID IN ('" . implode("','", $ids) . "')
                AND post_type='{$product_type}' AND post_status='publish'
               ";
        }
        $result = $this->wpdb->get_results( $sql, ARRAY_A );

        return $result;
    }

    /**
     * @param string $sku
     *
     * @return array
     */
    public static function get_product_from_sku( $sku ) {
        global $wpdb;

        $results = $wpdb->get_row( $wpdb->prepare(
            "select post_id from {$wpdb->postmeta} where meta_key = 'product_sku' and meta_value = '%s'",
            [ $sku ]
        ), ARRAY_A
        );

        return !empty($results) ? $results['post_id'] : null;
    }


    /**
     * @param int $product_id
     *
     * @return array
     */
    public static function get_product_with_meta_by_id( $product_id ) {

        $related_products['fields'] = (array) get_post( $product_id, false );
        $related_products['meta']   = get_post_meta( $product_id );

        if ( array_key_exists( 'meta', $related_products ) && is_array( $related_products['meta'] ) && array_key_exists( '_thumbnail_id', $related_products['meta'] ) ) {
            $attachment_id                 = $related_products['meta']['_thumbnail_id'][0];
            $related_products['image_src'] = wp_get_attachment_image_src( $attachment_id );
        } else {
            $related_products['image_src'] = '';
        }

        return $related_products;
    }

    /**
     * @param array $meta
     *
     * @return array
     */
    public static function format_meta_array( $meta ) {
        $meta_response = [];
        foreach ( $meta as $key => $array ) {
            $meta_response[$key] = $array[0];
        }

        return $meta_response;
    }

    /**
     * @param string $name
     *
     * @return array
     */
    public function get_product_by_name( $name ) {
        $product_type = Prodigy::get_prodigy_product_type();
        $result = $this->wpdb->get_results( $this->wpdb->prepare(
            "SELECT id FROM {$this->wpdb->posts}
                WHERE post_title = '%s'
                AND post_type='{$product_type}'",
            [ $name ]
        ), ARRAY_A
        );

        return empty($result) ? [] : $result;
    }

    /**
     * @param string $slug
     *
     * @return string
     */
    public function generate_slug( $slug ) {
        $words_array = str_word_count( $slug, 1 );
        $count_words = count( $words_array );
        if ( $count_words > 1 ) {
            $slug = implode( "_", $words_array );
        }

        return $slug;
    }

    /**
     * @param int $product_id
     *
     * @return array
     */
    public static function get_upsell_product_ids( $product_id ) {
        $upsell_products = get_post_meta( $product_id, Prodigy_Product::PRODUCT_RELATED_UPSELL_META );

        if (isset($upsell_products[0])) {
            return maybe_unserialize( $upsell_products[0] );
        }
    }

    /**
     * @param int $product_id
     *
     * @return array
     */
    public static function get_crossell_product_ids( $product_id ) {
        $crossell_products = get_post_meta( $product_id, Prodigy_Product::PRODUCT_RELATED_CROSSELL_META );

        if ( isset( $crossell_products[0] ) ) {
            return maybe_unserialize( $crossell_products[0] );
        }
    }

    /**
     * @param int $post_id
     * @param string $post_type
     *
     * @return bool
     */
    function delete_post_by_type( $post_id, $post_type ) {
        global $wpdb;

        $result = $wpdb->query( $wpdb->prepare( "
           DELETE posts,pt,pm
           FROM {$this->wpdb->posts} as posts
           LEFT JOIN {$this->wpdb->term_relationships} pt ON pt.object_id = posts.ID
           LEFT JOIN {$this->wpdb->postmeta} pm ON pm.post_id = posts.ID
           WHERE posts.post_type = %s and posts.ID = %d
           ", $post_type, $post_id )
        );

        return $result !== false;
    }

    /**
     * @param int $post_id
     * @param string $sku
     *
     * @return bool
     */
    public function is_check_unique_sku( $post_id, $sku ) {
        global $wpdb;

        $type = 'product_sku';
        $post_exists = $wpdb->get_var( $wpdb->prepare( "SELECT * FROM wp_postmeta as meta LEFT JOIN wp_posts as post ON post.ID = meta.post_id WHERE post.ID<>'%d' AND meta.meta_key = '%s' AND meta.meta_value = '%s' ", [ $post_id, $type, $sku ] ) );

        return $post_exists;
    }

}
