<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Uninstall_Plugin
 */
class Prodigy_Uninstall_Plugin {


    /**
     * Return a list tables.
     *
     * @return array
     */
    public static function get_tables() {
        global $wpdb;

        $tables = [
            "{$wpdb->prefix}prodigy_attribute_taxonomy",
            "{$wpdb->prefix}prodigy_product_attributes",
            "{$wpdb->prefix}prodigy_attributes_aggregation",
            "{$wpdb->prefix}prodigy_remote_cart_items",
            "{$wpdb->prefix}prodigy_cart_items",
            "{$wpdb->prefix}prodigy_orders",
            "{$wpdb->prefix}prodigy_user_info",
            "{$wpdb->prefix}prodigy_batches_sent_out_products",
        ];

        return $tables;
    }

    /**
     * Drop tables.
     *
     * @return void
     */
    public static function drop_tables() {
        global $wpdb;

        $tables = self::get_tables();

        if ( ! is_array( $tables ) ) return;
        $wpdb->query( "SET FOREIGN_KEY_CHECKS=0" );
        foreach ( $tables as $table ) {
            $wpdb->query( "DROP TABLE IF EXISTS {$table}" );
        }
        $wpdb->query( "SET FOREIGN_KEY_CHECKS=1" );
    }

    /**
     * @return bool
     */
    public static function delete_products() {

        $loop = new WP_Query( [ 'post_type' => get_option('pg_slug_product_type'), 'post_status' => 'any' ] );

        if ( $loop->have_posts() ) {
            $posts = $loop->posts;
            foreach ($posts as $post) {
                if ( ! empty( $post->ID ) ) {
                    wp_delete_post( $post->ID, true );
                }
            }
        }

        return true;
    }

    /**
     * clear categories
     */
    public static function delete_categories( ) {

        global $wpdb;

        $result_terms = $wpdb->get_results(
            $wpdb->prepare( "SELECT * FROM {$wpdb->term_taxonomy} WHERE taxonomy = %s",
                [ get_option('pg_slug_category_type') ]
            )
        );

        if ( ! empty( $result_terms ) ) {
            foreach ($result_terms as $result_term) {
                $wpdb->delete($wpdb->terms, ['term_id' => $result_term->term_id], ['%d'] );
                $wpdb->delete($wpdb->term_relationships, ['term_taxonomy_id' => $result_term->term_id], ['%d'] );
                $wpdb->delete($wpdb->termmeta, ['term_id' => $result_term->term_id], ['%d'] );
            }

            $wpdb->delete( $wpdb->term_taxonomy, [ 'taxonomy' => get_option('pg_slug_category_type') ] );
            $wpdb->delete( $wpdb->term_taxonomy, [ 'taxonomy' => get_option('pg_slug_tag_type') ] );
        }
    }

    /**
     * @param $term
     */
    public static function delete_prodigy_terms_by_term( $term ) {

        global $wpdb;

        $result_terms = $wpdb->get_results(
            $wpdb->prepare( "SELECT * FROM {$wpdb->term_taxonomy} WHERE taxonomy = %s",
                [ $term ]
            )
        );

        if ( ! empty( $result_terms ) ) {
            foreach ($result_terms as $result_term) {
                $wpdb->delete($wpdb->terms, ['term_id' => $result_term->term_id], ['%d']);
                $wpdb->delete($wpdb->term_relationships, ['term_taxonomy_id' => $result_term->term_id], ['%d']);
                $wpdb->delete($wpdb->termmeta, ['term_id' => $result_term->term_id], ['%d']);
            }

            $wpdb->delete( $wpdb->term_taxonomy, [ 'taxonomy' => get_option('pg_slug_category_type') ] );
        }

        unregister_taxonomy( $term );
    }

    /**
     *
     */
    public static function delete_attributes_terms() {
        global $wpdb;

        $result = $wpdb->get_results(
            "SELECT * FROM {$wpdb->prefix}prodigy_attribute_taxonomy"
        );

        if (!empty($result)) {
            foreach ($result as $attr) {
                self::delete_prodigy_terms_by_term($attr->slug);
            }
        }
    }

    /**
     * clear prodigy pages
     */
    public static function delete_pages () {

        $prodigy_pages = [
            'cart',
            'thank-you',
            'shop'
        ];

        foreach ($prodigy_pages as $prodigy_slug ) {
            $args = [
                'name'        => $prodigy_slug,
                'post_type'   => 'page',
                'numberposts' => 1
            ];
            $info_post = get_posts($args);
            wp_trash_post($info_post[0]->ID);
        }

    }

}

return new Prodigy_Uninstall_Plugin();