<?php
/**
 * Setup Wizard Class
 * Takes new users through some basic steps to setup their store.
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Prodigy_Wizard class.
 */
class Prodigy_Wizard {
    
    /**
     * @var array
     */
    private static $weights = [
        'lbs',
        'kg',
        'g',
        'oz',
    ];
    
    /**
     * @var array
     */
    private static $dimensions = [
        'in',
        'cm',
    ];
    
    /**
     * @var array
     */
    private static $name_settings = [
        'pg_weight',
        'pg_dimensions'
    ];
    
    public function __construct() {
        add_action( 'wp_ajax_remove_upload_image', [ $this, 'ajax_remove_upload_image' ] );
        add_action( 'wp_ajax_upload_logo_image', [ $this, 'ajax_upload_logo_image' ] );
        add_action( 'wp_ajax_update_settings_wizard', [ $this, 'update_settings_wizard' ] );
        add_action( 'wp_ajax_remove-indicator-sync-content', [ $this, 'remove_indicator_sync_content' ] );
        add_action( 'wp_ajax_set-indicator-sync-content', [ $this, 'set_indicator_sync_content' ] );
    }
    
    /**
     * set option sync content
     */
    public function remove_indicator_sync_content() {
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        update_option( 'pg_indicator_sync_content', 'no' );
        wp_send_json_success();
    }
    
    /**
     * set option sync content
     */
    public function set_indicator_sync_content() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        update_option( 'pg_indicator_sync_content', 'yes' );
        wp_send_json_success();
    }
    
    /**
     * @return array
     */
    public static function getDimensions(): array {
        return self::$dimensions;
    }
    
    /**
     * @return array
     */
    public static function getWeights(): array {
        return self::$weights;
    }
    
    /**
     * @return array
     */
    public static function getNameSettings(): array {
        return self::$name_settings;
    }
    
    /**
     * ajax remove logo from wizard (step 3)
     */
    public function ajax_remove_upload_image() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        if ( get_option( 'pg_image_logo' ) == (int) $_REQUEST['pg_image_id'] ) {
            $res = delete_option( 'pg_image_logo' );
            
            if ( empty( $res ) ) {
                wp_send_json_error();
            }
        }
        
        wp_send_json_success();
    }
    
    /**
     * ajax update settings on step 2 (step 2)
     */
    public function update_settings_wizard() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        $settings_name = filter_var( $_REQUEST['settings_name'], FILTER_SANITIZE_STRING );
        $settings_val  = filter_var( $_REQUEST['settings_val'], FILTER_SANITIZE_STRING );
        
        if ( in_array( $settings_name, self::getNameSettings() )
             && ( in_array( $settings_val, self::getDimensions() ) || in_array( $settings_val, self::getWeights() ) )
        ) {
            $res = update_option( $settings_name, $settings_val );
            
            if ( empty( $res ) ) {
                wp_send_json_error();
            }
        }
        wp_send_json_success();
    }
    
    
    /**
     * ajax remove logo from wizard (step 3)
     */
    public function ajax_upload_logo_image() {
    
        check_ajax_referer( 'prodigyajax-nonce', 'nonce_code' );
    
        if( ! current_user_can('administrator') )
            wp_send_json_error( 'You don\'t have access', 403 );
        
        if ( ! empty( (int) $_REQUEST['pg_image_id'] ) ) {
            $res = update_option( 'pg_image_logo', (int) $_REQUEST['pg_image_id'] );
            
            if ( empty( $res ) ) {
                wp_send_json_error();
            }
        }
        wp_send_json_success();
    }
    
}

new Prodigy_Wizard();
