<?php

defined( 'ABSPATH' ) || exit;

/**
 * Class ProdigyMigration106
 */
class ProdigyMigration107 implements InterfaceProdigyMigration {
    /**
     * @var wpdb
     */
    private $db;

    /**
     * ProdigyMigration constructor.
     */
    public function __construct() {
        global $wpdb;
        $this->db = $wpdb;
    }

    /**
     * @return bool
     */
    public function migration(): bool {

        $result_migration = false;

        $column1 = 'deleted_at';
        $column2 = 'order_token';
        $new_table_name = 'prodigy_batches_sent_out_products';

        $result1 = $result2 = $result3 = false;

        $add_column1 = $this->db->get_var( "SHOW COLUMNS FROM `{$this->db->prefix}prodigy_orders` LIKE '{$column1}';" );
        if ( empty( $add_column1 ) ) {

            $result1 = $this->db->query( "ALTER TABLE {$this->db->prefix}prodigy_orders add `{$column1}` TIMESTAMP NULL default null;" );
        }

        $add_column2 = $this->db->get_var( "SHOW COLUMNS FROM `{$this->db->prefix}prodigy_cart_items` LIKE '{$column2}';" );
        if ( empty( $add_column2 ) ) {
            $result2 = $this->db->query( "ALTER TABLE {$this->db->prefix}prodigy_cart_items add `{$column2}` VARCHAR(255) default null;" );
        }


        $table = $this->db->get_var( "SHOW TABLES LIKE '{$this->db->prefix}{$new_table_name}' " );

        if ( empty($table) ) {
            $result3 = $this->db->query(
                "CREATE TABLE {$this->db->prefix}{$new_table_name} (
                        id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                         local_batch_id varchar(55) NOT NULL,
                         remote_batch_id int(11) NOT NULL,
                         product_id int(11) NOT NULL,
                         remote_product_id int(11) NOT NULL,
                         completed tinyint(1) DEFAULT 0,
                         error text,
                         created_at timestamp DEFAULT CURRENT_TIMESTAMP,
                         updated_at timestamp DEFAULT CURRENT_TIMESTAMP
                    )ENGINE=InnoDB DEFAULT CHARSET=utf8"
            );
        }

        if (  $result1 && $result2 && $result3 ) {
            $result_migration = true;

        } else {
            $added_column1 = $this->db->get_var( "SHOW COLUMNS FROM `{$this->db->prefix}prodigy_orders` LIKE '{$column1}';" );
            if ( $added_column1 ) {
                $this->db->query( "ALTER TABLE {$this->db->prefix}prodigy_orders DROP `{$column1}`;" );
            }

            $added_column2 = $this->db->get_var( "SHOW COLUMNS FROM `{$this->db->prefix}prodigy_cart_items` LIKE '{$column2}';" );
            if ( $added_column2 ) {
                $this->db->query( "ALTER TABLE {$this->db->prefix}prodigy_cart_items DROP `{$column2}`;" );
            }

            $table = $this->db->get_var( "SHOW TABLES LIKE '{$this->db->prefix}{$new_table_name}' " );

            if ( !empty($table) ) {
                $this->db->query( "DROP TABLE {$this->db->prefix}{$new_table_name}" );
            }
        }

        $this->db->query("ALTER DATABASE `{$this->db->dbname}` CHARACTER SET utf8 COLLATE utf8_general_ci");

        return (bool) $result_migration;
    }

    /**
     * @return array
     */
    public function getInfo(): array {

        return [
            'prodigy_orders' => [
                'deleted_at'
            ],
            'prodigy_cart_items' => [
                'order_token'
            ],
            'prodigy_batches_sent_out_products' => [
                'local_batch_id',
                'remote_batch_id',
                'product_id',
                'remote_product_id',
                'completed',
                'error text',
                'created_at timestamp',
                'updated_at timestamp'
            ]
        ];
    }

    /**
     * @return bool
     */
    public function validate(): bool {
        $result1 = (bool) $this->db->get_var( "SHOW COLUMNS FROM `{$this->db->prefix}prodigy_orders` LIKE 'deleted_at';" );
        $result2 = (bool) $this->db->get_var( "SHOW COLUMNS FROM `{$this->db->prefix}prodigy_cart_items` LIKE 'order_token';" );
        $result3 = (bool) $this->db->get_var( "SHOW TABLES LIKE `{$this->db->prefix}prodigy_batches_sent_out_products` " );
        return ($result1 && $result2 && $result3);
    }

}