<?php

defined( 'ABSPATH' ) || exit;

/**
 * Class ProdigyUpdaterMigration
 */
class ProdigyUpdaterMigration {
    
    /**
     * @var InterfaceProdigyMigration
     */
    private $migration;
    
    /**
     * name of migration, prodigy_migration_106, prodigy_migration_107 etc.
     */
    const NAME_MIGRATION = 'prodigy_migration_';
    
    private static $migration_list =  [
        self::NAME_MIGRATION . 107 => true,
    ];
    
    /**
     * ProdigyUpdaterMigration constructor.
     *
     */
    public function __construct() {
        $currentMigration = 'ProdigyMigration'.self::getNumberMigration();
        $this->migration = new $currentMigration();
    }
    
    /**
     * @param InterfaceProdigyMigration $migration
     */
    public function setMigration(InterfaceProdigyMigration $migration)
    {
        $this->migration = $migration;
    }
    
    /**
     * @return bool
     */
    public static function checkCurrentVersionMigration() {
        
        if ( empty( self::isMigratableVersion() ) ) {
            return false;
        }
        
        if ( get_option( self::getCurrentOptionNameMigration() ) ) {
            return false;
        }
        
        return true;
    }
    
    /**
     * @return string|string[]
     */
    public static function getNumberMigration() {
        return str_replace( '.', '', PRODIGY_VERSION );
    }
    
    /**
     * @return string
     */
    public static function getCurrentOptionNameMigration() {
        return self::NAME_MIGRATION . self::getNumberMigration();
    }
    
    /**
     * @return bool|mixed
     */
    public function executeMigration() {
        if ( ! $this->validateMigration() ) {
            if ( $this->migration->migration() ) {
                return (bool) update_option(self::NAME_MIGRATION . self::getNumberMigration(), true);
            }
            update_option(self::NAME_MIGRATION . self::getNumberMigration(), false);
            return false;
        } else {
            update_option(self::NAME_MIGRATION . self::getNumberMigration(), true);
        }
        return false;
    }
    
    /**
     * @return mixed
     */
    private function validateMigration() {
        return $this->migration->validate();
    }
    
    /**
     * @return array
     */
    public function getInfoCurrentMigration()
    {
        return $this->migration->getInfo();
    }
    
    /**
     * @param bool $ver
     *
     * @return bool
     */
    public static function isMigratableVersion( $ver = false ) {
        $migration = self::getCurrentOptionNameMigration();
        
        if ( ! empty( $ver ) ) {
            $migration = $ver;
        }
        
        if ( ! array_key_exists( $migration, self::$migration_list ) ) {
            return false;
        }
        
        return self::$migration_list[ $migration ];
    }
    
}