<?php

/**
 * Prodigy common widget class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

defined( 'ABSPATH' ) || exit;

/**
 * Class Prodigy_Template_Loader
 */
class Prodigy_Template_Loader {

    public static $is_prodigy_theme_enable = false;

    public static function init() {
        self::$is_prodigy_theme_enable = current_theme_supports( 'prodigy' );
        add_filter( 'template_include', [ __CLASS__, 'template_prodigy_loader' ] );
        add_filter( 'comments_template', [ __CLASS__, 'prodigy_comments_template_loader' ] );
    }

    /**
     * Load comments template.
     *
     * @param string $template template to load.
     *
     * @return string
     */
    public static function prodigy_comments_template_loader( $template ) {
        if ( get_post_type() !== Prodigy::get_prodigy_product_type() ) {
            return $template;
        }

        $check_dirs = array(
            trailingslashit( get_stylesheet_directory() ) . PRODIGY_PLUGIN_DIRECTORY_NAME . '/',
            trailingslashit( get_template_directory() ) . PRODIGY_PLUGIN_DIRECTORY_NAME . '/',
            trailingslashit( get_stylesheet_directory() . '/prodigy' ),
            trailingslashit( get_template_directory() ),
            trailingslashit( PRODIGY_PLUGIN_PATH ) . 'templates/partials',
        );

        foreach ( $check_dirs as $dir ) {
            if ( file_exists( trailingslashit( $dir ) . 'single-products-reviews.php' ) ) {
                return trailingslashit( $dir ) . 'single-products-reviews.php';
            }
        }
    }



    /**
     * @param $template
     *
     * @return string
     */
    public static function template_prodigy_loader( $template ) {
        if ( is_embed() ) {
            return $template;
        }

        $default_file = self::get_template_loader_default_file();

        if ( $default_file ) {
            $template = PRODIGY_PLUGIN_PATH . 'templates/partials/' . $default_file;
        }

        return $template;
    }

    /**
     * @return string
     */
    private static function get_template_loader_default_file() {
        global $wp_query;

        $default_file = '';
        $request = $_SERVER['REQUEST_URI'];

        if ( is_singular( Prodigy::get_prodigy_product_type() ) ) {
            $default_file = 'single-product';
        } elseif ( is_product_taxonomy() || is_page( 'shop' ) ) {
            $object = get_queried_object();

            if ( is_tax( Prodigy::get_prodigy_category_type() ) ) {
                $default_file = 'category-prodigy_product_cat';
            } elseif ( is_tax( Prodigy::get_prodigy_tag_type() ) ) {
                $default_file = 'category-prodigy_product_tag';
            }
            else {
                $default_file = '';
            }
        } elseif ( is_post_type_archive( Prodigy::get_prodigy_product_type() ) ) {
            $default_file = self::$is_prodigy_theme_enable ? 'shop' : '';
        } elseif ( is_404() ) {
            $default_file = '404';
        } elseif ( $request === '/shop/' ) {

            if ( parse_url( get_permalink() )['path'] !== '/shop/' ) {
                $page = get_page_by_path( 'shop' );
                wp_redirect( get_permalink( $page->ID ) );
            }

        } elseif ( strpos($request, 'thank-you') ) {

            if ( parse_url( get_permalink() )['path'] != '/thank-you/' ) {
                $page = get_page_by_path( 'thank-you' );
                wp_redirect( get_permalink( $page->ID ) );
            }

            $default_file = self::is_page_is_shop('thank-you');
        } else {
            $default_file = '';
        }

        if ( $default_file == 'shop' ) {
            $page = get_page_by_path( 'shop' );
            if ( prodigy_get_page_id( 'shop' ) != $page->ID ) {
                $default_file = '';
            }
        }

        if ( $default_file == '404' ) {

            $old_info_prod = self::check_slug_product_exists();

            if ( isset( $old_info_prod ) && ! empty( $old_info_prod ) ) {
                wp_safe_redirect( get_permalink( $old_info_prod->ID ) );
            }

            $old_info_category = self::check_slug_category_exists();

            if ( isset( $old_info_category ) && ! empty( $old_info_category ) ) {
                wp_safe_redirect( get_category_link( $old_info_category->term_id ) );
            }

            $old_info_tag = self::check_slug_tag_exists();

            if ( isset( $old_info_tag ) && ! empty( $old_info_tag ) ) {
                wp_safe_redirect( get_category_link( $old_info_tag->term_id ) );
            }
        }

        if ( $default_file && $default_file != 'shop' ) {
            $default_file = $default_file . '.php';
        }

        return $default_file;
    }

    /**
     * @param $info_prod
     */
    private static function update_current_product_type( $info_prod ) {

        $my_post['ID']        = $info_prod->ID;
        $my_post['post_type'] = Prodigy::get_prodigy_product_type();

        wp_update_post( wp_slash( $my_post ) );
    }

    /**
     * @return array|bool|WP_Post|null
     */
    public static function check_slug_product_exists() {
        $result = false;

        $req_url       = explode( '/', $_SERVER['REQUEST_URI'] );
        $clear_req_url = array_filter( $req_url );

        $old_types = get_option( Prodigy::PRODIGY_OPTION_NAME_OLD_PRODUCT_TYPES );

        if ( ! empty( $clear_req_url ) && in_array( $clear_req_url[1], $old_types ) ) {
            $result = get_page_by_path( $clear_req_url[2], OBJECT, $clear_req_url[1] );
        }

        return $result;
    }

    /**
     * @return array|bool|false|WP_Term
     */
    public static function check_slug_category_exists() {
        $result = false;

        $req_url       = explode( '/', $_SERVER['REQUEST_URI'] );
        $clear_req_url = array_filter( $req_url );

        $old_types = get_option( Prodigy::PRODIGY_OPTION_NAME_OLD_CATEGORY_TYPES );

        if ( isset($clear_req_url[1], $clear_req_url[2]) && ! empty( $clear_req_url ) && in_array( $clear_req_url[1], $old_types ) ) {
            $result = get_term_by( 'slug', $clear_req_url[2], Prodigy::get_prodigy_category_type() );
        }

        return $result;
    }

    /**
     * @return array|bool|false|WP_Term
     */
    public static function check_slug_tag_exists() {
        $result = false;

        $req_url       = explode( '/', $_SERVER['REQUEST_URI'] );
        $clear_req_url = array_filter( $req_url );

        $old_types = get_option( Prodigy::PRODIGY_OPTION_NAME_OLD_TAG_TYPES );

        if ( ! empty( $clear_req_url ) && in_array( $clear_req_url[1], $old_types ) ) {
            $result = get_term_by( 'slug', $clear_req_url[2], Prodigy::get_prodigy_tag_type() );
        }

        return $result;
    }

    /**
     * @param string $page
     *
     * @return string
     */
    protected static function is_page_is_shop( string $page ) {

        $default_file = $page;

        if (
            ! empty( prodigy_get_page_id( 'shop' ) )
            && $post_info = get_post( prodigy_get_page_id( 'shop' ) )
        ) {
            if ( $post_info->post_name == $page ) {
                $default_file = '';
            }
        }

        return $default_file;
    }
}

add_action( 'init', array( 'Prodigy_Template_Loader', 'init' ) );
