<?php

/**
 * Prodigy product page class
 *
 * @version    1.0.0
 * @package    prodigy/includes/public/pages
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__, 2 ) ) . 'class-prodigy-product-variants.php';


/**
 * Class Prodigy_Product_Page
 */
class Prodigy_Product_Page {

    protected $product_variants;
    const COUNT_COMMENTS_ON_PAGE_PRODUCT = 5;

    /**
     * Prodigy_Product_Page constructor.
     */
    public function __construct() {
        $this->product_variants = new Prodigy_Product_Variants();
        add_action( 'wp_head', [ $this, 'myplugin_ajaxurl'] );

        add_action( 'wp_ajax_prodigy-public-get-variants-data', [ $this, 'prodigy_public_get_variants_data' ] );
        add_action( 'wp_ajax_nopriv_prodigy-public-get-variants-data', [ $this, 'prodigy_public_get_variants_data' ] );

        add_action( 'wp_ajax_prodigy-quick-edit', [ $this, 'prodigy_quick_view_get_product' ] );
        add_action( 'wp_ajax_nopriv_prodigy-quick-edit', [ $this, 'prodigy_quick_view_get_product' ] );

        add_action( 'wp_ajax_prodigy-public-get-product', [ $this, 'prodigy_public_get_product' ] );
        add_action( 'wp_ajax_nopriv_prodigy-public-get-product', [ $this, 'prodigy_public_get_product' ] );

        add_action( 'wp_ajax_prodigy-public-get-available-variants', [ $this, 'prodigy_public_get_available_variants' ] );
        add_action( 'wp_ajax_nopriv_prodigy-public-get-available-variants', [ $this, 'prodigy_public_get_available_variants' ] );
    
        add_action( 'wp_ajax_prodigy-public-get-comments', [ $this, 'prodigy_get_comments' ] );
        add_action( 'wp_ajax_nopriv_prodigy-public-get-comments', [ $this, 'prodigy_get_comments' ] );
    
        add_action( 'wp_ajax_prodigy-public-get-comments-count', [ $this, 'prodigy_get_comments_count' ] );
        add_action( 'wp_ajax_nopriv_prodigy-public-get-comments-count', [ $this, 'prodigy_get_comments_count' ] );
    }
    
    /**
     *
     * @return int
     */
    public function prodigy_get_comments_count() {
        $comm = get_comments(
            [
                'post_id' => (int) isset($_POST['post_id']) ?? 0,
                'count'   => true
            ]
        );
        
        if (!empty($comm)) {
            $count_p =  ceil($comm/self::COUNT_COMMENTS_ON_PAGE_PRODUCT);
            
            if ((int) isset($_POST['page']) ?? 0 > $count_p) {
                $comm = 0;
            }
        }
        
        wp_send_json_success($comm);
    }
    
    /**
     * get pagination comments
     */
    public function prodigy_get_comments() {

        $comments = get_comments(
            [
                'post_id' => (int) $_POST['post_id']
            ]
        );
        
        $page = (int) $_POST['page'];

        $data_i = [
            'callback' => 'prodigy_comments',
            'per_page' => self::COUNT_COMMENTS_ON_PAGE_PRODUCT,
            'page' => $page,
        ];
        
        $data = wp_list_comments(
            $data_i,
            $comments
        );
        
        if ( ! empty( $data ) ) {
            echo $data;
        }
        
        wp_die();
    }

    /**
     *  Get variants
     */
    public function prodigy_public_get_variants_data() {
        
        $variants = $_POST['variants'];
        $post_id = filter_var($_POST['post_id'], FILTER_VALIDATE_INT);
        
        if ( isset( $variants ) && is_array($variants) ) {
            $variant_name            = implode( "|", $variants );
            $variant_name = filter_var($variant_name, FILTER_SANITIZE_STRING);
            
            $variant = $this->product_variants->get_product_variant_by_name(
                $post_id,
                trim( $variant_name )
            );
            $variant_meta            = get_post_meta( $variant->ID );
            $variant_image_id        = get_post_meta( $variant->ID, 'category-image-id' );
            $variant_meta['img_url'] = wp_get_attachment_image_url( $variant_image_id[0], 'full' );


            $is_remote_product_data = get_post_meta( $post_id,
                Prodigy_Aggregate_Data_Synchronize::REMOTE_RELATIONSHIP_META,
                true
            );

            if ( ! empty( $is_remote_product_data['variants'] ) ) {
                foreach ( $is_remote_product_data['variants'] as $item ) {
                    if (isset($item['attributes']['sku'])) {
                        if ( $variant_meta['product_sku'][0] === $item['attributes']['sku'] ) {
                            // when variants
                            $variant_meta['variant_remote_id'][0] = (int) $item['id'];
                        }
                    }
                }
            }
            $variant_meta['variant_local_id'][0] = $variant->ID;
            $variant_meta['categories'] = json_encode($this->get_post_categories($post_id));


            echo wp_json_encode( $variant_meta );
            wp_die();
        }
    }

    /**
     * @param $post_id
     * @return array
     */
    public function get_post_categories($post_id) {
        $categories = get_the_terms( $post_id, Prodigy::get_prodigy_category_type() );

        $product_categories = [];
        if (!empty($categories)) {
            foreach ($categories as $key => $category) {
                $product_categories[$key] = $category->name;
            }
        }

        return $product_categories;
    }

    /**
     * Get main product
     */
    public function prodigy_public_get_product() {
        
        $post_id = filter_var($_POST['post_id'] ?? 0, FILTER_VALIDATE_INT);
        
        if ( isset( $post_id ) ) {
            $product = Prodigy_Product::get_product_with_meta_by_id( $post_id );

            $is_remote_product_data = get_post_meta( $post_id,
                Prodigy_Aggregate_Data_Synchronize::REMOTE_RELATIONSHIP_META,
                true
            );

            if ( ! empty( $is_remote_product_data ) ) {
                // when product
                $product['variant_product_id'] = $is_remote_product_data['master_variant_id'];
            }

            $product['categories'] = json_encode($this->get_post_categories($post_id));


            echo wp_json_encode( ! empty( $product ) ? $product : [] );
            wp_die();
        }
    }

    /**
     *  Get product for quick view modal window
     */
    public function prodigy_quick_view_get_product()
    {
        prodigy_setup_product_data( (int) $_POST['post_id'] );
        // for check page in theme
        $GLOBALS['quick_view'] = true;
        prodigy_get_template( 'single-product/layouts/product.php');
        wp_die();
    }

    /**
     * Get available variants for product_id
     */
    public function prodigy_public_get_available_variants() {
        $variants = $this->get_available_variants( (int) $_POST['post_id'] );

        echo wp_json_encode($variants);
        wp_die();
    }

    /**
     * @param int $parent_product_id
     *
     * @return array
     */
    public function get_available_variants( $parent_product_id ) {
        $product_variants = Prodigy_Product_Variants::get_product_variants( $parent_product_id );

        return $this->check_available_variants($product_variants);
    }

    /**
     * @param array $variants
     *
     * @return array
     */
    public function check_available_variants( $variants ) {
        $available_variants = [];
        foreach ( $variants as $key=>$product_variant ) {
            $variant_meta = get_post_meta( $product_variant['ID'] );
            if (
               !empty($variant_meta['product_regular_price'][0])
            ) {
                if ( isset($variant_meta['post_attributes'][0]) ) {
                    $available_variants[$key] = json_decode(str_replace("'", '"', $variant_meta['post_attributes'][0]), 1);
                }
            }
        }

        return $available_variants;
    }

    /**
     *  Set ajaxurl path
     */
    public function myplugin_ajaxurl() {
        echo '<script type="text/javascript">
           var ajaxurl = "' . admin_url('admin-ajax.php') . '";
         </script>';
    }
}

return new Prodigy_Product_Page();