<?php

/**
 * Prodigy product page class
 *
 * @version    1.0.0
 * @package    prodigy/includes/public/pages
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Product_Page
 */
class Shop_Page {

    const PRODIGY_SHOP_URL = '/shop';
    const ITEMS_PER_PAGE = 9;

    /**
     * Prodigy_All_Products_Page constructor.
     */
    public function __construct() {
        
        $request_uri = filter_var( $_SERVER["REQUEST_URI"], FILTER_SANITIZE_URL);
        
        add_action( 'wp_head', [ $this, 'myplugin_ajaxurl'] );
        $current_uri = parse_url ( $request_uri, PHP_URL_PATH );
        
        if ( $this->is_shop_page_url( $current_uri ) ) {
            add_filter( 'template_include', [ $this, 'prodigy_public_shop' ] );
        }
    }

    /**
     * @param string $url
     *
     * @return bool
     */
    function is_shop_page_url( $url ) {
        $count = 0;
        
        if ( isset( $_GET['page_id'] ) ) {
            $page_id = filter_var($_GET['page_id'], FILTER_VALIDATE_INT);
            $shop = get_the_title( $page_id );
        }
    
        /**
         * display shop on another page if it set
         */
        $post_info = get_post( prodigy_get_page_id( 'shop' ) );
        if ( isset( $post_info ) && isset( $post_info->post_name ) ) {
            $shop_page_url = $post_info->post_name;
        } else {
            $shop_page_url = 'shop';
        }
        
        $needle = [
            $shop_page_url
        ];

        foreach ( $needle as $substring ) {
            $count += substr_count( $url, $substring);
        }
        
        if ( isset($shop) ) {
            if ( strtolower( $shop ) == 'shop' ) $count = 1;
        }

        return (bool) $count;
    }

    /**
     * Get products for filter params
     */
    public function prodigy_public_shop() {
        global  $wp_query;
        
        /*
         * setting widget filters (attributes multi-select)
         */
        $query_type_attr = get_query_type_attr();
        
        $paged = ( get_query_var( 'paged' ) ) ? get_query_var( 'paged' ) : 1;
        $args = [
            'post_type' => Prodigy::get_prodigy_product_type(),
            'post_status' => 'publish',
            // get global setting
            'posts_per_page' => self::ITEMS_PER_PAGE,
            'paged' => $paged
        ];

        if ( isset( $_GET['search'] ) && !empty( $_GET['search'] ) && sanitize_text_field( $_GET['search'] ) ) {
            /*
             * Search by all symbols
             */
            $args['s'] = filter_var($_GET['search'], FILTER_SANITIZE_STRIPPED);
        }

        if ( isset( $_GET['sort'] ) && sanitize_text_field( $_GET['sort'] ) ) {
            $args = $this->add_query_sort( filter_var($_GET['sort'], FILTER_SANITIZE_STRIPPED), $args );
        }

        if ( isset( $wp_query->queried_object->name ) && sanitize_text_field( $wp_query->queried_object->name ) ) {
            $args['tax_query'] = [
                'relation' => 'AND',
                [
                    'taxonomy' => $wp_query->queried_object->taxonomy,
                    'field' => 'name',
                    'terms' => $wp_query->queried_object->name,
                ],
            ];
        }

        if ( isset( $_GET['attr'] ) && is_array($_GET['attr']) ) {
            foreach ( $_GET['attr'] as $attr => $values ) {
                if ( $attr ) {
                    $terms = explode( ';', sanitize_text_field($values) );
                    foreach ( $terms as $key=>$term ) {
                        $valid_terms = [];
                        if ( $term ) {
                            $valid_terms[$key] = sanitize_text_field( $term );
                        }
    
                        if ( isset($args['tax_query']) ) {
                            $array = [
                                'taxonomy' => sanitize_text_field($attr),
                                'field' => 'name',
                                'terms' => $valid_terms,
                            ];
                            array_push( $args['tax_query'], $array );
                        } else {
                            $args['tax_query'] = [
                                'relation' => $query_type_attr,
                                [
                                    'taxonomy' => $attr,
                                    'field' => 'name',
                                    'terms' => $valid_terms,
                                ]
                            ];
                        }
                    }
                }
            }
        }

        if ( isset( $_GET['price_min'] ) && $_GET['price_min'] && isset( $_GET['price_max'] ) && $_GET['price_max'] ) {
            $price_max = filter_var( $_GET['price_max'], FILTER_VALIDATE_FLOAT );
            $price_min = filter_var( $_GET['price_min'], FILTER_VALIDATE_FLOAT );
            if ( $price_max && $price_min ) {

                $args['meta_query'][] = [
                    'key' => 'product_sort_price',
                    'value' => [ $price_min, $price_max ],
                    'type' => 'numeric',
                    'compare' => 'between'
                ];

            }
        } else {
            if ( isset( $_GET['price_min'] ) && $_GET['price_min'] ) {
                $price_min = filter_var( $_GET['price_min'], FILTER_VALIDATE_FLOAT );
                if ( $price_min ) {
                    $args['meta_query'][] = [
                        'key' => 'product_sort_price',
                        'value' => $price_min,
                        'type' => 'numeric',
                        'compare' => '>='
                    ];
                }
            }

            if ( isset( $_GET['price_max'] ) && $_GET['price_max'] ) {
                $price_max = filter_var( $_GET['price_max'], FILTER_VALIDATE_FLOAT );
                if ( $price_max ) {
                    $args['meta_query'][] = [
                        'key' => 'product_sort_price',
                        'value' => $price_max,
                        'type' => 'numeric',
                        'compare' => '<='
                    ];
                }
            }
        }
        query_posts( $args );

        return prodigy_locate_template( 'category-prodigy_product_cat.php' );
    }

    /**
     * @param string $sort
     * @param array $args
     *
     * @return array
     */
    public function add_query_sort( $sort, $args ) {
        switch ( $sort ) {
           case 'date':
               $query = [
                   'orderby' => 'date',
                   'post_date' => 'DESC'
               ];
               break;
           case 'rating':
               $query = [
                   'meta_key' => 'product_average_rating',
                   'orderby' => 'meta_value_num',
                   'order' => 'DESC'
               ];
               break;
           case 'price-low-high':
               $query = [
                   'orderby' => 'meta_value_num+0',
                   'meta_key' => 'product_sort_price',
                   'order' => 'ASC'
               ];
               break;
           case 'price-high-low':
               $query = [
                   'orderby' => 'meta_value_num+0',
                   'meta_key' => 'product_sort_price',
                   'order' => 'DESC'
               ];
               break;
           default:
               $query = [
                   'orderby' => 'date',
                   'post_date' => 'DESC'
               ];
               break;
       }

       return array_merge( $args, $query );
    }

    /**
     *  Set ajaxurl path
     */
    public function myplugin_ajaxurl() {
        echo '<script type="text/javascript">
           var ajaxurl = "' . admin_url('admin-ajax.php') . '";
         </script>';
    }
}

new Shop_Page();
