<?php

/**
 * Prodigy common shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__ ) ) . '/../../includes/class-prodigy-product-attributes.php';

/**
 * Class Prodigy_Short_Code_Active_Filters
 */
class Prodigy_Short_Code_Active_Filters {
    
    private $product_attributes;
    private $all_attributes;
    
    public function __construct() {
        
        $this->product_attributes = new Prodigy_Product_Attributes();
        
        $attributes = $this->product_attributes->get_attribute_taxonomies();
        if ( ! empty( $attributes ) ) {
            $this->all_attributes = array_column( $attributes, 'slug' );
        }
        
        add_shortcode( 'active-filters-products', [ $this, 'output' ] );
    }
    
    /**
     * php array two dimensional to one
     *
     * @param array $list
     *
     * @return array|bool
     */
    public function array_flatten( array $list ) {
        
        if ( ! is_array( $list ) ) {
            return false;
        }
        
        $result = array();
        foreach ( $list as $key => $value ) {
            if ( is_array( $value ) ) {
                $result = array_merge( $result, $this->array_flatten( $value ) );
            } else {
                $result[ $key ] = $value;
            }
        }
        
        return $result;
    }
    
    /**
     * @return array
     */
    public function get_all_values_attr() {
        
        if ( isset( $_GET['attr'] ) || ( isset( $_GET['price_min'] ) && isset( $_GET['price_max'] ) ) ) {
            $get_list_args = $_GET['attr'];
            
            $result = [];
            
            $arr_price = [ 'price', 'tag' ];
            
            $term_info = get_queried_object();
            
            if ( is_tax( Prodigy::get_prodigy_tag_type() ) ) {
                $get_list_args['tag'] = $term_info->slug;
            }
            
            if ( empty( $get_list_args ) && empty( $_GET['price_min'] ) && empty( $_GET['price_max'] ) ) {
                return [];
            }
            
            if ( isset( $_GET['price_min'] ) and isset( $_GET['price_max'] ) ) {
                $get_list_args['price'] = 'Price $' . sanitize_text_field( prodigy_price_format( $_GET['price_min'] ) ) . ' - $' . sanitize_text_field( prodigy_price_format( $_GET['price_max'] ) );
            }
            
            foreach ( $get_list_args as $arg => $value ) {
                $arg = sanitize_text_field( $arg );
                if ( ! in_array( $arg, $arr_price ) ) {
                    $attribute_values = $this->product_attributes->get_attribute_taxonomies_by_name( $arg );
                    if ( empty( $attribute_values[0]->id ) ) {
                        continue;
                    }
                }
                $value  = sanitize_text_field( $value );
                $values = explode( ';', $value );
                
                foreach ( $values as $value_name ) {
                    $result[ $arg ][] = sanitize_text_field( $value_name );
                }
            }
            
            return $result;
        }
    }
    
    /**
     * @param array $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    public function output( $atts, $content = null, $name ) {
        
        $this->request_attrs = $atts;

        wp_enqueue_script( 'active-filter', plugin_dir_url( PRODIGY_PLUGIN_PATH . '/includes/public/shortcodes/js/active-filter.js' ) . 'active-filter.js', [ 'jquery' ] );
    
        wp_localize_script(
            'active-filter',
            'prodigy_active_filter',
            [
                'current_type_tag' => Prodigy::get_prodigy_tag_type()
            ]
        );
        
        $args_request = $this->get_all_values_attr();
        
        if ( empty( $args_request ) ) {
            return add_query_arg( [], '' );
        }
    
        ob_start();
    
        do_action( 'prodigy_shortcode_template_active_filters', [
            'args_request' => $args_request,
        ] );
        
        wp_reset_postdata();
        
        $content = ob_get_contents();
        ob_clean();
        
        return $content;
    }
}

return new Prodigy_Short_Code_Active_Filters();
