<?php

/**
 * Prodigy shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__ ) ) . '/../../includes/class-prodigy-product-attributes.php';

/**
 * Class Prodigy_Short_Code_Attributes_Filter
 */
class Prodigy_Short_Code_Attributes_Filter {
    
    private $product_attributes;
    private $all_attributes;
    private $get_request = [];
    const COUNT_SHOW_ATTRIBUTE_VALUES = 5;
    const SHOW_ATTRIBUTE_VALUE_BY_COUNT_PRODUCTS = 1;
    private $active_filters = false;
    protected $info_widget = [];
    protected $current_object;
    
    public function __construct() {
        
        if ( isset( $_GET['attr'] ) ) {
            $request_attr = $_GET['attr'];
        }
        
        $this->get_request['attr'] = $request_attr ?? [];
        $this->product_attributes  = new Prodigy_Product_Attributes();
        $attributes                = $this->product_attributes->get_attribute_taxonomies();
        if ( ! empty( $attributes ) ) {
            $this->all_attributes = array_column( $attributes, 'slug' );
        }
        add_shortcode( 'attributes-filter', [ $this, 'output' ] );
        
        $this->info_widget = get_info_widget( 'widget_filters_prodigy_widget' );
    }
    
    /**
     * @return bool|array
     */
    public function get_request_existing_attributes() {
        $result = [];
        
        foreach ( $this->all_attributes as $attribute ) {
            if ( array_key_exists( $attribute, $this->get_request['attr'] ) ) {
                $result[ $attribute ] = $this->get_request['attr'][ $attribute ];
            }
        }
        
        return $result;
    }
    
    /**
     * @param $attribute
     * @param $attr_value
     *
     * @return string
     */
    public function http_build_by_attr( $attribute, $attr_value ) {
        if ( ! in_array( $attribute, $this->all_attributes ) ) {
            return '';
        }
        
        $list['attr'] = $this->get_request_existing_attributes();
        
        if ( array_key_exists( $attribute, $list['attr'] ) ) {
            
            $attr_values = explode( '%3B', $list['attr'][ $attribute ] );
            
            array_push( $attr_values, $attr_value );
            
            $new_attr_values = array_unique( $attr_values );
            
            $list['attr'][ $attribute ] = implode( '%3B', $new_attr_values );
        } else {
            
            if ( in_array( $attribute, $this->all_attributes ) ) {
                $list['attr'][ $attribute ] = $attr_value;
            }
        }
        
        if ( get_query_var( 'taxonomy' ) == Prodigy::get_prodigy_category_type() ) {
            $path_redirect = $_SERVER['REQUEST_URI'];
        } else {
            $path_redirect = home_url( 'shop' );
        }
        
        $list_args = add_query_arg( $list, $path_redirect );
        
        $list_args = str_replace( ';', '%3B', $list_args );
        
        return $list_args;
    }
    
    /**
     * @param $filters
     *
     * @return array
     */
    public function get_output_attrs( $filters ) {
        $output_filter = [];
        
        foreach ( $filters as $filter ):
            if ( empty( $filter->attributes_value_real ) || ! in_array( $filter->id, $this->info_widget['list_attributes'] ) ) {
                continue;
            }
    
            foreach ( $filter->attributes_value_real as $attribute_values ):
    
                if ( $attribute_values->count_products < Prodigy_Short_Code_Attributes_Filter::SHOW_ATTRIBUTE_VALUE_BY_COUNT_PRODUCTS
                     && $attribute_values->real_count_products < Prodigy_Short_Code_Attributes_Filter::SHOW_ATTRIBUTE_VALUE_BY_COUNT_PRODUCTS
                ) {
                    continue;
                }
    
                if ( isset( $this->current_object->taxonomy ) && ! empty( $this->current_object->taxonomy == Prodigy::get_prodigy_category_type() )
                     && $attribute_values->real_count_products < Prodigy_Short_Code_Attributes_Filter::SHOW_ATTRIBUTE_VALUE_BY_COUNT_PRODUCTS
                ) {
                    continue;
                }
                
                $output_filter[$attribute_values->attribute_value] = $this->http_build_by_attr( $filter->slug, $attribute_values->attribute_value );
            
            endforeach;
            
        endforeach;
        
        return $output_filter;
    }
    
    /**
     * @param array $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    public function output( $atts, $content = null, $name ) {
        
        $this->request_attrs = $atts;
        $this->current_object = get_queried_object();
        $filters        = [];
        
        if ( ( empty( $_GET ) && empty( $current_object ) ) || $atts['query_type'] == 'or' ) {
            //all attributes
            $filters = $this->product_attributes->get_all_aggregation_products_by_attributes();
        } else {
            $this->active_filters = true;
            //attributes by products
            $custom_args  = get_agrs_for_filter();
            $custom_query = new WP_Query( $custom_args );
            
            $ids = wp_list_pluck( $custom_query->posts, 'ID' );
            
            if ( ! empty( $ids ) ) {
                $filters = $this->product_attributes->get_aggregation_attr_values( $ids );
            }
        }
        
        ob_start();
    
        do_action( 'prodigy_shortcode_template_attributes_filter', [
            'filters' => $filters,
            'current_object' => $this->current_object,
            'get_request' => $this->get_request['attr'],
            'active_filters' => $this->active_filters,
            'info_widget' => $this->info_widget,
            'filters_output' => $this->get_output_attrs($filters),
        ] );
        
        wp_reset_postdata();
        
        $content = ob_get_contents();
        ob_clean();
        
        return $content;
    }
}

return new Prodigy_Short_Code_Attributes_Filter();
