<?php

/**
 * Prodigy cart shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Short_Code_Cart
 */
class Prodigy_Short_Code_Cart {

    public function __construct() {
        add_shortcode( 'prodigy_cart_widget', [ $this, 'output' ] );
    }

    /**
     * @param array $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    public function output( $atts, $content = null, $name ) {
        $is_prodigy_theme_enable = Prodigy_Template_Loader::$is_prodigy_theme_enable;

        $cart = new Prodigy_Cart();
        $cart_products = [];
        if ( $_COOKIE['prodigy_user'] ) {
            $cart_items  = $cart->get_cart_items_for_user( sanitize_text_field($_COOKIE['prodigy_user']) );
            $order_token = $cart->get_token_for_current_order( sanitize_text_field($_COOKIE['prodigy_user']) );
            $proceed_url = $cart->generate_checkout_url($order_token);

            $subtotal_price = 0;

            $gallery_thumb = array_values( array(
                'width'  => 250,
                'height' => 90,
                'crop'   => 1,
            ) );

            if ( ! empty( $cart_items ) ) {
                foreach ( $cart_items as $item ) {
                    $product                                                  = get_prodigy_product( $item['local_product_id'] );
                    $cart_products[ $item['remote_product_id'] ]              = get_prodigy_product( $item['local_product_id'] );
                    $cart_products[ $item['remote_product_id'] ]->count_items = $item['count'];

                    $sale_price = $product->get_meta_field( 'product_sale_price' );
                    $regular_price = $product->get_meta_field( 'product_regular_price' );

                    $actual_price = $sale_price != "" ? $sale_price : $regular_price;

                    $cart_products[ $item['remote_product_id'] ]->actual_price = $actual_price;

                    $cart_products[ $item['remote_product_id'] ]->total_price = (float) $actual_price * (int) $item['count'];

                    $cur_attributes = json_decode( $item['attributes'], true );
                    if ( ! empty( $cur_attributes ) ) {
                        $attributes = [];
                        if (isset($cur_attributes['item'])) {
                            foreach ( $cur_attributes['item'] as $key => $attribute ) {
                                $attributes[ $cur_attributes['attribute'][ $key ] ] = $attribute;
                            }
                        }

                        $cart_products[ $item['remote_product_id'] ]->item_attributes = $attributes;
                    }
                }
            }
        }
        ob_start();

        do_action( 'prodigy_shortcode_template_cart', [
            'cart_products' => $cart_products,
            'gallery_thumb' => $gallery_thumb,
            'proceed_url'   => $proceed_url,
            'is_prodigy'    => $is_prodigy_theme_enable
        ] );
        
        wp_reset_postdata();

        $content = ob_get_contents();

        ob_clean();

        return $content;
    }
}

return new Prodigy_Short_Code_Cart();
