<?php

/**
 * Prodigy categories shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Short_Code_Cart
 */

class Prodigy_Short_Code_Categories {
    /**
     * Prodigy_Short_Code_Products constructor.
     */
    public function __construct() {
        add_shortcode( 'categories', [ $this, 'output' ] );
    }

    /**
     * @param string $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    function output( $atts, $content = null, $name ) {
        extract( shortcode_atts( array(
            'category_ids' => '',
            'img_ids' => '',
            'img_size' => 'full',
            /** feed, our_categories */
            'type' => ''
        ), $atts));


        if ($type === 'feed') {
            $img_ids      = explode( ",", $img_ids );
            $category_ids = explode( ",", $category_ids );

            $img1 = wp_get_attachment_image_src( (int) $img_ids[0], (string) $img_size );
            $img2 = wp_get_attachment_image_src( (int) $img_ids[1], (string) $img_size );
            $img3 = wp_get_attachment_image_src( (int) $img_ids[2], (string) $img_size );

            $term1 = get_term_by( 'id', (int) $category_ids[0], Prodigy::get_prodigy_category_type() );
            $term2 = get_term_by( 'id', (int) $category_ids[1], Prodigy::get_prodigy_category_type() );
            $term3 = get_term_by( 'id', (int) $category_ids[2], Prodigy::get_prodigy_category_type() );

            ob_start();

            do_action( 'prodigy_shortcode_template_feed_categories', [
                'img1'  => $img1,
                'img2'  => $img2,
                'img3'  => $img3,
                'term1' => $term1,
                'term2' => $term2,
                'term3' => $term3,
            ] );
        } elseif ($type === 'our_categories') {
            $images_src = prodigy_get_images_src( $img_ids, $img_size );
            $category_ids = explode( ",", $category_ids );

            $count_products = [];
            foreach ( $category_ids as $key => $id ) {
                $categories[$key] = get_term_by( 'id', (int) $id, Prodigy::get_prodigy_category_type() );
            }

            ob_start();

            do_action( 'prodigy_shortcode_template_our_categories', [
                'images_src'  => $images_src,
                'categories'  => $categories,
            ] );

            wp_enqueue_script( 'categories', plugin_dir_url( PRODIGY_PLUGIN_PATH . '/includes/public/shortcodes/js/categories.js' ) . 'categories.js', [ 'jquery' ] );
        }


        $content = ob_get_contents();
        ob_end_clean();

        return $content;
    }

}

return new Prodigy_Short_Code_Categories();