<?php

/**
 * Prodigy filters shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Short_Code_Left_Categories_Filters
 */
class Prodigy_Short_Code_Left_Categories_Filters {
    
    private $cat_current_id;
    private $list_parent_id;
    private $request_attrs;
    private $show_level_depth = 0;
    
    const LEVEL = 1;
    const DEFAULT_TOP_LEVEL_CATEGORY = 0;
    
    public function __construct() {
        add_shortcode( 'categories-filters', [ $this, 'output' ] );
    }
    
    /**
     * @param int $category_id
     *
     * @return array
     */
    protected function get_info_category_custom( $category_id, $params = [] ) {
        return get_categories( array(
            'taxonomy'     => Prodigy::get_prodigy_category_type(),
            'orderby'      => $this->request_attrs['sorting'],
            'hide_empty'   => $this->request_attrs['hide_empty'],
            'parent'       => $category_id,
            'hierarchical' => $this->request_attrs['hierarchical'],
            'pad_counts'   => true
        ) );
    }
    
    /**
     * @param int $parent_id_category
     * @param bool $sub_menu_button
     * @param bool $sub_level
     *
     * @return string
     */
    protected function get_list_sub_categories( $parent_id_category, $sub_menu_button = false, $sub_level = false ) {
        
        $this->show_level_depth ++;
        
        $collapse      = '';
        $id_collapse   = '';
        $class_show    = '';
        $line_menu     = '';
        $aria_expanded = 'false';
        
        if ( $sub_menu_button ) {
            if ( $parent_id_category != $this->cat_current_id && ( is_array( $this->list_parent_id ) && in_array( $parent_id_category, $this->list_parent_id ) ) ) {
                $class_show    = 'show';
                $aria_expanded = 'true';
            }
            
            $collapse    = 'collapse ' . $class_show;
            $id_collapse = 'id="collapse_' . $parent_id_category . '_1"';
            
        }
        
        $sub_categories = $this->get_info_category_custom( $parent_id_category );
        
        if ( empty( $sub_categories ) ) {
            return $line_menu;
        }
        
        if ( ! empty( $sub_level ) && ! empty( $this->request_attrs['max_depth'] ) && $this->request_attrs['max_depth'] < $this->show_level_depth ) {
            return $line_menu;
        }
        
        if ( $sub_menu_button ) {
            $line_menu .= '<button class="filter__card-btn" data-toggle="collapse" data-target="#collapse_' . $parent_id_category . '_' . $sub_level . '" aria-expanded="' . $aria_expanded . '" aria-controls="collapse_' . $parent_id_category . '_' . $sub_level . '"><i class="icon icon-arrow-down"></i></button>';
        }
        
        $line_menu .= '<ul class="filter__card-list ' . $collapse . '" ' . $id_collapse . '>'; //filter__card-list
        
        foreach ( $sub_categories as $key => $value ) {
            
            $sub_sub_categories = $this->get_info_category_custom( $value->term_id );
            
            $line_sub_menu = '';
            if ( ! empty( $sub_sub_categories ) ) {
                /*
                 * Recursively create a sub category
                 */
                $line_sub_menu = $this->get_list_sub_categories( $value->term_id, true, self::LEVEL );
            }
            
            $line_menu .= '<li class="filter__card-list-item" id="sub_cat_' . esc_attr( $value->term_id ) . '">';
            $line_menu .= '<a href="';
            $line_menu .= esc_url( get_category_link( $value->term_id ) );
            $line_menu .= '">';
            $line_menu .= esc_attr( $value->name );
            $line_menu .= '</a>';
            if ( $this->request_attrs['count'] ) {
                $line_menu .= ' ';
                $line_menu .= '(' . esc_attr( $value->count ) . ')';
            }
            
            if ( ! empty( $line_sub_menu ) ) {
                $line_menu .= $line_sub_menu;
            }
            $line_menu .= '</li>';
        }
        
        $line_menu .= '</ul>'; //filter__card-list
        
        return $line_menu;
    }
    
    
    /**
     * @return bool|int
     */
    public function get_cat_parent_id() {
        $cat_parent_id = false;
        
        if ( ! empty( $this->cat_current_id ) ) {
            $defaults = array(
                'format'    => 'slug',
                'separator' => '/',
                'link'      => false,
                'inclusive' => true,
            );
            
            $cat_parents = get_term_parents_list( $this->cat_current_id, Prodigy::get_prodigy_category_type(), $defaults );
            
            if ( empty( $cat_parents ) ) {
                return $cat_parent_id;
            }
            
            $cat_parent_slug = explode( '/', $cat_parents )[0];
            
            $cat_parent_id = get_term_by( 'slug', $cat_parent_slug, Prodigy::get_prodigy_category_type() )->term_id;
            
        }
        
        return $cat_parent_id;
    }
    
    /**
     * @param int $term_id
     *
     * @return array
     */
    public function get_id_parent_categories( $term_id ) {
        
        $list_categories = [];
        
        $defaults = array(
            'format'    => 'slug',
            'separator' => '/',
            'link'      => false,
        );
        
        $cat_parents = get_term_parents_list( $term_id, Prodigy::get_prodigy_category_type(), $defaults );
        
        if ( empty( $cat_parents ) ) {
            return $list_categories;
        }
        
        if ( ! empty( $cat_parents ) && is_string( $cat_parents ) ) {
            $cat_parent_slug = explode( '/', $cat_parents );
            
            foreach ( $cat_parent_slug as $parent ) {
                
                $info_parent_cat = get_term_by( 'slug', $parent, Prodigy::get_prodigy_category_type() );
                
                if ( empty( $info_parent_cat ) ) {
                    continue;
                }
                $list_categories[] = $info_parent_cat->term_id;
            }
        }
        
        return $list_categories;
    }
    
    /**
     * @param $categories
     *
     * @return array
     */
    public function get_subcategories_by_parent_categories($categories) {
        $list_sub_categories = [];
        foreach ( $categories as $category ) {
            $list_sub_categories[$category->term_id] = $this->get_list_sub_categories( $category->term_id );
        }
        
        return $list_sub_categories;
    }
    
    /**
     * @param array $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    public function output( $atts, $content = null, $name ) {
        
        $this->request_attrs = $atts;
        
        if ( isset( get_queried_object()->term_id ) ) {
            $this->cat_current_id = get_queried_object()->term_id;
            
            $this->list_parent_id = $this->get_id_parent_categories( $this->cat_current_id );
        }
        
        $cat_parent_id = $this->get_cat_parent_id();
        
        wp_enqueue_script( 'prodigy-category', plugin_dir_url( PRODIGY_PLUGIN_PATH . '/includes/public/shortcodes/js/prodigy-category.js' ) . 'prodigy-category.js', [ 'jquery' ] );
        wp_localize_script(
            'prodigy-category',
            'prodigy_category_left_menu',
            [
                'active_id' => $this->cat_current_id,
                'parent_id' => $cat_parent_id,
            ]
        );
        
        ob_start();
        
        /**
         * take top level menu
         */
        $categories = $this->get_info_category_custom( self::DEFAULT_TOP_LEVEL_CATEGORY );
        
        do_action( 'prodigy_shortcode_template_left_categories_filter', [
            'cat_parent_id' => $cat_parent_id,
            'categories'    => $categories,
            'subcategories' => $this->get_subcategories_by_parent_categories($categories),
            'request_count' => $this->request_attrs['count'] ?? ''
        ] );
        
        $content = ob_get_contents();
        ob_clean();
        
        return $content;
    }
}

return new Prodigy_Short_Code_Left_Categories_Filters();
