<?php

/**
 * Prodigy price filter shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Short_Code_Price_Filter
 */
class Prodigy_Short_Code_Price_Filter {
    
    const MIN_PRICE = 0;
    const MAX_PRICE = 1000;
    
    private $request_attrs;
    
    /**
     * Prodigy_Short_Code_Price_Filter constructor.
     */
    public function __construct() {
        add_shortcode( 'price-filter', [ $this, 'output' ] );
    }
    
    
    /**
     * @return array
     */
    public function getMinMaxPrice() {
        
        /*
         * get min max price
         */
        $custom_args = get_agrs_categories_for_filter();
        
        $custom_query = new WP_Query( $custom_args );
        $ids          = wp_list_pluck( $custom_query->posts, 'ID' );
        $obj_min      = new SplMinHeap();
        $obj_max      = new SplMaxHeap();
        
        if ( ! empty( $ids ) ) {
            foreach ( $ids as $id ) {
                $price      = get_post_meta( $id, 'product_regular_price', true );
                $sale_price = get_post_meta( $id, 'product_sale_price', true );
                
                if ( $price != '' ) {
                    if ( $sale_price != '' ) {
                        $price = $sale_price;
                    }
                    $obj_min->insert( $price );
                    $obj_max->insert( $price );
                }
                
                if ( $sale_price != '' ) {
                    $obj_min->insert( $sale_price );
                }
            }
        }
        
        $min_price = $obj_min->isEmpty() ? self::MIN_PRICE : $obj_min->top();
        $max_price = $obj_max->isEmpty() ? self::MAX_PRICE : $obj_max->top();
        
        return [ 'min_price' => $min_price, 'max_price' => $max_price ];
    }
    
    /**
     * @param string $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    public function output( $atts, $content = null, $name ) {
        
        $this->request_attrs = $atts;
        
        $prices = $this->getMinMaxPrice();
        
        wp_enqueue_script( 'price-filter', plugin_dir_url( PRODIGY_PLUGIN_PATH . '/includes/public/shortcodes/js/price-filter.js' ) . 'price-filter.js', [ 'jquery' ] );
        wp_enqueue_script(
            'ion.rangeSlider.min',
            Prodigy::plugin_url() . '/web/templates/js/ion.rangeSlider.min.js',
            [ 'jquery' ]
        );
        wp_localize_script(
            'price-filter',
            'prodigy_product_price',
            [
                'min'      => $prices['min_price'],
                'max'      => $prices['max_price'],
                'taxonomy' => get_query_var( 'taxonomy' ),
                'current_type_category' => Prodigy::get_prodigy_category_type()
            ]
        );
        
        ob_start();
        
        do_action( 'prodigy_shortcode_template_price_filter' );
        
        $content = ob_get_contents();
        ob_clean();
        
        return $content;
    }
}

return new Prodigy_Short_Code_Price_Filter();
