<?php

/**
 * Prodigy products shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Prodigy_Short_Code_Cart
 */

class Prodigy_Short_Code_Products {
    /**
     * Prodigy_Short_Code_Products constructor.
     */
    public function __construct() {
        add_shortcode( 'products', [ $this, 'output' ] );
    }

    /**
     * @param string $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    function output( $atts, $content = null, $name ) {

        $atts = shortcode_atts( array(
            'product_ids' => '',
            /** list, related, crossell  */
            'type' => 'list',
            'img_size' => 'full'
        ), $atts, 'products' );

        if ( ! empty( $atts['product_ids'] ) ) {
            $list_ids = explode( ',', $atts['product_ids'] );
        }

        if ($atts['type'] === 'crossell') {
            if ( empty( $list_ids ) && isset($GLOBALS['prodigy_product']) ) {
                $product = $GLOBALS['prodigy_product'];
                $products = $product->get_cross_sell_product_ids()[0];
            } else {
                $products = [];

                foreach ( $list_ids as $request_product_id ) {
                    $list_related        = Prodigy_Product::get_crossell_product_ids( $request_product_id );
                    $products = array_merge( (array) $products, (array) $list_related );
                }

                if ( ! empty( $products ) && is_array( $products ) ) {
                    shuffle( $products );
                }
            }

            if ( empty( $products ) ) {
                return '';
            }

            foreach ( (array) $products as $key => $id ) {
                $products[ $key ] = Prodigy_Product::get_product_with_meta_by_id( $id );
            }

            $count_prod_list = 0;
            $placeholder_url = plugin_dir_url( PRODIGY_PLUGIN_PATH . 'web/admin/images/placeholder.png' ) . 'placeholder.png';


            $params = [
                'products'        => $products,
                'placeholder_url' => $placeholder_url,
            ];

            if (isset($product)) {
                $params['product'] = $product;
            }

            do_action( 'prodigy_shortcode_template_products', $params);


        } elseif ( $atts['type'] === 'related' ) {
            $products = Prodigy_Product::get_upsell_product_ids( $atts['product_ids'] );

            foreach ( (array) $products as $key => $id ) {
                $products[ $key ] = Prodigy_Product::get_product_with_meta_by_id( $id );
            }

            $count_prod_list = 0;
            $placeholder_url = plugin_dir_url( PRODIGY_PLUGIN_PATH . 'web/admin/images/placeholder.png' ) . 'placeholder.png';

            $params = [
                'products'        => $products,
                'placeholder_url' => $placeholder_url,
            ];

            if (isset($product)) {
                $params['product'] = $product;
            }

            do_action( 'prodigy_shortcode_template_products', $params);

            ob_start();
        } elseif ( $atts['type'] === 'list' ) {
            ob_start();
            if ( Prodigy_Template_Loader::$is_prodigy_theme_enable ) {
                do_action( 'prodigy_shortcode_template_products_theme', [
                    'product_ids' => $list_ids,
                    'img_size'    => 'full'
                ]);
            }
        }

        wp_enqueue_script( 'products', plugin_dir_url( PRODIGY_PLUGIN_PATH . '/includes/public/shortcodes/js/products.js' ) . 'products.js', [ 'jquery' ] );
        wp_localize_script(
            'products',
            'prodigy_products',
            [
                'is_home_page' => is_page('Home')
            ]
        );

        wp_reset_postdata();

        $content = ob_get_contents();

        ob_clean();

        return $content;
    }

}

return new Prodigy_Short_Code_Products();