<?php

/**
 * Prodigy shortcode class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__, 2 ) ) . 'class-prodigy-cart.php';

/**
 * Class Prodigy_Short_Code_Thank_You
 */
class Prodigy_Short_Code_Thank_You {
    
    /**
     * @var Prodigy_Cart
     */
    private $cart;
    
    public function __construct() {
        $this->cart = new Prodigy_Cart();
        add_shortcode( 'thank-you-page', [ $this, 'output' ] );
    }
    
    /**
     * @param array $atts
     * @param null $content
     * @param string $name
     *
     * @return string
     */
    public function output( $atts, $content = null, $name ) {
        global $wp_session;
        
        $list_ids            = '';
        $ids                 = [];
        $arr_ids             = [];
        $up_sell_product_ids = [];
        
        $order_token = isset($_GET['order_token']) ? filter_var( $_GET['order_token'], FILTER_SANITIZE_STRIPPED ) : '';
    
        $order_info = [];
        if ( ! empty( $order_token ) ) {
            $order_info = $this->cart->get_order_info( $order_token );
        }
        
        $order_remote = '';
        if ( ! empty( $order_info['api_response'] ) ) {
            $order_remote = (string) json_decode( $order_info['api_response'] )->data->attributes->number;
            $products     = $this->cart->get_remote_products_data( $order_token );
            $ids          = wp_list_pluck( $products, 'local_product_id' );
        }
        
        foreach ( $ids as $id ) {
            $post_info = get_post( $id );
            
            if ( $post_info->post_type == Prodigy::PRODIGY_VARIATION_POST_TYPE ) {
                array_push( $arr_ids, (int) $post_info->post_parent );
            } else {
                array_push( $arr_ids, (int) $id );
            }
        }
        
        $arr_ids = array_unique( $arr_ids );
        
        if ( ! empty( $arr_ids ) ) {
            $list_ids = implode( ',', $arr_ids );
        }
        
        foreach ( $arr_ids as $request_product_id ) {
            $list_crossell = Prodigy_Product::get_crossell_product_ids( $request_product_id );
            
            if ( is_array( $up_sell_product_ids ) && is_array( $list_crossell[0] ) ) {
                $up_sell_product_ids = array_merge( $up_sell_product_ids, $list_crossell[0] );
            }
        }
        
        ob_start();
        
        do_action( 'prodigy_shortcode_template_thank_you', [
            'up_sell_product_ids' => $up_sell_product_ids,
            'list_ids' => $list_ids,
            'order_info' => $order_info,
            'order_remote' => $order_remote,
        ] );
        
        $content = ob_get_contents();
        
        ob_clean();
        
        /**
         * clear token while show empty thank page
         */
        unset( $wp_session['order_token'] );
        
        return $content;
    }
}

return new Prodigy_Short_Code_Thank_You();
