( function ( $, window ) {
    var prodigy_cart = {

        main_container: '',

        init: function() {
            if (window.location.pathname.replace(/\/+$/, '') === '/cart' ) {
                this.set_item_counter();
                this.remove_item();
                this.recalculate_subtotal_amount_cart_widget();
                this.check_empty_cart();
                this.check_minus_buttons();
                this.set_count_to_enter_press();
                this.cart_page_hook();
            }
        },

        cart_page_hook: function() {
            $(document).ready(function() {
                $('.entry-content').removeClass('entry-content').addClass('black');
            });
        },

        set_count_to_enter_press: function() {
            var self = this;
            $(document).on('keypress', '.counter-count-js', function(e) {
                if(e.which === 13) {
                    $(this).blur();
                    self.main_container = $(this).parent().parent().parent();

                    if (parseInt($(this).val()) > 1) {
                        self.main_container.find('.counter-minus-jsounter-minus-js').prop('disabled', false);
                        self.main_container.find('.counter-minus-js').removeAttr('style');
                    } else {
                        self.main_container.find('.counter-minus-jsounter-minus-js').prop('disabled', true);
                        self.main_container.find('.counter-minus-js').css('background', 'grey');
                    }
                    var count = $(this).val();
                    self.recalculate_total_amount(count, self.main_container);
                    self.recalculate_subtotal_amount();
                    var local_product_id = self.main_container.find('.remove-item-js').data('local-id');
                    var remote_product_id = self.main_container.find('.remove-item-js').data('remote-id');
                    self.update_storage(local_product_id, remote_product_id, count);
                }
            });
        },


        check_minus_buttons: function() {
            $('.counter-count-js').each(function () {
                var minus = $(this).prev();
                if (parseInt($(this).val()) === 1 ) {
                    minus.prop('disabled', true);
                    minus.css('background', 'grey');
                    minus.css('cursor', 'not-allowed');
                }
            });
        },

        remove_item: function() {
            var self = this;
            $(document).on('click','.remove-item-js', function () {
                $(this).parent().parent().remove();
                self.recalculate_subtotal_amount();
                self.remove_from_storage($(this).data('local-id'), $(this).data('remote-id'));
                self.check_empty_cart();
                if ($('.cart-item-js').length === 0) {
                    $('.continue-cart-js').parent().hide();
                }
                self.reload_widget_content();
            });

            $(document).on('click', '.widget-remove-item-js', function () {
                var product_id = $(this).data('local-id');
                var product_block = $(".cart-body-js").find('[data-local-id=' + product_id + ']');
                product_block.parent().parent().remove();
                self.recalculate_subtotal_amount();
                self.check_empty_cart();
            });
        },

        check_empty_cart: function() {
            if (parseInt($('.cart-item-js').length) === 0) {
                $('.cart-items-list-js').hide();
                $('.cart-subtotal-js').hide();
                $('.empty-cart-js').show();
                $('.cart-form-block').removeClass('offset-xl-1');
                $('.cart-form').addClass('d-flex justify-content-center');
            }
        },

        // reload cart widget content
        reload_widget_content: function() {
            var self = this;
            $('.cart-widget-item-list-js').load(getCurrentUrl() + " .cart-widget-item-list-js>*", function () {
                if (parseInt($('.widget-cart-item-js').length) === 0) {
                    $('.widget-cart-checkout-block-js').hide();
                    $('.widget-cart-subtotal-js').hide();
                    $('.widget-cart-empty-cart-message-js').show();
                }
                self.recalculate_subtotal_amount_cart_widget();
            });
        },

        recalculate_subtotal_amount_cart_widget: function () {
            var self = this;
            var subtotal_amount = 0;
            $('.widget-cart-product-price-js').each(function () {
                let amount = parse_price($(this).find('.widget-total-price-js').text());
                let count = parseInt($(this).find('.widget-cart-count-item-js').text());
                subtotal_amount += amount * count;
            });

            $('.widget-subtotal-price-js').text('$' + prodigy_price_format(subtotal_amount));
        },

        remove_from_storage: function ( product_id, remote_id ) {
            var self = this;
            var post_data = {
                action: 'prodigy-remove-item-cart',
                product_id: product_id,
                remote_id: remote_id,
            };

            $.ajax({
                type: 'post',
                data: post_data,
                url: ajax_url,
                dataType: 'json',
                success: function (data) {
                    if (data.error) {
                        self.show_error(data.error);
                    } else {
                        self.show_add_to_cart_message();
                        self.reload_widget_content();
                    }
                },
            });
        },

        show_error: function( message ) {
            var add_to_cart_message = $('.widget-cart-message-error-js');
            add_to_cart_message.find('span').html(message);
            add_to_cart_message.show();
            add_to_cart_message.delay(5000).fadeOut('slow');
        },


        update_storage: function(local_id, remote_id, count, price) {
            var self = this;
            var post_data = {
                action: 'prodigy-update-item-cart',
                local_id: local_id,
                remote_id: remote_id,
                count: count,
                price: price
            };

            $.ajax({
                type: 'post',
                data: post_data,
                url: ajax_url,
                success: function (data) {
                    if (data !== 'null') {
                        if ( typeof data.error !== 'undefined' ) {
                            self.show_error(data.error);
                        }
                    } else {
                        self.show_add_to_cart_message();
                        self.reload_widget_content();
                    }
                },
            });
        },

        show_add_to_cart_message() {
            var add_to_cart_message = $('.widget-cart-add-item-message-js');
            add_to_cart_message.show();
            add_to_cart_message.delay(5000).fadeOut('slow');
        },

        recalculate_total_amount: function ( count, container ) {
            var self = this;
            var field_price = container.find('.price-js');
            var total_price_field = container.find('.total-price-js');
            var text_price = field_price.text();
            var price = parse_price(text_price);
            var total_price = parseFloat(price) * count;
            total_price_field.text('$' + Number(total_price).toFixed(2));
            localStorage.setItem("subtotal_amount", '$' + Number(total_price).toFixed(2));
        },

        recalculate_subtotal_amount: function() {
            var self = this;
            var subtotal_amount = 0;
            $('.total-price-js').each(function () {
                subtotal_amount += parse_price($(this).text());
            });
            subtotal_amount = Number(subtotal_amount).toFixed(2);
            $('.subtotal-price-js').text('$'+ subtotal_amount);
        },

        set_cart_counter_analytics: function(object) {
            if (options.pg_google_track_id) {
                var item = [];
                item.push(object.data('cart-item'));
                item.push({
                    quantity: $('counter-count-js').val(),
                });

                gtag('event', 'change_product quantity_ from_ cart', {
                    "event_category": 'prodigy_ecommerce',
                    "items": item[0]
                });
            }
        },

        set_item_counter: function () {
            var self = this;
            var plus = $('.counter-plus-js');
            var minus = $('.counter-minus-js');
            const counterCountJs = $('.counter-count-js');

            counterCountJs.inputmask({ regex: "[0-9]*", rightAlign: false, });

            minus.on('click', function() {
                self.main_container = $(this).parent().parent().parent();
                var current_count = self.main_container.find('.counter-count-js').val();
                var count = parseInt(current_count)-1;
                self.set_cart_counter_analytics($(this));

                if (count <= 1 ) {
                    $(this).prop('disabled', true);
                    $(this).css('background', 'grey');
                    $(this).css('cursor', 'not-allowed');
                }

                self.main_container.find('.counter-count-js').val(count);
                self.recalculate_total_amount(count, self.main_container);
                self.recalculate_subtotal_amount();

                var local_product_id = self.main_container.find('.remove-item-js').data('local-id');
                var remote_product_id = self.main_container.find('.remove-item-js').data('remote-id');
                var price_str = self.main_container.find('.price-js').text();
                var price = parse_price(price_str);
                self.update_storage(local_product_id, remote_product_id, count, price);
            });

            plus.on('click', function() {
                self.main_container = $(this).parent().parent().parent();
                var count = self.main_container.find('.counter-count-js').val();
                count = parseInt(count)+1;

                self.set_cart_counter_analytics($(this));

                if (count == 2) {
                    self.main_container.find('.counter-minus-js').prop('disabled', false);
                }

                if (count > 1 ) {
                    self.main_container.find('.counter-minus-js').removeAttr('style');
                }

                self.main_container.find('.counter-count-js').val(count);
                self.recalculate_total_amount(count,self.main_container);
                self.recalculate_subtotal_amount();

                var local_product_id = self.main_container.find('.remove-item-js').data('local-id');
                var remote_product_id = self.main_container.find('.remove-item-js').data('remote-id');
                var price_str = self.main_container.find('.price-js').text();
                var price = parse_price(price_str);
                self.update_storage(local_product_id, remote_product_id,  count, price);
            });
        },

    };

    window.prodigyCart = prodigy_cart;

})(jQuery, window);

jQuery(function ($) {
    window.prodigyCart.init();
});


