<?php


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( dirname( __FILE__ ) ) . '/../../class-prodigy-cart.php';

/**
 * Class Prodigy_Short_Code_Cart_Page
 */

class Prodigy_Short_Code_Cart_Page {

    private $cart;

    private $api_client;

    private $user_session;

    private $order_token;

    // messages for wp_logger
    private $log_errors = [
        0 => 'Do not set item in ',
        1 => 'Do not update item in ',
        2 => 'Do not update order for product ',
        3 => 'Do not update order for product ',
        4 => 'Do not update item in prodigy_cart_items',
        5 => "errors servers 'error send product to API' ",
        6 => 'Add to cart response is empty',
        7 => 'Do not set order in ',
        8 => 'Do not add product to the cart, empty user session ',
        9 => 'Do not set item in ',
        10 => '"errors from bd" error line item delete',
        11 => '"errors from bd" \'error do not check order ',
        12 => '"errors from hosted system" error check order ',
        13 => 'Cart doesn\'t reset!',
        14 => 'Error init order on API',
        15 => 'Do not delete order om API ',
    ];

    /**
     * Prodigy_Short_Code_Cart_Page constructor.
     */
    public function __construct() {
        add_shortcode( 'prodigy_cart', [ $this, 'output' ] );

        if (isset($_COOKIE['prodigy_user'])) {
            $prodigy_user       = filter_var( $_COOKIE['prodigy_user'], FILTER_SANITIZE_STRING );
            $this->user_session = $prodigy_user;
        }


        $request_uri = filter_var($_SERVER["REQUEST_URI"], FILTER_SANITIZE_URL);

        $this->cart         = new Prodigy_Cart();
        $this->api_client   = new Prodigy_Api_Client();
        $this->order_token  = $this->cart->get_token_for_current_order( $this->user_session );

        if ( ! empty( $this->order_token ) ) {
            $this->set_checkout_url();
        }
        $shop_page_uri = parse_url( $request_uri, PHP_URL_PATH );

        if ( substr_count( $shop_page_uri, "api-listener" ) ) {
            $this->callback_api_listener();
        }

        add_action( 'wp_head', [ $this, 'myplugin_ajaxurl' ] );
        add_action( 'wp_ajax_prodigy-add-remote-cart', [ $this, 'prodigy_add_product_to_remote_cart' ] );
        add_action( 'wp_ajax_nopriv_prodigy-add-remote-cart', [ $this, 'prodigy_add_product_to_remote_cart' ] );
        
        add_action( 'wp_ajax_prodigy-remove-item-cart', [ $this, 'prodigy_remove_item_from_cart' ] );
        add_action( 'wp_ajax_nopriv_prodigy-remove-item-cart', [ $this, 'prodigy_remove_item_from_cart' ] );
        
        add_action( 'wp_ajax_prodigy-update-item-cart', [ $this, 'prodigy_update_cart' ] );
        add_action( 'wp_ajax_nopriv_prodigy-update-item-cart', [ $this, 'prodigy_update_cart' ] );
        
        add_action( 'wp_ajax_prodigy-get-checkout-url', [ $this, 'prodigy_get_checkout_url' ] );
        add_action( 'wp_ajax_nopriv_prodigy-get-checkout-url', [ $this, 'prodigy_get_checkout_url' ] );
        
        add_action( 'wp_ajax_prodigy-check-empty-cart', [ $this, 'prodigy_check_empty_cart' ] );
        add_action( 'wp_ajax_nopriv_prodigy-check-empty-cart', [ $this, 'prodigy_check_empty_cart' ] );
    }


    /**
     * Check product by removed
     */
    public function prodigy_check_empty_cart() {
        $cart_items_in_cart_table = $this->cart->get_cart_items_for_cart_table(sanitize_text_field($_COOKIE['prodigy_user']));
        $products = get_option('prodigy_cart_removed_products');

        echo json_encode(empty($cart_items_in_cart_table) && empty($products));
        wp_die();
    }

    /**
     * @param array $atts
     * @param null  $content
     *
     * @return mixed
     */
    public function output( $atts ) {
        ob_start();

        wp_enqueue_script( 'cart-page', plugin_dir_url( PRODIGY_PLUGIN_PATH . '/includes/public/shortcodes/js/cart-page.js' ) . 'cart-page.js', [ 'jquery' ] );

        do_action( 'prodigy_shortcode_template_cart_page');

        wp_reset_postdata();

        $content = ob_get_contents();
        ob_clean();

        return $content;
    }

    /**
     * @param    int $error_id
     * @param string $message
     * @param string $error_type
     *
     * @return string
     */
    public function get_cart_error( $error_id, $message = '', $error_type = 'error' ) {
        $error_message = !empty($message) ? $message : $this->log_errors[$error_id];

        if ( PRODIGY_DEBUG_MODE ) {
            do_action( 'logger', $error_message, $error_type );
        }

        return "Something went wrong, please try again later.<br/> Error code: " . sprintf('%03d', $error_id + 1);
    }

    /**
     *  Set checkout url
     */
    public function prodigy_get_checkout_url() {
        $proceed_url = '';
        if ( isset($this->user_session, $this->order_token )  ) {
            $proceed_url = $this->cart->generate_checkout_url( $this->order_token );
        }

        echo wp_json_encode(['url' => $proceed_url]);
        wp_die();
    }

    /**
     * Parse cart api callback
     */
    public function callback_api_listener() {
        $order_token = filter_var( $_GET['token'], FILTER_SANITIZE_STRIPPED );
        $server_name = filter_var($_SERVER["SERVER_NAME"], FILTER_SANITIZE_URL);

        if ( isset( $order_token ) ) {
            if ( ! isset( $this->user_session ) ) {
                $order = $this->cart->get_in_process_order_for_token( $order_token );
                if ( ! empty( $order ) ) {
                    $this->cart->set_user_session( $order['local_user_hash'] );
                }
            }

            $order_response = $this->api_client->check_order( $order_token );
            if ( ! empty( $order_response ) ) {
                if ( $order_response['data']['attributes']['checkout-state'] == 'confirmation' ) {
                    /**
                     * update status to completed
                     */
                    $is_update_status = $this->cart->update_status_orders(
                        $order_token, Prodigy_Cart::PRODIGY_ORDER_COMPLETED
                    );

                    if ( $is_update_status ) {
                        $is_reset_cart = $this->cart->reset_cart_after_order_closing( $order_token );
                        if ( !$is_reset_cart ) {
                            $result['error'] = $this->get_cart_error(13);
                        }
                    }

                    global $wp_session;
                    $wp_session['order_token'] = $order_token;

                    header( 'Location:' . $server_name . '/thank-you?order_token=' . $order_token );
                } else {
                    header( 'Location:' . $server_name . '/cart' );
                }
                exit;

            } elseif( isset($order_response->errors) ) {
                $result['error'] = $this->get_cart_error(12, $order_response->errors, $this->log_errors[12] );
            }
        }
    }

    /**
     * Set checkout url
     */
    public function set_checkout_url() {
        if ( isset($this->user_session, $this->order_token )  ) {
            $GLOBALS['proceed_url'] = $this->cart->generate_checkout_url( $this->order_token );
        }
    }

    /**
     * Remove item from cart
     */
    public function prodigy_remove_item_from_cart() {
    
        $result = [];
        
        $remote_id = filter_var($_POST['remote_id'], FILTER_VALIDATE_INT);
        $product_id = filter_var($_POST['product_id'], FILTER_VALIDATE_INT);

        $remote_product_data = $this->cart->get_remote_product_data(
            $this->user_session,
            $this->order_token,
            $remote_id
        );

        if ( isset( $remote_product_data['token'], $remote_product_data['line_item_id'] ) ) {
            $delete_remote_product = $this->api_client->delete_line_item(
                $remote_product_data['line_item_id'],
                $remote_product_data['token']
            );
        }

        if ( !isset($delete_remote_product->errors) ) {
            $this->cart->remove_item_from_local_cart( $product_id, $this->user_session );
            $this->cart->remove_item_from_remote_cart( $remote_id, $this->order_token, $this->user_session );

            $is_empty_cart = $this->cart->is_empty_cart( $this->order_token );

            if ($is_empty_cart) {
                $delete_order = $this->api_client->delete_order( $this->order_token );
                if ( isset( $delete_order->errors ) || $delete_order === false ) {
                    $error_message = $delete_order->errors ?? 'Delete order error';
                    $status = $delete_order->errors ?? 'error';
                    $result['error'] = $this->get_cart_error(15, $error_message, $status);
                } else {
                    $this->cart->delete_local_order( $this->order_token );
                }
            }

            $order_data = $this->api_client->check_order( $this->order_token );
            if ( ! isset( $order_data->errors ) ) {
                $order_token  = $this->cart->get_token_for_current_order( $this->user_session );
                $this->cart->update_order_data_in_storage(
                    $order_token,
                    $this->user_session,
                    json_encode( $order_data )
                );
            } else {
                $result['error'] = $this->get_cart_error(11, $order_data->errors, $this->log_errors[11]. $this->order_token);
            }
        } else {
            $result['error'] = $this->get_cart_error(10, $delete_remote_product->errors, $this->log_errors[10]);
        }

        echo wp_json_encode( $result );
        wp_die();
    }

    /**
     * @return string
     */
    public function update_order_data() {
        $order_data = $this->api_client->check_order( $this->order_token );

        $result = false;
        if ( ! empty( $order_data ) ) {
            $result = $this->cart->update_order_data_in_storage( $this->order_token, $this->user_session, wp_json_encode( $order_data ) );
        }

        return (bool) $result;
    }

    /**
     * Update cart
     */
    public function prodigy_update_cart() {
        $remote_id = filter_var($_POST['remote_id'], FILTER_VALIDATE_INT);
        $count     = filter_var($_POST['count'], FILTER_VALIDATE_INT);
        $price = filter_var($_POST['price'], FILTER_VALIDATE_FLOAT);

        $result = null;

        $remote_product_data = $this->cart->get_remote_product_data( $this->user_session, $this->order_token, $remote_id );

        $line_item_request   = $this->cart->prepare_line_item_request( $remote_product_data, $count, $price );
        $line_item_response  = $this->api_client->update_line_items(
            $remote_product_data['line_item_id'],
            $line_item_request
        );

        if ( !isset($line_item_response->errors) ) {
            $update_cart_items_remote = $this->cart->update_items_from_remote_cart_in_storage(
                $remote_id,
                $count,
                $this->user_session,
                $this->order_token
            );

            $update_cart_items = $this->cart->update_items_from_local_cart_in_storage(
                $remote_id,
                $count,
                $this->user_session
            );
            if ( $update_cart_items_remote && $update_cart_items ) {
                $update_order = $this->update_order_data();
                if ( !$update_order ) {
                    $message = $this->log_errors[2] . $remote_id;
                    $result['error'] = $this->get_cart_error(2, $message);
                }
            } else {
                $message = $this->log_errors[1] . $this->cart->local_data_table;
                $result['error'] = $this->get_cart_error(1, $message);
            }
        } else {
            $result['error'] = $this->get_cart_error(
                5,
                $line_item_response->errors,
                $this->log_errors[5] . $remote_product_data['line_item_id']
            );
        }

        echo wp_json_encode( $result );
        wp_die();
    }

    /**
     * Send query to hosted system
     */
    public function prodigy_add_product_to_remote_cart() {

        $remote_product_id = filter_var($_POST['remote_product_id'], FILTER_VALIDATE_INT);



        $result = null;

        $product = get_post($_POST['local_product_id']);

        if ($product->post_status !== 'publish') {
            $result['error'] = 'Current product has been removed!';
        }

        if ( isset( $this->user_session ) && ! empty( $remote_product_id ) ) {
            if ( isset($this->order_token) ) {
                $is_order_deleted = $this->cart->is_deleted_order( $this->order_token );

                if ($is_order_deleted) {
                    $request = $this->cart->set_cart_prepare_request( $_POST );
                } else {
                    $request = $this->cart->set_cart_prepare_request( $_POST, $this->order_token );
                }
            } else {
                $request = $this->cart->set_cart_prepare_request( $_POST );
            }

            /* init order and push item in current order */
            $cart_response = $this->api_client->set_cart( $request );

            if ( isset( $cart_response->errors ) ) {
                $result['error'] = $this->get_cart_error(
                    14,
                    $cart_response->errors,
                    'error init order ' . $this->order_token
                );
            } elseif ( empty( $cart_response ) ) {
                $result['error'] = $this->get_cart_error(
                    6,
                    'error init order ' . $this->order_token
                );
            } else {
                // if check current order

                if ( isset( $this->order_token ) ) {
                    $is_order_deleted = $this->cart->is_deleted_order( $this->order_token );

                    if ($is_order_deleted) {
                        $save_order = $this->cart->create_order( $cart_response, $this->user_session );
                        if ( ! $save_order ) {
                            $message = $this->log_errors[7] . $this->cart->order_data_table;

                            $result['error'] = $this->get_cart_error(
                                7,
                                $message
                            );
                        }
                    }
                }

                if (!isset($this->order_token)) {
                    $save_order = $this->cart->create_order( $cart_response, $this->user_session );
                    if ( ! $save_order ) {
                        $message = $this->log_errors[7] . $this->cart->order_data_table;

                        $result['error'] = $this->get_cart_error(
                            7,
                            $message
                        );
                    }
                }

                $set_remote_cart_item = $this->cart->save_item_for_remote_cart( $this->user_session, $_POST,
                    $cart_response );
                if ( $set_remote_cart_item ) {
                    $order_token  = $this->cart->get_token_for_current_order( $this->user_session );

                    $set_local_cart_item = $this->cart->add_item_to_cart(
                        $_POST,
                        $this->user_session,
                        $order_token,
                        $cart_response
                    );
                    if ( $set_local_cart_item ) {
                        if ( isset( $this->order_token ) ) {
                            $this->update_order_data();
                        }
                    } else {
                        $message         = $this->log_errors[0] . $this->cart->local_data_table;
                        $result['error'] = $this->get_cart_error( 0, $message );
                    }
                } else {
                    $message         = $this->log_errors[9] . $this->cart->remote_data_table;
                    $result['error'] = $this->get_cart_error( 9, $message );
                }
            }
        } else {
            $message         = $this->log_errors[8] . $this->user_session . ' or product ' . $remote_product_id;
            $result['error'] = $this->get_cart_error( 8, $message );
        }


        $this->set_checkout_url();

        echo wp_json_encode( $result );
        wp_die();
    }

    /**
     *  Set ajaxurl path
     */
    public function myplugin_ajaxurl() {
        echo '<script type="text/javascript">
           var ajaxurl = "' . admin_url( 'admin-ajax.php' ) . '";
         </script>';
    }


}

return new Prodigy_Short_Code_Cart_Page();