<?php

/**
 * Displays the map
 *
 * @return void
 */
function prodigy_map_shortcode( $atts ) {

    $atts = shortcode_atts(
        array(
            'address'           => false,
            'width'             => '100%',
            'height'            => '400px',
            'enablescrollwheel' => 'true',
            'zoom'              => 10,
            'disablecontrols'   => 'false',
            'key'               => '',
            'only_map'          => 'true'
        ),
        $atts
    );

    $address = $atts['address'];

    if( $address ) :
        ob_start();
        $coordinates = prodigy_map_get_coordinates( $address, false, sanitize_text_field( $atts['key'] ) );

        if( ! is_array( $coordinates ) ) {
            return;
        }

        wp_enqueue_script( 'google-maps-api', '//maps.google.com/maps/api/js?key=' . sanitize_text_field( $atts['key'] ) );
        wp_print_scripts( 'google-maps-api' );

        $map_id = uniqid( 'pg_map_' ); // generate a unique ID for this map
        ?>
        <?php if ($atts['only_map'] === 'true'): ?>
            <div class="col-md-12 col-lg-5 offset-lg-1 offset-xl-2 contacts__map">
        <?php endif ?>
            <div class="pg_map_canvas" id="<?php echo esc_attr( $map_id ); ?>" style="height: <?php echo esc_attr( $atts['height'] ); ?>; width: <?php echo esc_attr( $atts['width'] ); ?>"></div>
            <script type="text/javascript">
                var pg_map_<?php echo $map_id; ?>;
                function pg_run_map_<?php echo $map_id ; ?>(){
                    var location = new google.maps.LatLng("<?php echo $coordinates['lat']; ?>", "<?php echo $coordinates['lng']; ?>");
                    var map_options = {
                        zoom: <?php echo $atts['zoom']; ?>,
                        center: location,
                        scrollwheel: <?php echo 'true' === strtolower( $atts['enablescrollwheel'] ) ? '1' : '0'; ?>,
                        disableDefaultUI: <?php echo 'true' === strtolower( $atts['disablecontrols'] ) ? '1' : '0'; ?>,
                        mapTypeId: google.maps.MapTypeId.ROADMAP
                    }
                    pg_map_<?php echo $map_id ; ?> = new google.maps.Map(document.getElementById("<?php echo $map_id ; ?>"), map_options);
                    var marker = new google.maps.Marker({
                        position: location,
                        map: pg_map_<?php echo $map_id ; ?>
                    });
                }
                pg_run_map_<?php echo $map_id ; ?>();
            </script>
        <?php if ($atts['only_map'] === 'true'): ?>
            </div>
        <?php endif ?>
        <?php
        return ob_get_clean();
    else :
        return __( 'This Google Map cannot be loaded because the maps API does not appear to be loaded', 'prodigy-theme' );
    endif;
}

add_shortcode( 'pg_map', 'prodigy_map_shortcode' );

/**
 * Retrieve coordinates for an address
 *
 * @return void
 */
function prodigy_map_get_coordinates( $address, $force_refresh = false, $api_key = '' ) {

    $address_hash = md5( $address );

    $coordinates = get_transient( $address_hash );

    if ( $force_refresh || $coordinates === false ) {

        $args       = apply_filters( 'pg_map_query_args', array( 'key' => $api_key, 'address' => urlencode( $address ), 'key' => $api_key ) );
        $url        = add_query_arg( $args, 'https://maps.googleapis.com/maps/api/geocode/json' );
        $response 	= wp_remote_get( $url );

        if( is_wp_error( $response ) ) {
            return;
        }

        $data = wp_remote_retrieve_body( $response );

        if( is_wp_error( $data ) ) {
            return;
        }

        if ( $response['response']['code'] == 200 ) {

            $data = json_decode( $data );

            if ( $data->status === 'OK' ) {

                $coordinates = $data->results[0]->geometry->location;

                $cache_value['lat'] 	= $coordinates->lat;
                $cache_value['lng'] 	= $coordinates->lng;
                $cache_value['address'] = (string) $data->results[0]->formatted_address;

                // cache coordinates for 3 months
                set_transient($address_hash, $cache_value, 3600*24*30*3);
                $data = $cache_value;

            } elseif ( $data->status === 'ZERO_RESULTS' ) {
                return __( 'No location found for the entered address.', 'prodigy-theme' );
            } elseif( $data->status === 'INVALID_REQUEST' ) {
                return __( 'Invalid request. Did you enter an address?', 'prodigy-theme' );
            } else {
                return __( 'Something went wrong while retrieving your map, please ensure you have entered the short code correctly.', 'prodigy-theme' );
            }

        } else {
            return __( 'Unable to contact Google API service.', 'prodigy-theme' );
        }

    } else {
        // return cached results
        $data = $coordinates;
    }

    return $data;
}
