<?php

/**
 * Prodigy active filters widget class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Active_Filters_Prodigy_Widget
 */
class Active_Filters_Prodigy_Widget extends WP_Widget {
    
    /**
     * @var string
     */
    protected $default_title;
    
    /**
     * Active_Filters_Prodigy_Widget constructor.
     */
    public function __construct() {
        
        $this->default_title = esc_html__( 'Active Filters', 'prodigy' );
        
        $widget_ops = array(
            'classname'   => 'active_filters_prodigy_widget',
            'description' => 'Prodigy Active Filters',
        );
        parent::__construct( 'active_filters_prodigy_widget', 'Prodigy Active Filters', $widget_ops );
        
        add_action( 'widgets_init', [ $this, 'active_filters_prodigy_widget' ] );
    }
    
    /**
     * register widget
     */
    public function active_filters_prodigy_widget() {
        register_widget( 'Active_Filters_Prodigy_Widget' );
    }
    
    /**
     * @param array $args
     * @param array $instance
     */
    public function widget( $args, $instance ) {
        
        /**
         * show widget only catalog
         */
        if ( ! Common_Prodigy_Widget::is_catalog_page() ) {
            return;
        }
        
        echo $args['before_widget'];
        
        if ( ! empty( $instance['title'] ) ) {
            echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ) . $args['after_title'];
        }
        
        prodigy_get_active_filters( $instance );
        
        echo $args['after_widget'];
    }
    
    /**
     * @param array $instance
     *
     * @return string|void
     */
    public function form( $instance ) {
        $title = isset( $instance['title'] ) ? $instance['title'] : $this->default_title;
        ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>">
                <?php esc_attr_e( 'Title:', 'prodigy' ); ?>
            </label>

            <input
                    class="widefat"
                    id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>"
                    type="text"
                    value="<?php echo esc_attr( $title ); ?>">
        </p>
        
        <?php
    }
    
    /**
     * @param array $new_instance
     * @param array $old_instance
     *
     * @return array
     */
    public function update( $new_instance, $old_instance ) {
        $instance          = [];
        $instance['title'] = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
        
        return $instance;
    }
}

return new Active_Filters_Prodigy_Widget();