<?php

/**
 * Prodigy common widget class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Categories_Prodigy_Widget
 */
class Categories_Prodigy_Widget extends WP_Widget {
    
    /**
     * @var string
     */
    protected $default_title;

    /**
     * Categories_Prodigy_Widget constructor.
     */
    public function __construct() {
    
        $this->default_title = esc_html__( 'Browse', 'prodigy' );
        
        $widget_ops = array(
            'classname'   => 'categories_prodigy_widget',
            'description' => 'Prodigy Product Categories',
        );
        parent::__construct( 'categories_prodigy_widget', 'Prodigy Product Categories', $widget_ops );
        
        add_action( 'widgets_init', [ $this, 'categories_prodigy_widget' ] );
    }
    
    /**
     * register widget
     */
    public function categories_prodigy_widget() {
        register_widget( 'Categories_Prodigy_Widget' );
    }
    
    /**
     * @param array $args
     * @param array $instance
     */
    public function widget( $args, $instance ) {
        
        echo $args['before_widget'];
        
        if ( ! empty( $instance['title'] ) ) {
            echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ) . $args['after_title'];
        }
        
        prodigy_get_categories_filter( $instance );
        
        echo $args['after_widget'];
    }
    
    /**
     * @param array $instance
     *
     * @return string|void
     */
    public function form( $instance ) {
        $title = isset( $instance['title'] ) ? $instance['title'] : $this->default_title;
        $sorting            = ! empty( $instance['sorting'] ) ? $instance['sorting'] : '';
        $count              = ! empty( $instance['count'] ) ? $instance['count'] : '';
        $hierarchical       = ! empty( $instance['hierarchical'] ) ? $instance['hierarchical'] : '';
        $hide_empty         = ! empty( $instance['hide_empty'] ) ? $instance['hide_empty'] : '';
        $max_depth          = ! empty( $instance['max_depth'] ) ? $instance['max_depth'] : '';
        ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>">
                <?php esc_attr_e( 'Title:', 'prodigy' ); ?>
            </label>

            <input
                    class="widefat"
                    id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>"
                    type="text"
                    value="<?php echo esc_attr( $title ); ?>">
        </p>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'sorting' ) ); ?>">
                <?php esc_attr_e( 'Order by', 'prodigy' ); ?>
            </label>
            <select class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'sorting' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'sorting' ) ); ?>">
                <option value="ID" <?php if ( $sorting == 'ID' ) {
                    echo 'selected';
                } ?>>Category ID
                </option>
                <option value="slug" <?php if ( $sorting == 'slug' ) {
                    echo 'selected';
                } ?>>Name
                </option>
            </select>
        </p>
        <p>
            <input
                    class="checkbox"
                    id="<?php echo esc_attr( $this->get_field_id( 'count' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'count' ) ); ?>"
                    type="checkbox"
                    value="1"
                <?php if ( $count ) {
                    echo 'checked';
                } ?>
            >
            <label for="<?php echo esc_attr( $this->get_field_id( 'count' ) ); ?>">
                <?php esc_attr_e( 'Show product counts', 'prodigy' ); ?>
            </label>
        </p>
        <p>
            <input
                    class="checkbox"
                    id="<?php echo esc_attr( $this->get_field_id( 'hierarchical' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'hierarchical' ) ); ?>"
                    type="checkbox"
                    value="1"
                <?php if ( $hierarchical ) {
                    echo 'checked';
                } ?>
            >
            <label for="<?php echo esc_attr( $this->get_field_id( 'hierarchical' ) ); ?>">
                <?php esc_attr_e( 'Show hierarchy', 'prodigy' ); ?>
            </label>
        </p>
        <p>
            <input
                    class="checkbox"
                    id="<?php echo esc_attr( $this->get_field_id( 'hide_empty' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'hide_empty' ) ); ?>"
                    type="checkbox"
                    value="1"
                <?php if ( $hide_empty ) {
                    echo 'checked';
                } ?>
            >
            <label for="<?php echo esc_attr( $this->get_field_id( 'hide_empty' ) ); ?>">
                <?php esc_attr_e( 'Hide empty categories', 'prodigy' ); ?>
            </label>
        </p>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'max_depth' ) ); ?>">
                <?php esc_attr_e( 'Maximum depth', 'prodigy' ); ?>
            </label>
            <input
                    class="widefat"
                    id="<?php echo esc_attr( $this->get_field_id( 'max_depth' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'max_depth' ) ); ?>"
                    type="text"
                    value="<?php echo esc_attr( $max_depth ); ?>"
                <?php if ( $max_depth ) {
                    echo 'checked';
                } ?>
            >

        </p>
        <?php
    }
    
    /**
     * @param array $new_instance
     * @param array $old_instance
     *
     * @return array
     */
    public function update( $new_instance, $old_instance ) {
        $instance                       = [];
        $instance['title']              = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
        $instance['sorting']            = ( ! empty( $new_instance['sorting'] ) ) ? sanitize_text_field( $new_instance['sorting'] ) : 'ID';
        $instance['count']              = ( ! empty( $new_instance['count'] ) ) ? sanitize_text_field( $new_instance['count'] ) : false;
        $instance['hierarchical']       = ( ! empty( $new_instance['hierarchical'] ) ) ? sanitize_text_field( $new_instance['hierarchical'] ) : false;
        $instance['hide_empty']         = ( ! empty( $new_instance['hide_empty'] ) ) ? sanitize_text_field( $new_instance['hide_empty'] ) : false;
        $instance['max_depth']          = ( ! empty( $new_instance['max_depth'] ) ) ? sanitize_text_field( $new_instance['max_depth'] ) : false;
        
        return $instance;
    }
}

return new Categories_Prodigy_Widget();