<?php

/**
 * Prodigy filters widget class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Filters_Prodigy_Widget
 */
class Filters_Prodigy_Widget extends WP_Widget {
    
    /**
     * @var string
     */
    protected $default_title;
    protected $list_attrs;
    
    /**
     * Filters_Prodigy_Widget constructor.
     */
    public function __construct() {
        
        $this->default_title = esc_html__( 'Filter By', 'prodigy' );
        
        $widget_ops = array(
            'classname'   => 'filters_prodigy_widget',
            'description' => 'Prodigy Product Filters',
        );
        parent::__construct( 'filters_prodigy_widget', 'Prodigy Product Filters', $widget_ops );
        
        $obj_attrs        = new Prodigy_Product_Attributes();
        $this->list_attrs = $obj_attrs->get_attribute_taxonomies();
        
        add_action( 'widgets_init', [ $this, 'filters_prodigy_widget' ] );
    }
    
    /**
     * register widget
     */
    public function filters_prodigy_widget() {
        register_widget( 'Filters_Prodigy_Widget' );
    }
    
    /**
     * @param array $args
     * @param array $instance
     */
    public function widget( $args, $instance ) {
        
        /**
         * show widget only catalog
         */
        if ( ! Common_Prodigy_Widget::is_catalog_page() ) {
            return;
        }
        
        echo $args['before_widget'];
        
        if ( ! empty( $instance['title'] ) ) {
            echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ) . $args['after_title'];
        }
        
        prodigy_get_products_attributes_filter( $instance );
        
        if ( ! empty( $instance['filter_by_price'] && have_posts() ) ) {
            prodigy_get_products_price_filter( $instance );
        }
        
        echo $args['after_widget'];
    }
    
    /**
     * @param array $instance
     *
     * @return string|void
     */
    public function form( $instance ) {
        
        wp_enqueue_script(
            'attributes-filter',
            plugin_dir_url( dirname( __FILE__, 2 ) ) . '/web/admin/js/widgets/attributes-filter.js',
            [
                'jquery',
                'jquery-ui-accordion'
            ],
            false,
            true
        );
        
        $title           = isset( $instance['title'] ) ? $instance['title'] : $this->default_title;
        $query_type      = ! empty( $instance['query_type'] ) ? $instance['query_type'] : '';
        $filter_by_price = ! empty( $instance['filter_by_price'] ) ? $instance['filter_by_price'] : '';
        $list_attributes = empty( $instance['list_attributes'] ) ? wp_list_pluck( $this->list_attrs, 'id' ) : $instance['list_attributes'];
        ?>

        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>">
                <?php esc_attr_e( 'Title:', 'prodigy' ); ?>
            </label>

            <input
                    class="widefat"
                    id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>"
                    type="text"
                    value="<?php echo esc_attr( $title ); ?>">
        </p>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'query_type' ) ); ?>">
                <?php esc_attr_e( 'Query type:', 'prodigy' ); ?>
            </label>
            <select class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'query_type' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'query_type' ) ); ?>">
                <option value="and" <?php if ( $query_type == 'and' ) {
                    echo 'selected';
                } ?>>AND
                </option>
                <option value="or" <?php if ( $query_type == 'or' ) {
                    echo 'selected';
                } ?>>OR
                </option>
            </select>
        </p>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'list_attributes' ) ); ?>">
                <?php esc_attr_e( 'Attributes multi-select:', 'prodigy' ); ?>
            </label>
            <select class="widefat js-list-attributes-widget-filter"
                    multiple="true"
                    data-plugin="select2"
                    id="<?php echo esc_attr( $this->get_field_id( 'list_attributes' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'list_attributes' ) ); ?>[]"
            >
                <?php foreach ( $this->list_attrs as $attr ):
                    $selected_filter = '';
                    if ( in_array( $attr->id, $list_attributes ) ) {
                        $selected_filter = 'selected';
                    }
                    ?>
                    <option value="<?php esc_html_e( $attr->id ); ?>" <?php echo $selected_filter; ?>><?php esc_html_e( $attr->name ); ?></option>
                <?php endforeach; ?>
            </select>
        </p>
        <p>
            <input
                    class="checkbox"
                    id="<?php echo esc_attr( $this->get_field_id( 'filter_by_price' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'filter_by_price' ) ); ?>"
                    type="checkbox"
                    value="1"
                <?php if ( $filter_by_price ) {
                    echo 'checked';
                } ?>
            >
            <label for="<?php echo esc_attr( $this->get_field_id( 'filter_by_price' ) ); ?>">
                <?php esc_attr_e( 'Filter by price', 'prodigy' ); ?>
            </label>
        </p>
        
        <?php
        
        $sidebar_id = is_active_widget( 0, 0, 'filters_prodigy_widget' );
        
        if ( ! empty( $instance ) && $sidebar_id ): ?>
            <script type="text/javascript">
                (function ($) {

                    $('.js-list-attributes-widget-filter').select2(
                        {
                            width: '100%',
                        }
                    );
                })(jQuery);

            </script>
        
        <?php
        endif;
    }
    
    /**
     * @param array $new_instance
     * @param array $old_instance
     *
     * @return array
     */
    public function update( $new_instance, $old_instance ) {
        
        $active_filter = is_active_widget( false, false, 'filters_prodigy_widget', false );
        
        $instance                    = [];
        $instance['title']           = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
        $instance['query_type']      = ( ! empty( $new_instance['query_type'] ) ) ? sanitize_text_field( $new_instance['query_type'] ) : 'and';
        $instance['filter_by_price'] = ( ! empty( $new_instance['filter_by_price'] ) ) ? sanitize_text_field( $new_instance['filter_by_price'] ) : '';
        $instance['list_attributes'] = ( ! empty( $new_instance['list_attributes'] ) ) ? $new_instance['list_attributes'] : '';
        
        if (
            ! empty( $active_filter )
            && ! empty( array_diff( $old_instance['list_attributes'], $new_instance['list_attributes'] ) )
        ) {
            update_option( 'prodigy_widget_change_default_state_filter', (bool) $active_filter );
        }
        
        return $instance;
    }
}

return new Filters_Prodigy_Widget();