<?php

/**
 * Prodigy search widget class
 *
 * @version    1.0.0
 * @package    Prodigy
 * @subpackage Prodigy/includes
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Class Search_Prodigy_Widget
 */
class Search_Prodigy_Widget extends WP_Widget {
    
    /**
     * @var string
     */
    protected $default_title;
    
    /**
     * Search_Prodigy_Widget constructor.
     */
    public function __construct() {
        
        $this->default_title = esc_html__( 'Products search', 'prodigy' );
        
        $widget_ops = array(
            'classname'   => 'search_prodigy_widget',
            'description' => 'Prodigy Product Search',
        );
        parent::__construct( 'search_prodigy_widget', 'Prodigy Product Search', $widget_ops );
        
        add_action( 'widgets_init', [ $this, 'search_prodigy_widget' ] );
    }
    
    /**
     * register widget
     */
    function search_prodigy_widget() {
        register_widget( 'Search_Prodigy_Widget' );
    }
    
    /**
     * @param array $args
     * @param array $instance
     */
    public function widget( $args, $instance ) {
        echo $args['before_widget'];
        
        if ( ! empty( $instance['title'] ) ) {
            echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ) . $args['after_title'];
        }
        
        echo do_shortcode( '[prodigy-search]' );
        
        echo $args['after_widget'];
    }
    
    /**
     * @param array $instance
     *
     * @return string|void
     */
    public function form( $instance ) {
        
        $title = isset( $instance['title'] ) ? $instance['title'] : $this->default_title;
        
        ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>">
                <?php esc_attr_e( 'Title:', 'prodigy' ); ?>
            </label>

            <input
                    class="widefat"
                    id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>"
                    type="text"
                    value="<?php echo esc_attr( $title ); ?>"
            >
        </p>
        <?php
    }
    
    /**
     * @param array $new_instance
     * @param array $old_instance
     *
     * @return array
     */
    public function update( $new_instance, $old_instance ) {
        $instance          = [];
        $instance['title'] = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
        
        return $instance;
    }
}

return new Search_Prodigy_Widget();