<?php

/**
 * Plugin Name:       Prodigy Commerce
 * Plugin URI:        https://prodigycommerce.com
 * Description:       Provides a comprehensive set of tools to build unparalleled eCommerce experiences on WordPress.
 * Version:           1.1.2
 * Author:            Prodigy Commerce
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       prodigy
 *
 * @package           Prodigy
 */

define( 'PRODIGY_VERSION', '1.1.2' );

//GET HOSTNAME INFO
$hostname  = $_SERVER['SERVER_NAME'];

if (file_exists(plugin_dir_path( __FILE__ ) . "env.ini")) {
    $path = plugin_dir_path( __FILE__ ) . "env.ini";
    $ini_array = parse_ini_file( "env.ini", true );
}

$prodigy_environment     = 'production';
$prodigy_api_domain      = 'api.prodigycommerce.com';
$prodigy_main_domain     = 'app.prodigycommerce.com';
$prodigy_checkout_domain = 'prodigycommerce.com';
$prodigy_protocol_domain = 'https://';
$prodigy_debug_mode      = false;

if ( ! empty( $ini_array ) && key_exists( 'env_section', $ini_array ) ) {
    $ini_section = $ini_array['env_section'];
    if ( key_exists( 'APP_ENV', $ini_section ) ) {
        $prodigy_environment = $ini_section['APP_ENV'];
    }
    if ( key_exists( 'APP_API_DOMAIN', $ini_section ) ) {
        $prodigy_api_domain = $ini_section['APP_API_DOMAIN'];
    }
    if ( key_exists( 'APP_MAIN_DOMAIN', $ini_section ) ) {
        $prodigy_main_domain = $ini_section['APP_MAIN_DOMAIN'];
    }
    if ( key_exists( 'APP_PROTOCOL_DOMAIN', $ini_section ) ) {
        $prodigy_protocol_domain = $ini_section['APP_PROTOCOL_DOMAIN'];
    }
    if ( key_exists( 'APP_CHECKOUT_DOMAIN', $ini_section ) ) {
        $prodigy_checkout_domain = $ini_section['APP_CHECKOUT_DOMAIN'];
    }
    if ( key_exists( 'APP_DEBUG_MODE', $ini_section ) ) {
        $prodigy_debug_mode = $ini_section['APP_DEBUG_MODE'];
    }
}

if ( !defined( 'PRODIGY_PLUGIN_PATH' ) ) {
    define( 'PRODIGY_ENVIRONMENT', $prodigy_environment );
    define( 'PRODIGY_CHECKOUT_DOMAIN', $prodigy_checkout_domain );
    define( 'PRODIGY_API_DOMAIN', $prodigy_api_domain );
    define( 'PRODIGY_MAIN_DOMAIN', $prodigy_main_domain );
    define( 'PRODIGY_PROTOCOL_DOMAIN', $prodigy_protocol_domain );
    define( 'PRODIGY_DEBUG_MODE', $prodigy_debug_mode );
    define( 'PRODIGY_PLUGIN_PATH', dirname(__FILE__).'/' );
    define( 'PRODIGY_PLUGIN_DIRECTORY_NAME', get_prodigy_plugin_directory_name() );
    define( 'HOSTED_PATH_LOGIN', '/customer/login' );
}

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
    die;
}

// Define PRODIGY_PLUGIN_FILE.
if ( ! defined( 'PRODIGY_PLUGIN_FILE' ) ) {
    define( 'PRODIGY_PLUGIN_FILE', __FILE__ );
}

// Define PRODIGY_WIZARD_PATH.
if ( ! defined( 'PRODIGY_WIZARD_PATH' ) ) {
    define( 'PRODIGY_WIZARD_PATH', 'web/wizard/' );
}

if ( ! function_exists( 'getallheaders' ) ) {
    function getallheaders() {
        $headers = [];
        foreach ( $_SERVER as $name => $value ) {
            if ( substr( $name, 0, 5 ) == 'HTTP_' ) {
                $headers[ str_replace( ' ', '-', ucwords( strtolower( str_replace( '_', ' ', substr( $name, 5 ) ) ) ) ) ] = $value;
            }
        }

        return $headers;
    }
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-prodigy-activator.php
 */
function activate_prodigy() {
    require_once plugin_dir_path( __FILE__ ) . 'includes/class-prodigy-activator.php';
    Prodigy_Activator::activate();
}

/**
 * Get plugin directory name
 *
 * @return string
 */
function get_prodigy_plugin_directory_name() {
    $plugin_path = plugin_dir_path( __FILE__ ) ;
    $plugin_directories_list = explode('/', $plugin_path);
    $clear_directory_names = array_filter($plugin_directories_list);

    return end($clear_directory_names);
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-prodigy-deactivator.php
 */
function deactivate_prodigy() {
    require_once plugin_dir_path( __FILE__ ) . 'includes/class-prodigy-deactivator.php';
    Prodigy_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_prodigy' );
register_deactivation_hook( __FILE__, 'deactivate_prodigy' );


/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-prodigy.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_prodigy() {

    $plugin = new Prodigy();
    $plugin->run();

}

run_prodigy();
