<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @version      1.0.0
 * @package prodigy/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Prodigy
 * @subpackage Prodigy/public
 * @author     test <test@gmail.com>
 */
class Prodigy_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 */
	public function __construct( $plugin_name ) {
        $this->plugin_name = $plugin_name;
		$this->version = false;
        add_action( 'wp_head', [ $this, 'set_ajaxurl_to_script' ] );
        add_action( 'wp_head', [ $this, 'add_analytics' ] );
        add_action( 'wp_enqueue_scripts', [ $this, 'my_theme_enqueue_styles' ], 99 );
        add_action( 'widgets_init', [ $this, 'hide_excess_widgets' ] );
    }

    /**
     * Hide excess widgets in theme
     */
    function hide_excess_widgets() {
        if ('prodigy-wp-theme' ===  get_option( 'template' )) {
            unregister_widget('Search_Prodigy_Widget');
            unregister_widget('Cart_Prodigy_Widget');
        }
    }

    public function add_analytics() {
	    $google_track_id = get_option('pg_google_track_id');

        if ( !empty($google_track_id) ) {
            ?>
            <!-- Global site tag (gtag.js) - Google Analytics -->
            <script async src="https://www.googletagmanager.com/gtag/js?id=<?php echo $google_track_id ?>"></script>
            <script>
                window.dataLayer = window.dataLayer || [];
                function gtag(){dataLayer.push(arguments);}
                gtag('js', new Date());
                gtag('config', "<?php echo $google_track_id ?>");

                gtag('config',"<?php echo $google_track_id ?>", {
                    'linker': {
                        'domains': [ '<?php echo $_SERVER['SERVER_NAME'] ?>', 'prodigycommerce.com'],
                    }
                });
            </script>
            <?php
        }
    }


    // register public style after all
    function my_theme_enqueue_styles() {
        if ( 'prodigy-wp-theme' !==  get_option( 'template' ) )  {
            wp_enqueue_style(
                'public',
                plugin_dir_url( __FILE__ ) . 'assets/css/public.css',
                [],
                $this->version,
                'all'
            );
        }
    }

    /**
     * Set ajax_url in scripts
     */
    function set_ajaxurl_to_script() {
        echo '<script type="text/javascript">
           var ajax_url = "' . admin_url('admin-ajax.php') . '";
         </script>';
    }

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
        wp_enqueue_style(
            'photoswipe',
            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/css/photoswipe/photoswipe.css',
            [],
            $this->version,
            'all'
        );
        wp_enqueue_style(
            'photoswipe-skin',
            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/css/photoswipe/default-skin/default-skin.css',
            [],
            $this->version,
            'all'
        );

        wp_enqueue_style(
            'slick',
            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/css/slick/slick.css',
            [],
            $this->version
        );
        wp_enqueue_style(
            'slick.theme',
            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/css/slick/slick-theme.css',
            [],
            $this->version
        );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

        wp_enqueue_script(
            'public',
            plugin_dir_url( __FILE__ ) . 'assets/js/public.js',
            [ 'jquery' ],
            $this->version,
            false
        );

        $scriptData = array(
            'pg_google_track_id' => get_option('pg_google_track_id'),
        );

        wp_localize_script('public', 'options', $scriptData);

        $product_type = Prodigy::get_prodigy_product_type();
        wp_localize_script('public', 'product_type', $product_type);

        $category_type = Prodigy::get_prodigy_category_type();
        wp_localize_script('public', 'category_type', $category_type);

        wp_enqueue_script(
            'jquery.flexslider.min',
            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/js/jquery.flexslider.min.js',
            [ 'jquery' ],
            $this->version,
            false
        );
        wp_enqueue_script(
            'photoswipe.min',
            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/js/photoswipe.min.js',
            [ 'jquery' ],
            $this->version,
            false
        );
        wp_enqueue_script(
            'photoswipe-ui-default.min',
            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/js/photoswipe-ui-default.min.js',
            [ 'jquery' ],
            $this->version,
            false
        );
        wp_enqueue_script(
            'jquery.magnific-popup.min',

            dirname( plugin_dir_url(__FILE__) ) . '/web/templates/js/jquery.magnific-popup.min.js',
            [ 'jquery' ],
            $this->version,
            false
        );
        
        wp_enqueue_script(
            'jquery.inputmask.bundle.min',
            dirname( plugin_dir_url(__FILE__) ) . '/web/libraries/inputmask/js/jquery.inputmask.bundle.min.js',
            [ 'jquery' ]
        );
        
        wp_enqueue_style(
            'inputmask',
            dirname( plugin_dir_url( __FILE__ ) ) . '/web/libraries/inputmask/css/inputmask.css'
        );

        wp_enqueue_script( 'popper.min', dirname( plugin_dir_url(__FILE__) ) . '/web/templates/js/popper.min.js', [],
            $this->version, true );
        wp_enqueue_script( 'bootstrap.min', dirname( plugin_dir_url(__FILE__) ) . '/web/templates/js/bootstrap.min.js', ['jquery'], $this->version, true );
        wp_enqueue_script( 'slick.min', dirname( plugin_dir_url(__FILE__) ) . '/web/templates/js/slick.min.js', ['jquery'], $this->version, false );

        if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
            wp_enqueue_script( 'comment-reply' );
        }
    }

}
