<?php defined( 'ABSPATH' ) || exit; ?>
<?php
$cart          = new Prodigy_Cart();
$cart_products = [];

add_option('prodigy_cart_removed_products', []);
$is_show = false;

if ( $_COOKIE['prodigy_user'] ) {
    $cart_items = $cart->get_cart_items_for_user( sanitize_text_field($_COOKIE['prodigy_user']) );

    $subtotal_price = 0;
    $gallery_thumb = array_values( array(
        'width'  => 150,
        'height' => '',
        'crop'   => 0,
    ) );

    if ( ! empty( $cart_items ) ) {
        foreach ( $cart_items as $item ) {
            $product                                                  = get_prodigy_product( $item['local_product_id'] );
            $cart_products[ $item['remote_product_id'] ]              = get_prodigy_product( $item['local_product_id'] );
            $cart_products[ $item['remote_product_id'] ]->count_items = $item['count'];

            $sale_price = $product->get_meta_field( 'product_sale_price' );
            $regular_price = $product->get_meta_field( 'product_regular_price' );

            $actual_price = $sale_price != "" ? $sale_price : $regular_price;

            $cart_products[ $item['remote_product_id'] ]->actual_price = $actual_price;

            $cart_products[ $item['remote_product_id'] ]->total_price = (float) $actual_price * (int) $item['count'];

            $cur_attributes = json_decode( $item['attributes'], true );
            if ( ! empty( $cur_attributes ) ) {
                $attributes = [];
                if (isset($cur_attributes['item'])) {
                    foreach ( $cur_attributes['item'] as $key => $attribute ) {
                        $attributes[ $cur_attributes['attribute'][ $key ] ] = $attribute;
                    }
                }

                $cart_products[ $item['remote_product_id'] ]->item_attributes = $attributes;
            }
        }
    }
}
$removed_product = get_option('prodigy_cart_removed_products');
$is_prodigy_theme_enable = Prodigy_Template_Loader::$is_prodigy_theme_enable;
?>


<?php if (empty($cart_products)): ?>

    <?php if ($is_prodigy_theme_enable): ?>
        <div class="container" style="display: <?php echo !empty($removed_product) ? 'block' : 'none' ?> ">
            <div class="row">
                <div class="col-lg-4 offset-lg-4">
                    <div class="notification notification--info">
                        <i class="icon icon-info font-20 in-white mr-16 align-self-start"></i>
                        <span class="notification__txt">One or more items are no longer available and were removed from your cart</span>
                    </div>
                </div>
            </div>
        </div>
    <?php else: ?>
        <div class="container" style="display: <?php echo !empty($removed_product) ? 'block' : 'none' ?> ">
            <div class="row">
                <div class="col-lg-4 offset-lg-4">
                    <div class="prodigy-cart-dropdown__alert widget-cart-message-error-js mb-20" style="display: <?php echo !empty($removed_product) ? 'flex' : 'none' ?> ">
                        <i class="icon icon-error mr-8"></i>
                        <span>One or more items are no longer available and were removed from your cart</span>
                    </div>
                </div>
            </div>
        </div>
    <?php endif; ?>
    <?php delete_option('prodigy_cart_removed_products') ?>
<?php endif; ?>

<?php if (!empty($cart_products)): ?>
    <div class="cart-items-list-js">
        <h4><?php esc_html_e( 'YOU ORDER', 'prodigy' ); ?></h4>

        <?php if ($is_prodigy_theme_enable): ?>
            <div class="notification notification--info" style="display: <?php echo !empty($removed_product) ? 'block' : 'none' ?>">
                <i class="icon icon-info font-20 in-white mr-16 align-self-start"></i>
                <span class="notification__txt">One or more items are no longer available and were removed from your cart</span>
            </div>
        <?php else: ?>
            <div class="prodigy-cart-dropdown__alert widget-cart-message-error-js mb-20" style="display: <?php echo !empty($removed_product) ? 'flex' : 'none' ?> ">
                <i class="icon icon-error mr-8"></i>
                <span>One or more items are no longer available and were removed from your cart</span>
            </div>
        <?php endif; ?>
        <?php delete_option('prodigy_cart_removed_products') ?>

        <div class="overflow-auto">
            <table class="prodigy-cart__table">
                <thead>
                <tr class="text-left">
                    <th class="prodigy-cart__table-product-cell"><?php esc_html_e( 'Product', 'prodigy' ); ?></th>
                    <th class="prodigy-cart__table-desc-cell"></th>
                    <th class="prodigy-cart__table-price-cell"><?php esc_html_e( 'Price', 'prodigy' ); ?></th>
                    <th class="prodigy-cart__table-quantity-cell"><?php esc_html_e( 'Quantity', 'prodigy' ); ?></th>
                    <th class="prodigy-cart__table-price-cell text-right"><?php esc_html_e( 'Total', 'prodigy' ); ?></th>
                    <th class="prodigy-cart__table-del-cell"></th>
                </tr>
                </thead>
                <tbody class="cart-body-js">
                <?php foreach ( $cart_products as $remote_product_id => $product ): ?>

                    <?php $subtotal_price += $product->total_price; ?>

                    <?php
                    $main_product = Prodigy_Product::get_product_with_meta_by_id( $product->get_field( 'post_parent' ) );

                    $mpost_title = isset($main_product['fields']['post_title']) ? $main_product['fields']['post_title'] : '';
                    $mID = isset($main_product['fields']['ID']) ? $main_product['fields']['ID'] : '';

                    $attachment_id = ! empty( $product->get_meta_field( '_thumbnail_id' ) ) ?
                        $product->get_meta_field( '_thumbnail_id' ) :
                        $product->get_meta_field( 'category-image-id' );

                    $image = wp_get_attachment_image_src( $attachment_id );
                    $image_src = isset($image[0]) ? "<img width='150' src='$image[0]' />" : "" ;

                    $product_name = ! empty( $product->get_field( 'post_parent' ) ) ?
                        $mpost_title :
                        $product->get_field( 'post_title' );
                    $variant_name = empty( $product->get_field( 'post_parent' ) ) ? '' : $product->get_field( 'post_title' );

                    $product_id = ! empty( $product->get_field( 'post_parent' ) ) ?
                        $mID :
                        $product->get_field( 'ID' );


                    $data_analytics['id'] = $remote_product_id;
                    $data_analytics['name'] = $product_name;
                    $data_analytics['list_name'] = $product->get_meta_field('product_sku');
                    $data_analytics['variant'] = $variant_name;
                    $data_analytics['quantity'] = $product->count_items;
                    $data_analytics['list_position'] = $product->get_meta_field('post_parent');
                    $data_analytics['price'] = prodigy_price_format($product->actual_price);
                    ?>

                    <tr>
                        <td class="cart-item-js prodigy-cart__table-product-td" data-cart-item = '<?php echo json_encode($data_analytics) ?>'>
                            <div class="d-flex flex-column justify-content-center align-items-center prodigy-cart__placeholder">
                                <a class="d-flex align-items-center justify-content-center flex-grow-1 w-100" href="<?php echo get_permalink( $product_id ) ?>">
                                    <?php echo empty($image_src) ? '<i class="icon icon-image flex-shrink-0 text-center"></i>' : $image_src; ?>
                                </a>
                            </div>
                        </td>
                        <td class="pl-16">
                            <div class="prodigy-cart__table-info">
                                <p class="prodigy-cart__table-info-title">
                                    <a href="<?php echo get_permalink( $product_id ) ?>">
                                        <?php echo esc_attr($product_name)?>
                                    </a>
                                </p>
                                <p class="text-nowrap font-800 price-js d-block mb-8 d-md-none">
                                    $<?php echo esc_attr(prodigy_price_format( $product->actual_price )); ?>
                                </p>
                                <div class="prodigy-counter d-md-none mb-16">
                                    <button
                                            class="main-button main-button--counter main-button--counter-minus icon icon-minus counter-minus-js"
                                            type="button"
                                        <?php echo ( $product->count_items == 1 ) ? 'disabled' : '' ?>
                                    ></button>
                                    <input class="prodigy-counter__total main-input counter-count-js" type="text" value="<?php echo esc_attr($product->count_items); ?>"  maxlength="3"/>
                                    <button class="main-button main-button--counter main-button--counter-plus icon icon-plus counter-plus-js" type="button">
                                    </button>
                                </div>
                                <p class="mb-0"><?php esc_attr_e('Sku:', 'prodigy')?>
                                    <?php echo esc_attr($product->get_meta_field( 'product_sku' )); ?>
                                </p></br>
                                <ul class="prodigy-cart-item__info-variants">
                                    <?php $attributes = json_decode( str_replace( "'", '"', stripslashes( $product->get_meta_field('post_attributes') ) ), 1 ); ?>
                                    <?php if (!empty($product->get_meta_field('post_attributes'))): ?>
                                        <?php foreach ( $attributes as $key => $item ): ?>
                                            <li><?php echo $key . ' : ' . esc_attr($item) ?></li>
                                        <?php endforeach; ?>
                                    <?php endif; ?>
                                </ul>
                            </div>
                        </td>
                        <td class="d-none d-md-table-cell px-md-2">
                            <p class="text-nowrap font-800 price-js mb-0">
                                $<?php echo esc_attr(prodigy_price_format( $product->actual_price )); ?>
                            </p>
                        </td>
                        <td class="d-none d-md-table-cell">
                            <div class="prodigy-counter pl-12 pr-12">
                                <button
                                        class="main-button main-button--counter main-button--counter-minus icon icon-minus counter-minus-js"
                                        type="button"
                                    <?php echo ( $product->count_items == 1 ) ? 'disabled' : '' ?>
                                ></button>
                                <input class="prodigy-counter__total main-input counter-count-js" type="text" value="<?php echo esc_attr($product->count_items); ?>"  maxlength="3"/>
                                <button class="main-button main-button--counter main-button--counter-plus icon icon-plus counter-plus-js" type="button">
                                </button>
                            </div>
                        </td>
                        <td class="text-right d-none d-md-table-cell px-md-2">
                        <span class="text-nowrap font-800 total-price-js">
                            $<?php echo esc_attr( prodigy_price_format( $product->total_price ));?>
                        </span>
                        </td>
                        <td class="prodigy-cart__table-del-btn-cell">
                            <button
                                    class="icon-btn icon icon-small-close remove-item-js"
                                    data-cart-item = '<?php echo json_encode($data_analytics) ?>'
                                    type="button"
                                    data-local-id="<?php echo esc_attr( $product->get_field( 'ID' )); ?>"
                                    data-remote-id="<?php echo esc_attr($remote_product_id); ?>"
                            >
                            </button>
                        </td>
                    </tr>
                <?php endforeach; ?>
                </tbody>
            </table>
        </div>
    </div>

    <div class="cart-subtotal-js">
        <h4 class="mb-20"><?php esc_html_e('CART TOTAL', 'prodigy'); ?></h4>
        <div class="prodigy-cart__total mb-40">
            <p class="prodigy-cart__total-info">
                <span><?php esc_html_e('Subtotal', 'prodigy'); ?></span>
                <span class="subtotal-price-js">$<?php echo esc_attr( prodigy_price_format( $subtotal_price )); ?></span>
            </p>
            <div class="mb-20 mt-20 divider"></div>
            <div class="d-flex flex-column flex-md-row align-items-center justify-content-between">
                <a
                        class="main-button main-button--outline continue-cart-js mb-16 flex-grow-1 flex-md-grow-0"
                        href="<?php echo esc_url( get_permalink( get_page_by_title( 'shop' ) ) ); ?>"
                >
                    <i class="icon icon-arrow-left mr-12"></i>
                    CONTINUE SHOPPING
                </a>
                <a
                        class="main main-button mb-16 flex-grow-1 flex-md-grow-0 checkout-button-js"
                        href="<?php echo esc_url($GLOBALS['proceed_url']); ?>"
                >
                    <?php esc_html_e( 'PROCEED TO CHECKOUT', 'prodigy' ); ?>
                </a>
            </div>
        </div>
    </div>

<?php endif; ?>



