<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

    global $product;

    if ( ! comments_open() ) {
        return;
    }

    if ( isset( $_COOKIE['prodigy_user'] ) ) {
        $author_name = $wpdb->get_results(
                $wpdb->prepare(
                "SELECT name FROM {$wpdb->prefix}prodigy_user_info WHERE user_session_hash=%s;", sanitize_text_field( $_COOKIE['prodigy_user'] )
                )
        );

        $author_name_value = isset( $author_name[0] ) ? $author_name[0]->name : '';
    }

    $product = $GLOBALS['prodigy_product'];
    $post_title = $product->get_field('post_title');
    $post_id = $product->get_field('ID');

    $rating_count = $product->get_count_rating();
    $review_count = $product->get_count_reviews();
    $average      = $product->get_average_rating();
    $full_size      = array_values( array(
        'width'  => 1600,
        'height' => 1600,
        'crop'   => 1,
    ) );
    $thumburl_info = wp_get_attachment_image_src( $product->get_meta_field('_thumbnail_id'), $full_size );
    if (is_array($thumburl_info)) {
        $thumburl = array_shift( $thumburl_info );
    }
?>


<div class="prodigy-reviews-ratings">
    <div>
        <p class="prodigy-reviews-ratings__title">Average Customer Ratings</p>

        <div class="prodigy-reviews-ratings__rating">
            <div class="d-flex align-items-md-center">
                <?php if ( get_option( 'pg_product_rating' ) ): ?>
                    <?php echo pg_get_rating_html( $average, $rating_count ); ?>
                <?php endif; ?>

                <?php if ( comments_open() ) : ?>
                    <?php if ( get_option( 'pg_product_review' )): ?>
                        <p class="prodigy-reviews-ratings__average">
                            <?php echo $average; ?>
                        </p>
                    <?php endif ?>
                <?php endif ?>
            </div>

            <p class="prodigy-reviews-ratings__count count">
                <?php printf( _n( '%s review', '%s reviews', $review_count, 'prodigy' ), '' .esc_html( $review_count )) ?>
            </p>
        </div>
    </div>

    <button class="main-button" type="button" data-toggle="modal" data-target="#myModal">Write a review</button>
</div>

<div id="reviews">
    <ul id="comments" class="prodigy-review">

        <?php if ( have_comments() ) : ?>
            <?php

            $comments = get_comments(
                [ 'post_id' => $post_id ]
            );

            wp_list_comments(
                [
                    'callback' => 'prodigy_comments',
                    'per_page' => Prodigy_Product_Page::COUNT_COMMENTS_ON_PAGE_PRODUCT,
                    'page' => 1
                ],
                $comments
            );

            ?>

            <?php if ( get_comment_pages_count() > 1 ) :
                echo '<nav class="prodigy-pagination">';
                paginate_comments_links( apply_filters( 'prodigy_comment_pagination_args', array(
                    'prev_text' => '&larr;',
                    'next_text' => '&rarr;',
                    'type'      => 'list',
                ) ) );
                echo '</nav>';
            endif; ?>
        <?php endif; ?>
        <div class="d-flex align-items-center justify-content-center">
            <div class="d-flex align-items-center cursor-pointer link-show-more-reviews-js">
                <span data-page="2" class="per-page-js">Show more reviews</span>
                <i class="icon icon-arrow-down"></i>
            </div>
        </div>
    </ul>
</div>

<div class="modal fade prodigy-review-modal" id="myModal">
    <div class="modal-dialog modal-dialog-centered prodigy-review-modal__dialog">
        <div class="container">
            <div class="row">
                <div class="modal-content prodigy-review-modal__content col-12 col-lg-8 offset-lg-2">
                    <div class="prodigy-review-modal__img-wrp d-none d-md-block">
                        <?php if ( ! empty( $thumburl ) ) : ?>
                            <img src="<?php echo esc_url( $thumburl ); ?>" class="mb-20" alt="<?php echo esc_attr($post_title); ?>">
                        <?php else: ?>
                            <i class="icon icon-image mb-20 prodigy-review-modal__img-wrp-icon" style="font-size: 100px;"></i>
                        <?php endif; ?>
                        <p class="prodigy-review-modal__img-caption"><?php echo esc_attr($post_title); ?></p>
                    </div>
                    <div class="prodigy-review-modal__body">
                        <a role="button" rel="noopener" class="icon icon-small-close cursor-pointer prodigy-review-modal__icon-close" data-dismiss="modal" aria-label="Close"></a>
                        <div id="review_form_wrapper" class="prodigy-review-modal__form">
                            <div id="review_form" class="prodigy-comment">
                                <?php
                                $commenter = wp_get_current_commenter();

                                /*
                                * Format standard comments output
                                */
                                $comment_form = [
                                    'fields'       => [],
                                    'submit_field' => '<div class="form-submit"  style="clear: both">%1$s %2$s</div>',
                                    'class_submit' => 'main-button submit-product-button cursor-pointer',
                                    'label_submit' => esc_html__( 'Post a review', 'prodigy' ),
                                    'comment_notes_before' => ''
                                ];

                                /*
                                * Google captcha
                                */
                                if ( ! empty( get_option( 'pg_captcha_launch' ) ) ) {
                                    $comment_form['comment_notes_after'] =
                                        '<div class="g-recaptcha mb-20" data-callback="prodigyRecaptchaCallback" data-sitekey="' . get_option( 'pg_captcha_site_key' ) . '"></div>';
                                }

                                $is_enable_rating              = get_option( 'pg_product_rating' );
                                $user                          = wp_get_current_user();

                                /*
                                * Format standard comments output
                                */
                                $comment_form['comment_field'] = '';
                                if ( ! empty( $is_enable_rating ) && $user->roles[0] !== 'administrator' ) {
                                    $comment_form['comment_field'] =
										'<div class="prodigy-review-modal__body-section comment-form-rating d-md-flex align-items-md-center">
											<label class="prodigy-review-modal__body-label mr-16 mb-md-0" for="rating">
												' . esc_html__( 'Product rating *', 'prodigy' ) . '
											</label>
											<div class="d-flex align-items-center">
												<select name="rating" id="prodigy-rating" required>
													<option value="">' . esc_html__( 'Rate&hellip;', 'prodigy' ) . '</option>
													<option value="5">' . esc_html__( 'Perfect', 'prodigy' ) . '</option>
													<option value="4">' . esc_html__( 'Good', 'prodigy' ) . '</option>
													<option value="3">' . esc_html__( 'Average', 'prodigy' ) . '</option>
													<option value="2">' . esc_html__( 'Not that bad', 'prodigy' ) . '</option>
													<option value="1">' . esc_html__( 'Very poor', 'prodigy' ) . '</option>
												</select>
												<small class="font-14 ml-16 d-none d-md-inline-block">Click to rate</small>
											</div>
                                        </div>';
                                }

                                $comment_form['comment_field'] .=
                                    '<div class="prodigy-review-modal__body-section">' . '
                                        <label class="prodigy-review-modal__body-label" for="author">' . esc_html__( 'Your name *', 'prodigy' ) . '
                                        </label>
                                        <input id="author" class="prodigy-review-modal__body-section-text  comment-author-js" name="author" type="text" value="' . $author_name_value . '" size="30" required />
                                    </div>';

                                $comment_form['comment_field'] .=
                                    '<div class="prodigy-review-modal__body-section mb-20">
                                        <label class="prodigy-review-modal__body-label" for="comment">
                                            ' . esc_html__( 'Review *', 'prodigy' ) . '&nbsp;
                                        </label>
                                        <textarea
                                            placeholder="' . esc_html__( 'Write your review here...', 'prodigy' ) . '"
                                            id="comment"
                                            class="prodigy-review-modal__body-section-text prodigy-comment__text"
                                            name="comment"
                                            cols="45"
                                            rows="8"
                                            required></textarea>
                                    </div>';

                                comment_form( $comment_form );
                                ?>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>
