jQuery(document).ready(function ($) {

    var meta_image_frame;
    var meta_mobileimage_frame;

    // Runs when the image button is clicked.
    $('#prodigy_image_button').click(function(e){

        // Prevents the default action from occuring.
        e.preventDefault();

        // If the frame already exists, re-open it.
        if ( meta_image_frame ) {
            meta_image_frame.open();
            return;
        }

        // Sets up the media library frame
        meta_image_frame = wp.media.frames.meta_image_frame = wp.media({
            title: prodigy_image.title,
            button: { text:  prodigy_image.button },
            library: { type: 'image' },
        });

        // Runs when an image is selected.
        meta_image_frame.on('select', function() {

            // Grabs the attachment selection and creates a JSON representation of the model.
            var media_attachment = meta_image_frame.state().get('selection').first().toJSON();

            // Sends the attachment URL to our custom image input field.
            $('#prodigy_image').val(media_attachment.url);
            $('.prodigy-image-container').append('<span class="prodigy-close"></span>');
            // If the image is smaller than the thumbnail size, use the main URL
            if (typeof media_attachment.sizes.thumbnail === 'undefined') {
                $('#prodigy_image_src').attr('src', media_attachment.url);
            } else {
                $('#prodigy_image_src').attr('src', media_attachment.sizes.thumbnail.url);
            }
        });

        // Opens the media library frame.
        meta_image_frame.open();
    });

    $('#prodigy-mobileimage__button').click(function(e){

        // Prevents the default action from occuring.
        e.preventDefault();

        // If the frame already exists, re-open it.
        if ( meta_mobileimage_frame ) {
            meta_mobileimage_frame.open();
            return;
        }

        // Sets up the media library frame
        meta_mobileimage_frame = wp.media.frames.meta_mobileimage_frame = wp.media({
            title: prodigy_mobileimage.title,
            button: { text:  prodigy_mobileimage.button },
            library: { type: 'image' },
        });

        // Runs when an image is selected.
        meta_mobileimage_frame.on('select', function(){

            // Grabs the attachment selection and creates a JSON representation of the model.
            var media_attachment = meta_mobileimage_frame.state().get('selection').first().toJSON();

            // Sends the attachment URL to our custom image input field.
            $('#prodigy_mobileimage').val(media_attachment.url);
            $('.prodigy-image-container--mobile').append('<span class="prodigy-close"></span>');
            // If the image is smaller than the thumbnail size, use the main URL
            if (typeof media_attachment.sizes.thumbnail === 'undefined') {
                $('#prodigy_mobileimage_src').attr('src', media_attachment.url);
            } else {
                $('#prodigy_mobileimage_src').attr('src', media_attachment.sizes.thumbnail.url);
            }
        });

        // Opens the media library frame.
        meta_mobileimage_frame.open();
    });

    var meta_gallery_frame;
    // Runs when the image button is clicked.
    $('#prodigy-gallery__button').click(function(e){

        //Attachment.sizes.thumbnail.url/ Prevents the default action from occuring.
        e.preventDefault();

        // If the frame already exists, re-open it.
        if ( meta_gallery_frame ) {
            meta_gallery_frame.open();
            return;
        }

        // Sets up the media library frame
        meta_gallery_frame = wp.media.frames.meta_gallery_frame = wp.media({
            title: prodigy_gallery.title,
            button: { text:  prodigy_gallery.button },
            library: { type: 'image' },
            multiple: true,
            states: [
                new wp.media.controller.Library({
                    title: 'Add images to product gallery',
                    filterable: 'all',
                    multiple: true
                })
            ]
        });

        meta_gallery_frame.on('open', function() {
            var selection = meta_gallery_frame.state().get('selection');
            var library = meta_gallery_frame.state('gallery-edit').get('library');
            var ids = $('#prodigy_gallery').val();
            if (ids) {
                idsArray = ids.split(',');
                idsArray.forEach(function(id) {
                    attachment = wp.media.attachment(id);
                    attachment.fetch();
                    selection.add( attachment ? [ attachment ] : [] );
                });
            }
        });

        meta_gallery_frame.on('ready', function() {
            $( '.media-modal' ).addClass( 'no-sidebar' );
            //fixBackButton();
        });

        // When an image is selected, run a callback.
        //meta_gallery_frame.on('update', function() {
        meta_gallery_frame.on('select', function() {
            var imageIDArray = [];
            var imageHTML = '';
            var metadataString = '';
            images = meta_gallery_frame.state().get('selection');
            imageHTML += '<ul class="prodigy-gallery__list">';
            images.each(function(attachment) {
                imageIDArray.push(attachment.attributes.id);
                //imageHTML += '<li>'+button+'<img id="'+attachment.attributes.id+'" src="'+attachment.attributes.url+'"></li>';
                if (typeof attachment.attributes.sizes.thumbnail === 'undefined') {
                    imageHTML += '<li><div class="prodigy-gallery-container"><span class="prodigy-gallery__button-close"><img id="'+attachment.attributes.id+'" src="'+attachment.attributes.url+'"></span></div></li>';
                } else {
                    imageHTML += '<li><div class="prodigy-gallery-container"><span class="prodigy-gallery__button-close"><img id="'+attachment.attributes.id+'" src="'+attachment.attributes.sizes.thumbnail.url+'"></span></div></li>';
                }
            });
            imageHTML += '</ul>';
            metadataString = imageIDArray.join(",");
            if (metadataString) {
                $("#prodigy_gallery").val(metadataString);
                $("#prodigy_gallery_src").html(imageHTML);
            }
        });

        // Finally, open the modal
        meta_gallery_frame.open();

    });

    $(document.body).on('click', '.prodigy-close', function(event){
        event.preventDefault();
        if (confirm('Are you sure you want to remove this image?')) {
            $('.prodigy_container').remove();
            $("#prodigy_image").val('');
        }
    });

    $(document.body).on('click', '.prodigy-close', function(event){
        event.preventDefault();
        if (confirm('Are you sure you want to remove this image?')) {
            $('.prodigy-image-container--mobile').remove();
            $("#prodigy_mobileimage").val('');
        }
    });


    $(document.body).on('click', '.prodigy-gallery__button-close', function(event){
        event.preventDefault();
        if (confirm('Are you sure you want to remove this image?')) {
            var removedImage = $(this).children('img').attr('id');
            var oldGallery = $("#prodigy_gallery").val();
            var newGallery = oldGallery.replace(','+removedImage,'').replace(removedImage+',','').replace(removedImage,'');
            $(this).parents().eq(1).remove();
            $("#prodigy_gallery").val(newGallery);

            $.ajax({
                url: ajaxurl,
                type: 'post',
                data: {
                    action: 'prodigy-delete-variant-image',
                    post_id: $('.current_post').val(),
                    remove_image: removedImage,
                    nonce_code : prodigyajax.nonce
                },
                success: function (data) {
                    if (data) {
                        $('.custom_media_url').each(function () {
                            if ($(this).data('id') == removedImage) {
                                $(this).next().find('img').remove();
                                $(this).remove();
                            }
                        })
                    }
                }
            });
        }
    });
});
