jQuery(document).ready(function ($) {
    $('#product-form').validate({
        rules: {
            pg_custom_expiration_time: {
                required: true
            },
        },
        errorPlacement: function(error,element) {
            return true;
        },
        highlight: function (element, errorClass, validClass) {
            $(element).addClass('form-error-input');
            return false;  // ensure this function stops
        },
        unhighlight: function (element, errorClass, validClass) {
            $(element).removeClass('form-error-input');
            return false;  // ensure this function stops
        },
        submitHandler: function( form ) {
            if ($(form).valid()) {
                form.submit();
                showMessage($('button[type=submit]'));
            } else {
                $('html, body').animate({
                    scrollTop: ($('form .form-error-input').offset().top - 300)
                }, 200);

                return false;
            }
        },
    });

    var expiration_time_input = $('.expiration-custom-js');

        expiration_time_input.inputmask('integer', {
            max: 9999,rightAlign: false, placeholderText: 'from 1h to 9999h'
        });

    var expiration_time_select = $('.expiration-time-js');
    expiration_time_select.change(function () {
       if ($(this).val() === 'custom') {
           expiration_time_input.show();
       } else {
           expiration_time_input.hide();
       }
    });

    if (expiration_time_select.val() === 'custom') {
        expiration_time_input.show();
    } else {
        expiration_time_input.hide();
        expiration_time_input.val('');
    }

    var captcha_block = $('.captcha-block-js');

    if ($('.captcha-launch-js').prop('checked') == true) {
        captcha_block.show();
    } else {
        captcha_block.hide();
    }

    $('.captcha-launch-js').click(function () {
        if ($(this).prop('checked') == true) {
            captcha_block.show();
        } else {
            captcha_block.hide();
        }
    });


    // for setting page
    var currentUrl = document.URL;
    current_form = prodigyGetUrlParam('tab');

    $(document).on("submit", "form", function (e) {
        window.onbeforeunload = null;
    });



    $('#saveModal').dialog({
        title: 'Save Settings',
        autoOpen: false,
        draggable: false,
        width: 'auto',
        modal: true,
        resizable: false,
        closeOnEscape: true,
        position: {
            my: "center",
            at: "center",
            of: window
        }
    });

    checkDataFormToUpdate();

    // check for prodigy settings page
    if (currentUrl.indexOf('prodigy_settings') != -1) {
        var url = new URL(currentUrl);
        var tabParam = url.searchParams.get("tab");
        if (tabParam == null) {
            $('#general-tab').addClass('nav-tab-active');
            $('#general').removeAttr("style");
        }

        if (tabParam == 'product') {
            $('#manage_stock').click(function () {
                if ($(this).prop('checked') == true) {
                    $('.manage-visible').hide();
                } else {
                    $('.manage-visible').show();
                }
            })
        }
    }

    let pg_store_key = $("input[name=pg_store_key]").val();

    if (!pg_store_key) {
        $('#connect_store_key_block').show();
    }

    $('#connect_store_key').on('click', function (event) {
        let pg_store_key = $("input[name=pg_store_key]").val();
        connect_to_store(pg_store_key);
    });

    $('#change_store_key').on('click', function (event) {
        $('#connect_store_key_block').show();
    });

    $('#disconnect_store_key').on('click', function (event) {
        let pg_store_key = $("input[name=pg_store_key]").val();
        disconnect_store_key(pg_store_key);
    });

    $('.update-store-js').on('click', function (event) {
        update_store_info();
    });

    $('.prodigy-type-slug-js').on('change', function (event) {
        validate_type_url($(this).val());
    });

    $('.update-current-migration-js').on('click', function (event) {
        update_current_migration();
    });

    $('.setup-to-wizard-js').on('click', function (event) {
        setup_to_wizard();
    });

    $('.refresh-permalink-prodigy-js').on('click', function (event) {
        refresh_permalink_prodigy_js();
    });

    function setup_to_wizard() {
        window.location.href = url_redirect;
    }

    function refresh_permalink_prodigy_js() {
        window.location.href = refresh_permalink_prodigy;
    }

    function connect_to_store(pg_store_key) {
        $.ajax({
            dataType: 'json',
            url: ajaxurl,
            type: 'post',
            data: {
                action: 'connect-store-prodigy',
                pg_store_key: pg_store_key,
                nonce_code : prodigyajax.nonce
            },
            success: function () {
                $('.error-message').hide();
                $('#connect_store_key_block').hide();
                $('#pg_store_key_view').text(pg_store_key);
                $('#change_store_key_block').show();
            },
            fail: function () {
                $('.error-message').show();
            },
            statusCode: {
                404: function (data) {
                    alert(data.responseJSON.data);
                },
                422: function (data) {
                    alert(data.responseJSON.data);
                },
            }
        });
    }

    function disconnect_store_key(pg_store_key) {
        $.ajax({
            dataType: 'json',
            url: ajaxurl,
            type: 'post',
            data: {
                action: 'disconnect-store-key-prodigy',
                pg_store_key: pg_store_key,
                nonce_code : prodigyajax.nonce
            },
            success: function () {
                $('#connect_store_key_block').show();
                $('#pg_store_key_view').text('');
                $("input[name=pg_store_key]").val('');
            },
            fail: function () {
                $('.error-message').show();
            },
            statusCode: {
                404: function (data) {
                    alert(data.responseJSON.data);
                },
                422: function (data) {
                    alert(data.responseJSON.data);
                },
            }
        });
    }


    /**
     * Update store info from hosted system
     */
    function update_store_info() {

        $.ajax({
            dataType: 'json',
            url: ajaxurl,
            type: 'post',
            data: {
                nonce_code : prodigyajax.nonce,
                action: 'update-store-info'
            },
            success: function (data) {
                alert(data.data.message);
                $('.name-store-js').text(data.data.pg_domain_hosted_system);
                $('.subdomen-store-js').text(data.data.pg_url_domain_hosted_system);
            },
            fail: function () {
                $('.error-message').show();
            },
            statusCode: {
                404: function (data) {
                    alert(data.responseJSON.data);
                },
                422: function (data) {
                    alert(data.responseJSON.data);
                },
            }
        });
    }


    /**
     * Update store info from hosted system
     */
    function update_current_migration() {
        $.ajax({
            dataType: 'json',
            url: ajaxurl,
            type: 'post',
            data: {
                action: 'update-current-migration',
                key: 'update_current_migration',
                nonce_code : prodigyajax.nonce
            },
            success: function (data) {
                alert(data.data);
                $('.migrate-db-js').remove();
            },
            fail: function () {
                $('.error-message').show();
            },
            statusCode: {
                404: function (data) {
                    alert(data.responseJSON.data);
                },
                422: function (data) {
                    alert(data.responseJSON.data);
                },
            }
        });
    }


    /**
     * Update store info from hosted system
     */
    function validate_type_url(slug) {

        $.ajax({
            dataType: 'json',
            url: ajaxurl,
            type: 'post',
            data: {
                action: 'validate-type-url',
                nonce_code : prodigyajax.nonce,
                slug: slug
            },
            success: function (data) {
            },
            fail: function (data) {
                $('.error-message').show();
            },
            statusCode: {
                404: function (data) {
                    alert(data.responseJSON.data);
                },
                422: function (data) {
                    alert(data.responseJSON.data);
                },
            }
        });
    }


    function showMessage ( button ) {
        button.click(function () {
            localStorage.reloadAfterPageLoad = true;
            window.location.reload();
        });

        if (localStorage.reloadAfterPageLoad) {
            $('#show-message').show();
            localStorage.reloadAfterPageLoad = false;
            localStorage.clear();
        }
    }

    function checkDataFormToUpdate() {

        let current_form = prodigyGetUrlParam( 'tab' );
        let isShowConfirm = false;
        window.isDirty = false;

        $( '#' + current_form + '-form :input:not([type=submit])' ).each( function() {

            if ( $( this ).is( ':radio' ) || $( this ).is( ':checkbox' ) ) {
                var checked = $( this ).is( ':checked' ) ? 'checked' : 'unchecked';
                $( this ).data(  'initialValue', checked );
            } else {
                $( this ).data( 'initialValue', $( this ).val() );
            }
        });


        $( 'a' ).click( function ( event ) {
            window.isDirty = false;
            let currentValue = null;

            $( '#' + current_form + '-form :input:not([type=submit])' ).each( function () {
                if ( $( this ).is( ':radio' ) || $( this ).is( ':checkbox' ) ) {
                    currentValue =  $( this ).is( ':checked' ) ? 'checked' : 'unchecked';
                } else {
                    currentValue = $( this ).val();
                }

                if ( $( this ).data( 'initialValue' ) !== currentValue ) {
                    window.isDirty = true;
                }
            });

            if ( window.isDirty === true ) {

                let currentUrl = document.URL;
                if ( currentUrl.indexOf('prodigy_settings') != -1 ) {
                    event.preventDefault();

                    $( '#saveModal' ).dialog( 'open' );
                }
            } else {
                window.isDirty = false;
            }

            var url = $( this ).attr( 'href' );

            $(document).on("click", '.settings-button-save', function() {
                let $form = $( '#' + current_form + '-form' );
                let data = $form.serialize();
                $.post( window.location.href, data, function () {
                    window.location.href = '/wp-admin/' + url;
                } );
            });

            $( '.close-setting-popup' ).click( url, function ( e ) {
                e.preventDefault();
                window.location.href = '/wp-admin/' + url;
            });

        });
    }
});
