(function($, window) {
  var prodigy_product = {
    _weight_type_mapper: ["lbs", "g", "kg", "oz"],

    _dimension_type_mapper: ["in", "cm"],

    _stock_status_mapper: ["IN STOCK", "OUT OF STOCK"],

    _add_to_cart_key: "add_item_to_cart",

    product: {},
    variant: {},

    is_admin: false,

    checkout_url: "",

    init: function() {
      var current_url = window.location.pathname.replace(/\/+$/, "");

      if (current_url.includes(product_type) || current_url.includes(category_type)) {
        this.check_current_user_is_admin();
        this.set_global_product();
        this.set_product_counter();
        this.set_product_gallery();
        this.set_attributes_behavior();
        this.set_product_tabs();
        this.show_tab_review();
        this.send_captcha();
        this.disable_submit_comment();
        this.reset_focus_to_press_enter();
        this.recalculate_subtotal_amount_cart_widget();
        this.set_default_link_review();
        this.init_show_more();
        this.change_product_quantity_analytic_event();
      }
    },


    set_analytic_add_item_to_cart: function() {
      if (options.pg_google_track_id) {

        var current_url = window.location.pathname.replace(/\/+$/, "");


        if (current_url.includes(product_type)) {
          var self = this;
          if (!$.isEmptyObject(self.variant)) {
            var product = self.variant;
            var regular_price = (typeof product.product_regular_price !== 'undefined') ? product.product_regular_price[0] : '';
            var sale_price = (typeof product.product_sale_price !== 'undefined') ? product.product_sale_price[0] : '';

            gtag('event', 'add_to_cart', {
              "event_category": 'prodigy_ecommerce',
              "items": [
                {
                  "id": product.variant_remote_id[0],
                  "name": product.post_title[0],
                  "category": product.categories,
                  "variant": product.product_sku[0],
                  "price": self.get_actual_price(regular_price, sale_price),
                  "quantity": parseInt($('.counter-count-js').val()),
                }
              ]
            });
          } else {
            var product = self.product;
            var regular_price = (typeof product.meta.product_regular_price !== 'undefined') ? product.meta.product_regular_price[0] : '';
            var sale_price = (typeof product.meta.product_sale_price !== 'undefined') ? product.meta.product_sale_price[0] : '';
            gtag('event', 'add_to_cart', {
              "event_category": 'prodigy_ecommerce',
              "items": [
                {
                  "id": product.variant_product_id,
                  "name": product.fields.post_title,
                  "list_name": (typeof product.meta.product_sku !== 'undefined') ? product.meta.product_sku[0] : '',
                  "category": product.categories,
                  "variant": '',
                  "price": self.get_actual_price(regular_price, sale_price),
                  "quantity": parseInt($('.counter-count-js').val()),
                }
              ]
            });
          }
        }
      }
    },

    change_product_quantity_analytic_event: function() {
      if (options.pg_google_track_id) {
        var current_url = window.location.pathname.replace(/\/+$/, "");

        if (current_url.includes(product_type)) {
          var self = this;

          var old_quantity = parseInt($('.counter-count-js').val());

          $('.counter-btn-plus-js, .counter-btn-minus-js').on('click', function () {
            if (!$.isEmptyObject(self.variant)) {
              var product = self.variant;
              var regular_price = (typeof product.product_regular_price !== 'undefined') ? product.product_regular_price[0] : '';
              var sale_price = (typeof product.product_sale_price !== 'undefined') ? product.product_sale_price[0] : '';
              gtag('event', 'change_product_quantity', {
                "event_category": 'prodigy_product',
                "items": [
                  {
                    "id": product.variant_remote_id[0],
                    "name": product.post_title[0],
                    "category": product.categories,
                    "sku": product.product_sku[0],
                    "price": self.get_actual_price(regular_price, sale_price),
                    "new_quantity": parseInt($('.counter-count-js').val()),
                    "old_quantity": old_quantity,
                  }
                ]
              });
            } else {
              var product = self.product;
              var regular_price = (typeof product.meta.product_regular_price !== 'undefined') ? product.meta.product_regular_price[0] : '';
              var sale_price = (typeof product.meta.product_sale_price !== 'undefined') ? product.meta.product_sale_price[0] : '';

              gtag('event', 'change_product_quantity', {
                "event_category": 'prodigy_product',
                "items": [
                  {
                    "id": product.variant_product_id,
                    "name": product.fields.post_title,
                    "sku": (typeof product.meta.product_sku !== 'undefined') ? product.meta.product_sku[0] : '',
                    "category": product.categories,
                    "variant": '',
                    "price": self.get_actual_price(regular_price, sale_price),
                    "new_quantity": parseInt($('.counter-count-js').val()),
                    "old_quantity": old_quantity,
                  }
                ]
              });
            }
          });
        }
      }
    },

    set_view_product_analytic_event: function(data) {
      var current_url = window.location.pathname.replace(/\/+$/, "");

      var regular_price = (typeof data.product_regular_price !== 'undefined') ? data.product_regular_price[0] : '';
      var sale_price = (typeof data.product_sale_price !== 'undefined') ? data.product_sale_price[0] : '';

      if (current_url.includes(product_type)) {
        var self = this;

          gtag('event', 'view_variant', {
            "event_category": 'prodigy_product',
            "items": [
              {
                "id": data.variant_remote_id[0],
                "name": data.post_title[0],
                "category": data.categories,
                "variant": data.product_sku[0],
                "price": self.get_actual_price(regular_price, sale_price)
              }
            ]
          });
      }
    },

    captcha_callback: function(val) {
      $(".recaptcha-checkbox").attr("aria-checked", true);
      this.check_enable_comment_fields();
    },

    reset_focus_to_press_enter: function() {
      $(document).on("keypress", ".counter-count-js", function(e) {
        if (e.which === 13) {
          if ($(this).is(":focus")) {
            if (parseInt($(this).val()) > 1) {
              $(".counter-btn-minus-js").prop("disabled", false);
              $(".counter-btn-minus-js").removeAttr("style");
            }
            $(this).blur();
          }
        }
      });
    },

    send_captcha: function() {
      $(".submit-product-button").click(function(e) {
        var response;
        $.ajax({
          type: "post",
          data: $("#commentform").serialize() + "&action=google-captcha-url",
          dataType: "json",
          url: ajax_url,
          async: false,
          success: function(data) {
            if (data.nocaptcha === "true") {
              response = 1;
            } else if (data.spam === "true") {
              response = 1;
            } else {
              response = 0;
            }
          }
        });
      });
    },

    // Send products to remote cart
    add_product_to_cart: function(
      form_count_products,
      local_product_id,
      remote_product_id,
      attributes_item,
      price
    ) {
      var self = this;

      var post_data = {
        action: "prodigy-check-product",
        product_id: local_product_id,
        nonce_code : prodigyajax.nonce
      };

      var result = [];

      $.ajax({
        type: "post",
        data: post_data,
        dataType: "json",
        url: ajax_url,
        success: function(data) {
          if (!data.success) {
            self.show_error("Current product has been removed!", "error");
          } else {
            self.add_to_cart(
              form_count_products,
              local_product_id,
              remote_product_id,
              attributes_item,
              price
            );
          }
        }
      });
    },

    add_to_cart: function(
      form_count_products,
      local_product_id,
      remote_product_id,
      attributes_item,
      price
    ) {
      var self = this;

      var post_data = {
        action: "prodigy-add-remote-cart",
        local_product_id: local_product_id,
        remote_product_id: remote_product_id,
        count: form_count_products,
        price: price,
        attributes: attributes_item
      };

      $.ajax({
        type: "post",
        data: post_data,
        dataType: "json",
        url: ajax_url,
        success: function(data) {
          if (data !== null) {
            if (window.prodigy_cart_widget !== undefined) {
              window.prodigy_cart_widget.add_to_cart(true);
            }
            self.show_error(data.error);
          } else {
            if (window.prodigy_cart_widget !== undefined) {
              window.prodigy_cart_widget.add_to_cart(false);
            }
            self.show_add_to_cart_message();
            // redirect to cart page
            if (
              $(".cart-redirect-js").data("cart-redirect") ===
              "redirect_to_cart"
            ) {
              var cart_url = window.location.origin + "/cart";
              window.location.replace(cart_url);
            }
            self.reload_widget_content();
          }
        }
      });
    },

    show_add_to_cart_message() {
      var add_to_cart_button = $(".add-to-cart-js");
      add_to_cart_button.text("ADDED TO CART");
      add_to_cart_button.prop("disabled", true);
      add_to_cart_button.css("background", "grey");

      setTimeout(function() {
        add_to_cart_button.prop("disabled", false);
        add_to_cart_button.text("ADD TO CART");
        add_to_cart_button.removeAttr("style");
      }, 5000);
    },

    // reload cart widget content
    reload_widget_content: function() {
      var self = this;
      $(".cart-widget-item-list-js").load(
        getCurrentUrl() + " .cart-widget-item-list-js>*",
        function() {
          self.recalculate_subtotal_amount_cart_widget();
          var checkout_url = self.set_checkout_url();
          $(".checkout-url-js").prop("href", checkout_url);
        }
      );
    },

    set_checkout_url: function() {
      var post_data = {
        action: "prodigy-get-checkout-url"
      };
      var result;
      $.ajax({
        type: "post",
        data: post_data,
        async: false,
        dataType: "json",
        url: ajax_url,
        success: function(data) {
          if (data.url !== "") {
            result = data.url;
          }
        },
        error: function() {
          result = "";
        }
      });

      return result;
    },

    recalculate_subtotal_amount_cart_widget: function() {
      var self = this;
      var subtotal_amount = 0;
      $(".widget-cart-product-price-js").each(function() {
        let amount = parse_price(
          $(this)
            .find(".widget-total-price-js")
            .text()
        );
        let count = parseInt(
          $(this)
            .find(".widget-cart-count-item-js")
            .text()
        );
        subtotal_amount += amount * count;
      });

      $(".widget-subtotal-price-js").text(
        "$" + prodigy_price_format(subtotal_amount)
      );
      localStorage.setItem("subtotal_amount", "$" + prodigy_price_format(subtotal_amount));
    },

    show_error: function(message, type = "success") {
      var add_to_cart_message = $(".widget-cart-message-error-js");
      add_to_cart_message.find("span").html(message);
      add_to_cart_message.show();
      add_to_cart_message.delay(5000).fadeOut("slow");
      if (type === "error") {
        add_to_cart_message.addClass("prodigy-cart-dropdown__error-alert");
      }
    },

    add_item_to_cart: function() {
      var self = this;


      var called = false;
      if (!called) {
        $(document)
          .off("click", "button.add-to-cart-js")
          .on("click", "button.add-to-cart-js", function() {
            called = true;
            // if isset variants
            if ($(".attribute_values_js").length > 0) {
              var local_product_id = $(this).attr("data-local-id");
              var remote_product_id = $(this).attr("data-remote-id");
            } else {
              var local_product_id = self.product.fields.ID;
              var remote_product_id = self.product.variant_product_id;
            }

            self.set_analytic_add_item_to_cart();

            var price_str = $(".sale-price-container-js")
              .text()
              .trim();
            var price = parse_price(price_str);
            var form_count_products = $(".counter-count-js").val();

            var attributes = { attribute: {}, item: {} };

            $(".attribute_values_js").each(function(key) {
              var $option = $(this).find("option:selected");
              attributes["attribute"][key] = $option.val();
              attributes["item"][key] = $option.text();
            });

            // set items in remote cart
            self.add_product_to_cart(
              form_count_products,
              local_product_id,
              remote_product_id,
              attributes,
              price
            );
          });
      }
    },

    disable_submit_comment: function() {
      var self = this;

      $("#comment").on("input", function(e) {
        self.check_enable_comment_fields();
      });

      $(".comment-author-js").on("input", function(e) {
        self.check_enable_comment_fields();
      });

      $("body").on("change", "#prodigy-rating", function(e) {
        self.check_enable_comment_fields();
      });
    },

    check_enable_comment_fields: function() {
      var comment_submit = $(".main-button.submit-product-button");

      var rating_form = $(".comment-form-rating").length;

      if (!this.is_admin) {
        var rating_val = $("#prodigy-rating").val();
      }

      var comment_val = $("#comment").val();
      var name_val = $(".comment-author-js").val();
      var captcha_val = $("#g-recaptcha-response").val();
      var captcha = $("#g-recaptcha-response");
      var is_rating_enable = $(".prodigy-comment__rating").length > 0;

      if (document.body.classList.contains("logged-in")) {
        if (this.is_admin) {
          var enable_submit = comment_val;
        } else {
          var enable_submit = rating_val && comment_val;
        }
        if (captcha.length > 0 && !this.is_admin) {
          var enable_submit = rating_val && comment_val && captcha_val;
        } else if (captcha.length > 0 && this.is_admin) {
          var enable_submit = comment_val && captcha_val;
        }
      } else {
        if (typeof rating_val !== "undefined") {
          var enable_submit = rating_val && comment_val && name_val;

          if (captcha.length > 0 && !this.is_admin) {
            var enable_submit =
              rating_val && comment_val && name_val && captcha_val;
          } else if (captcha.length > 0 && this.is_admin) {
            var enable_submit = comment_val && name_val && captcha_val;
          } else if (captcha.length > 0 && this.is_admin && is_rating_enable) {
            var enable_submit = comment_val && name_val && rating_val;
          } else if (
            captcha.length == 0 &&
            this.is_admin &&
            !is_rating_enable
          ) {
            var enable_submit = comment_val && name_val;
          }
        } else {
          var enable_submit = comment_val && name_val;

          if (captcha.length > 0 && !this.is_admin) {
            var enable_submit = comment_val && name_val && captcha_val;
          } else if (captcha.length > 0 && this.is_admin) {
            var enable_submit = comment_val && name_val && captcha_val;
          } else if (captcha.length > 0 && this.is_admin && is_rating_enable) {
            var enable_submit = comment_val && name_val;
          } else if (
            captcha.length == 0 &&
            this.is_admin &&
            !is_rating_enable
          ) {
            var enable_submit = comment_val && name_val;
          }
        }
      }

      if (enable_submit) {
        comment_submit.prop("disabled", false);
        comment_submit.removeAttr("style");
      } else {
        comment_submit.prop("disabled", true);
        comment_submit.css("background", "grey");
        comment_submit.css("cursor", "not-allowed");
      }
    },

    check_current_user_is_admin: function() {
      var self = this;
      var post_data = {
        action: "prodigy-is-admin"
      };

      $.ajax({
        type: "post",
        data: post_data,
        dataType: "json",
        url: ajax_url,
        success: function(is_user_admin_response) {
          self.is_admin = is_user_admin_response;
          self.check_enable_comment_fields();

          return is_user_admin_response;
        }
      });
    },

    set_product_gallery: function() {
      var $sliderItemTotal = $(".total-image-count-js"),
        $sliderItemCurrent = $(".current-image-number-js");

      $("#gallery-thumbs").flexslider({
        controlNav: false,
        animation: "slide",
        slideshowSpeed: 0,
        slideshow: false,
        animationLoop: false,
        sync: "#gallery",
        start: function(slider) {
          $(".flexslider").css("overflow", "visible");
          $sliderItemCurrent.text(slider.currentSlide + 1);
          $sliderItemTotal.text(slider.count);
          if (slider.count <= 1) {
            $(".prodigy-product-gallery-nav-item-js").remove();
          }
        },
        after: function(slider) {
          $sliderItemCurrent.text(slider.currentSlide + 1);
          $sliderItemTotal.text(slider.count);
        },
        before: function(slider) {
          var animateSlide = slider.slides.eq(slider.animatingTo + 1);
          $(animateSlide)
            .find("img")
            .animate({ opacity: 1 }, 600);
        },
        controlsContainer: $(".custom-controls-container"),
        customDirectionNav: $(
          ".prodigy-product-gallery-nav-js .prodigy-product-gallery-nav-item-js"
        )
      });

      $("#gallery").flexslider({
        selector: ".prodigy-product__gallery-wrapper .prodigy-product__gallery-image",
        animation: "slide",
        controlNav: false,
        animationLoop: false,
        directionNav: true,
        slideshow: false,
        itemWidth: 150,
        itemMargin: 10,
        asNavFor: "#gallery-thumbs",
      });

      var $target = $("#gallery");
      $target.on(
        "click",
        ".icon-fullscreen-js",
        this.open_photo_swipe.bind(this)
      );
      $target.on("click", ".prodigy-product__gallery-image a", function(e) {
        e.preventDefault();
      });
    },

    get_gallery_images: function() {
      var $slides = $("ul li", "#gallery-thumbs"),
        items = [];

      if ($slides.length > 0) {
        $slides.each(function(i, el) {
          var img = $(el).find("img");

          if (img.length) {
            var large_image_src = img.attr("data-large_image"),
              large_image_w = img.attr("data-large_image_width"),
              large_image_h = img.attr("data-large_image_height"),
              item = {
                src: large_image_src,
                w: large_image_w,
                h: large_image_h,
                title: img.attr("data-caption")
                  ? img.attr("data-caption")
                  : img.attr("title")
              };
            items.push(item);
          }
        });
      }

      return items;
    },

    open_photo_swipe: function(e) {
      e.preventDefault();

      var pswpElement = $(".pswp")[0],
        items = this.get_gallery_images(),
        eventTarget = $(e.target),
        clicked;

      if (
        eventTarget.is(".prodigy-product-gallery__trigger") ||
        eventTarget.is(".prodigy-product-gallery__trigger img")
      ) {
        clicked = this.$target.find(".flex-active-slide");
      } else {
        clicked = eventTarget.closest(".prodigy-product__gallery-image");
      }

      var options = $.extend(
        {
          index: $(clicked).index()
        },
        {}
      );

      // Initializes and opens PhotoSwipe.
      var photoswipe = new PhotoSwipe(
        pswpElement,
        PhotoSwipeUI_Default,
        items,
        options
      );
      photoswipe.init();
    },

    prepare_variants: function() {
      var self = this;
      $(".attribute_values_js").focus(function() {
        var variantSelected = true;
        var variants = [];
        var selector = $(this);

        $(".attribute_values_js").each(function() {
          var $option = $(this).find("option:selected");
          var value = $option.val();
          var text = $option.text();

          if (!value) {
            variantSelected = false;
          } else {
            variants.push(text);
          }
        });

        var available_variants_list = self.get_available_variants();

        //  attributes available list
        var selectedAttributes = [];
        $(".attribute_values_js").each(function() {
          var $current_select = $(this),
            slug = $current_select.data("slug");
          if (slug != selector.data("slug") && $current_select.val()) {
            selectedAttributes[slug] = $current_select
              .find("option:selected")
              .text();
          }
        });
        // format variants list which fit for available attributes
        var used_variants = [];
        for (var index in available_variants_list) {
          var current_variant = available_variants_list[index];
          var is_use_variant = true;
          for (var attr_name in selectedAttributes) {
            if (current_variant[attr_name] !== selectedAttributes[attr_name]) {
              is_use_variant = false;
            }
          }
          if (is_use_variant) {
            used_variants.push(current_variant);
          }
        }
      });
    },

    set_attributes_behavior: function() {
      var self = this;
      self.prepare_variants();

      $(".attribute_values_js").change(function() {
        var variantSelected = true;

        var variants = [];
        var selector = $(this);

        $(".attribute_values_js").each(function() {
          var $option = $(this).find("option:selected");
          var value = $option.val();
          var text = $option.text();

          if (!value) {
            variantSelected = false;
          } else {
            variants.push(text);
          }
        });

        if (variantSelected) {
          var post_data = {
            action: "prodigy-public-get-variants-data",
            post_id: $("#product_id").val(),
            variants: variants
          };

          $.ajax({
            type: "post",
            data: post_data,
            dataType: "json",
            url: ajax_url,
            success: function(data) {
              self.set_variants_data(selector, data);
              self.variant = data;
              if (typeof data.check_requires_shipping !== "undefined") {
                var enable_shipping = data.check_requires_shipping[0] === "on";
                if (typeof this._weight_type_mapper !== "undefined") {
                  var weight_value =
                    data.product_weight_value[0] +
                    this._weight_type_mapper[data.product_weight_type[0]];
                  var dimension_value =
                    data.shipping_dimension_width[0] +
                    "x" +
                    data.shipping_dimension_width[0] +
                    "x" +
                    data.shipping_dimension_length[0] +
                    this._dimension_type_mapper[data.dimension[0]];
                  self.set_shipping_info_for_variant(
                    enable_shipping,
                    weight_value,
                    dimension_value
                  );
                }
              }
            }
          });
        } else {
          self.show_main_product_info();
          self.pg_variations_image_reset(selector);

          if (
            self.product.meta &&
            typeof self.product.meta.product_requires_shipping !== "undefined"
          ) {
            var meta = self.product.meta;
            var enable_shipping = meta.product_requires_shipping[0] === "on";
            if (typeof this._weight_type_mapper !== "undefined") {
              var weight_value =
                meta.product_weight_value[0] +
                this._weight_type_mapper[meta.product_weight_type[0]];
              var dimension_value =
                meta.shipping_dimension_width[0] +
                "x" +
                meta.shipping_dimension_width[0] +
                "x" +
                meta.shipping_dimension_length[0] +
                this._dimension_type_mapper[meta.shipping_dimensions[0]];
              self.set_shipping_info_for_variant(
                enable_shipping,
                weight_value,
                dimension_value
              );
            }
          }
        }
      });
    },

    get_available_variants: function() {
      var post_data = {
        action: "prodigy-public-get-available-variants",
        post_id: $("#product_id").val()
      };

      var variants = [];

      $.ajax({
        type: "post",
        data: post_data,
        dataType: "json",
        url: ajax_url,
        async: false,
        success: function(data) {
          variants = data;
        }
      });

      return variants;
    },

    set_shipping_info_for_variant: function(
      enable_shipping,
      weight_value,
      dimension_value
    ) {
      $(".prodigy-product__additional tr").each(function() {
        if (enable_shipping) {
          var col_value = $(this)
            .find("td:eq(0)")
            .text();

          if (col_value === "WEIGHT") {
            $(this)
              .find("td:eq(1)")
              .text(weight_value);
          }

          if (col_value === "DIMENSIONS") {
            $(this)
              .find("td:eq(1)")
              .text(dimension_value);
          }
        }
      });
    },

    productCounter: function(counterCount) {
      counterCount.value = parseInt(counterCount.val());
      var currentCount = +counterCount.value;

      return {
        plus: function() {
          return ++currentCount;
        },
        minus: function() {
          return --currentCount;
        },
        reset: function(value) {
          currentCount = value;
        },
        getCount: function() {
          return currentCount;
        }
      };
    },

    set_product_counter: function() {
      const counterCount = $(".counter-count-js");
      const minusBtn = $(".counter-btn-minus-js");
      const plusBtn = $(".counter-btn-plus-js");
      const minItems = 1;
      const maxItems = Infinity;

      var counter = this.productCounter(counterCount);
      counterCount.inputmask({ regex: "[0-9]*", rightAlign: false });

      $(document).on("click", ".counter-btn-minus-js", function() {
        if (counter.getCount() !== minItems) {
          counterCount.val(counter.minus());
          if (counter.getCount() <= 1) {
            minusBtn.prop("disabled", true);
            minusBtn.css("background", "grey");
            minusBtn.css("cursor", "not-allowed");
          }
        }
      });

      $(document).on("click", ".counter-btn-plus-js", function() {
        if (counter.getCount() < maxItems) {
          counterCount.val(counter.plus());
          if (counter.getCount() > 1) {
            minusBtn.prop("disabled", false);
            minusBtn.removeAttr("style");
          }
        }
      });

      $(document).on("change", ".counter-count-js", function() {
        const value = +this.value;

        counter.reset(value);

        if (value >= maxItems) {
          this.value = maxItems;
          counter.reset(maxItems);
        }
        if (isNaN(value) || !value) {
          this.value = minItems;
          counter.reset(minItems);
        }
      });
    },

    show_price: function(price, sale_price, select_variant = 0) {
      var attribute_select = $(".attribute_values_js");
      var stock_info = $(".prodigy-product-stock-js");
      var regular_price_info = $(".regular-price-container");
      var sale_price_info = $(".sale-price-container");
      var sale_price_value_block = $(".sale-price");
      var regular_price_value_block = $(".regular-price");

      if (attribute_select.length !== 0 && !select_variant) {
        stock_info.hide();
        regular_price_info.hide();
        sale_price_info.hide();
      } else {
        if (
          typeof sale_price !== "undefined" ||
          (typeof sale_price !== "undefined" && parseInt(sale_price[0]) !== 0)
        ) {
          if (price[0] === "" || parseInt(sale_price[0]) === 0) {
            stock_info.hide();
            regular_price_info.hide();
            sale_price_info.hide();
          } else {
            regular_price_info.show();
            sale_price_info.show();
            stock_info.show();
          }

          if (sale_price[0] === "") {
            sale_price_info.show();
            if (typeof price[0] && price) {
              sale_price_value_block.text(prodigy_price_format(price[0]));
              regular_price_info.hide();
            }
          } else {
            if (price && typeof price[0] !== "undefined") {
              regular_price_value_block.text(prodigy_price_format(price[0]));
              sale_price_value_block.text(prodigy_price_format(sale_price[0]));
              regular_price_info.show();
              sale_price_info.show();
            }
          }
        } else {
          sale_price_info.hide();
          regular_price_info.hide();
        }
      }
    },

    show_main_product_info: function() {
      if (
        typeof this.product.meta !== "undefined" &&
        this.product.meta.product_regular_price !== null
      ) {
        this.show_price(
          this.product.meta.product_regular_price,
          this.product.meta.product_sale_price
        );
        this.show_status(this.product.meta.product_stock_status);
      }

      if (
        this.product.meta &&
        typeof this.product.meta.product_sku !== "undefined" &&
        this.product.meta.product_sku[0]
      ) {
        var main_product_sku = this.product.meta.product_sku[0];
        $(".product_sku_value").text(main_product_sku);
      }
    },

    show_status: function(status, select_variant = 0) {
      if (typeof status !== "undefined") {
        var stock_block = $(".prodigy-product-stock-js");
        var cart_button = $("button.add-to-cart-js");
        var plus = $(".counter-btn-plus-js");
        var minus = $(".counter-btn-minus-js");
        var count_input = $(".counter-count-js");

        if (parseInt(status[0]) === 0) {
          if ($(".attribute_values_js").length !== 0 && !select_variant) {
            cart_button.prop("disabled", true);
            cart_button.css("background", "grey");
            cart_button.css("cursor", "not-allowed");
            plus.prop("disabled", true);
            plus.css("background", "grey");
            plus.css("cursor", "not-allowed");
            count_input.prop("disabled", true);

            minus.prop("disabled", true);
            minus.css("background", "grey");
            minus.css("cursor", "not-allowed");

            stock_block.hide();
          } else {
            if (parseInt($(".counter-count-js").val()) === 1) {
              minus.prop("disabled", true);
              minus.css("background", "grey");
              minus.css("cursor", "not-allowed");
            } else {
              minus.prop("disabled", false);
              minus.removeAttr("style");
            }

            count_input.prop("disabled", false);
            plus.prop("disabled", false);
            plus.removeAttr("style");

            cart_button.prop("disabled", false);
            cart_button.removeAttr("style");
            stock_block.text(this._stock_status_mapper[0]);
          }
        } else {
          minus.css("cursor", "not-allowed");
          minus.prop("disabled", true);
          minus.css("background", "grey");

          count_input.prop("disabled", true);
          plus.css("cursor", "not-allowed");
          plus.prop("disabled", true);
          plus.css("background", "grey");
          cart_button.prop("disabled", true);
          cart_button.css("background", "grey");
          cart_button.css("cursor", "not-allowed");
          stock_block.text(this._stock_status_mapper[1]);
        }
      }
    },

    get_actual_price: function(price, sale_price) {
      if (sale_price === '') {
        return price;
      } else {
        return sale_price;
      }
    },

    set_variants_data: function(selector, data) {
      var self = this;

      if (typeof data !== 'undefined') {

        if (options.pg_google_track_id) {
          self.set_view_product_analytic_event(data);
        }

        var cart_button = $("button.add-to-cart-js");
        if (
            data &&
            typeof data.product_sku !== "undefined" &&
            data.product_sku[0]
        ) {
          var sku = data.product_sku[0];
        }
        this.show_status(data.product_stock_status, true);
        this.show_price(
            data.product_regular_price,
            data.product_sale_price,
            true
        );

        if (sku !== "" && typeof sku !== "undefined") {
          $(".product_sku_value").text(sku);
        } else {
          $(".prodigy-product__tags-item product_sku").hide();
        }

        // set product data for cart

        if (typeof data !== "undefined") {
          if (typeof data.variant_remote_id !== "undefined") {
            cart_button.attr("data-remote-id", data.variant_remote_id[0]);
          }

          if (typeof data.variant_local_id !== "undefined") {
            cart_button.attr("data-local-id", data.variant_local_id[0]);
          }
        }

        var $product = selector.closest(".product-container-js"),
            $product_gallery = $product.find(".images-gallery-js"),
            $gallery_nav = $product.find("#gallery-thumbs"),
            $gallery_img = $gallery_nav.find("li:eq(0) img"),
            $product_img_wrap = $product_gallery
                .find(
                    ".prodigy-product__gallery-image, .prodigy-product__gallery-image--placeholder"
                )
                .eq(0),
            $product_img = $product_img_wrap.find(".wp-post-image"),
            $product_link = $product_img_wrap.find("a").eq(0);

        if (data.img_url) {
          var galleryHasImage =
              $gallery_nav.find('li img[data-o_src="' + data.img_url + '"]')
                  .length > 0;

          // If the gallery has the image, reset the images. We'll scroll to the correct one.
          if (galleryHasImage) {
            self.pg_variations_image_reset();
          }

          // See if gallery has a matching image we can slide to.
          var slideToImage = $gallery_nav.find(
              'li img[src="' + data.img_url + '"]'
          );

          self.pg_set_variation_attr($product_img, "src", data.img_url);
          self.pg_set_variation_attr(
              $product_img_wrap,
              "data-thumb",
              data.img_url
          );
          self.pg_set_variation_attr($gallery_img, "src", data.img_url);
          self.pg_set_variation_attr($product_link, "href", data.img_url);
        } else {
          self.pg_variations_image_reset(selector);
        }
      }
    },

    pg_variations_image_reset: function(selector) {
      if (typeof selector !== "undefined") {
        var $product = selector.closest(".product-container-js"),
          $product_gallery = $product.find(".images-gallery-js"),
          $gallery_nav = $product.find("#gallery-thumbs"),
          $gallery_img = $gallery_nav.find("li:eq(0) img"),
          $product_img_wrap = $product_gallery
            .find(
              ".prodigy-product__gallery-image, .prodigy-product__gallery-image--placeholder"
            )
            .eq(0),
          $product_img = $product_img_wrap.find(".wp-post-image"),
          $product_link = $product_img_wrap.find("a").eq(0);

        this.pg_reset_variation_attr($product_img, "src");
        this.pg_reset_variation_attr($product_img, "width");
        this.pg_reset_variation_attr($product_img, "height");
        this.pg_reset_variation_attr($product_img, "srcset");
        this.pg_reset_variation_attr($product_img, "sizes");
        this.pg_reset_variation_attr($product_img, "title");
        this.pg_reset_variation_attr($product_img, "data-caption");
        this.pg_reset_variation_attr($product_img, "alt");
        this.pg_reset_variation_attr($product_img, "data-src");
        this.pg_reset_variation_attr($product_img, "data-large_image");
        this.pg_reset_variation_attr($product_img, "data-large_image_width");
        this.pg_reset_variation_attr($product_img, "data-large_image_height");
        this.pg_reset_variation_attr($product_img_wrap, "data-thumb");
        this.pg_reset_variation_attr($gallery_img, "src");
        this.pg_reset_variation_attr($product_link, "href");
      }
    },

    pg_reset_variation_attr: function(element, attr) {
      if (undefined !== element.attr("data-o_" + attr)) {
        element.attr(attr, element.attr("data-o_" + attr));
      }
    },

    pg_set_variation_attr: function(element, attr, value) {
      if (undefined === element.attr("data-o_" + attr)) {
        element.attr(
          "data-o_" + attr,
          !element.attr(attr) ? "" : element.attr(attr)
        );
      }
      if (false === value) {
        element.removeAttr(attr);
      } else {
        element.attr(attr, value);
      }
    },

    set_global_product: function() {
      var post_data = {
        action: "prodigy-public-get-product",
        post_id: $("#product_id").val()
      };

      var self = this;

      $.ajax({
        type: "post",
        data: post_data,
        dataType: "json",
        url: ajax_url,
        success: function(data) {
          self.product = data;
          self.show_main_product_info(false);

          // init cart
          self.add_item_to_cart();
        }
      });
    },

    show_tab_review: function() {
      $("body").on("click", ".prodigy-review-link-js", function() {
        $("#tab-reviews")
          .addClass("active")
          .show();
        $("#tab-li-reviews").addClass("active");
        $("#tab-description").hide();
        $("#tab-additional_information").hide();
        $(".description_tab").removeClass("active");
        $(".additional_information_tab").removeClass("active");
        $(".reviews_tab").addClass("active");
        document.getElementById("tab-reviews").scrollIntoView();
      });
    },

    set_product_tabs: function() {
      $(".description_tab").addClass("active");
      $("#tab-additional_information").hide();
      $("#tab-reviews").hide();


      $('.prodigy-review-link-js, .description_tab, .additional_information_tab, .reviews_tab').on('click', function() {
        window.location.href = '#prodigy-tabs';
      });

      $('.prodigy-review-link-js').on('click', function(){
        if ($('.prodigy-review__item').length === 0 ) {
          window.location.href = '#prodigy-tabs';
        }
      });


      $("body")
        // Tabs
        .on("init", ".prodigy-tabs-js, .prodigy-tabs", function() {
          $(".pg-tab, .prodigy-tabs .panel:not(.panel .panel)").hide();
          var hash = window.location.hash;
          var url = window.location.href;
          var $tabs = $(this)
            .find(".pg-tabs, ul.tabs")
            .first();

          if (
            hash.toLowerCase().indexOf("comment-") >= 0 ||
            hash === "#reviews" ||
            hash === "#tab-reviews"
          ) {
            $tabs.find("li.reviews_tab a").click();
          } else if (
            url.indexOf("comment-page-") > 0 ||
            url.indexOf("cpage=") > 0
          ) {
            $tabs.find("li.reviews_tab a").click();
          } else if (hash === "#tab-additional_information") {
            $tabs.find("li.additional_information_tab a").click();
          } else {
            var $tab = $tabs.find("li:first a");
            var $tabs_wrapper = $tab.closest(".prodigy-tabs-js, .prodigy-tabs");
            $tabs.find("li").removeClass("active");
            $tabs_wrapper.find(".pg-tab, .panel:not(.panel .panel)").hide();

            $tab.closest("li").addClass("active");
            $tabs_wrapper.find($tab.attr("href")).show();
          }
        })
        .on("click", ".pg-tabs li a, ul.tabs li a", function(e) {
          var $tab = $(this);
          var $tabs_wrapper = $tab.closest(".prodigy-tabs-js, .prodigy-tabs");
          var $tabs = $tabs_wrapper.find(".pg-tabs, ul.tabs");

          $tabs.find("li").removeClass("active");
          $tabs_wrapper.find(".pg-tab, .panel:not(.panel .panel)").hide();

          $tab.closest("li").addClass("active");
          $tabs_wrapper.find($tab.attr("href")).show();
        })

        // Star ratings for comments
        .on("init", "#prodigy-rating", function() {
          $("#prodigy-rating")
            .hide()
            .before(
              "" +
                '<div class="stars prodigy-comment__rating">' +
                '<a class="star-1 prodigy-comment__star icon icon-star" href="#">1</a>' +
                '<a class="star-2 prodigy-comment__star icon icon-star" href="#">2</a>' +
                '<a class="star-3 prodigy-comment__star icon icon-star" href="#">3</a>' +
                '<a class="star-4 prodigy-comment__star icon icon-star" href="#">4</a>' +
                '<a class="star-5 prodigy-comment__star icon icon-star" href="#">5</a>' +
                "</div>"
            );
        })
        .on("click", "#respond div.stars a", function() {
          var $star = $(this),
            $rating = $(this)
              .closest("#respond")
              .find("#prodigy-rating"),
            $container = $(this).closest(".stars");
          $rating.val($star.text()).trigger("change");
          $star.siblings("a").removeClass("active");
          $star.addClass("active");
          $container.addClass("selected");

          return false;
        })

        .on("click", "#reviews #comments .justify-content-center", function() {
          window.prodigyProduct
            .get_count_review()
            .done(function(result) {
              let count_review = result.data;
              if (count_review > 0) {
                window.prodigyProduct.get_content_review();
              }
            })
            .fail(function() {});
        });

      $(".prodigy-tabs-js, .prodigy-tabs, #prodigy-rating").trigger("init");
    },

    get_content_review: function() {
      let self = $("#reviews #comments .justify-content-center");
      let page = $(".per-page-js").data("page");
      let post_data = {
        action: "prodigy-public-get-comments",
        post_id: $("#product_id").val(),
        page: page
      };

      $.ajax({
        type: "post",
        data: post_data,
        dataType: "text",
        url: ajax_url,
        success: function(data) {
          self.before(data);
          $(".per-page-js").data("page", page + 1);

          window.prodigyProduct.get_count_review().done(function(res) {
            if (res.data === 0) {
              window.prodigyProduct.hide_link_show_more_reviews();
            }
          });
        }
      });
    },

    get_count_review: function() {
      let post_data = {
        action: "prodigy-public-get-comments-count",
        post_id: $("#product_id").val(),
        page: $(".per-page-js").data("page")
      };

      return $.ajax({
        type: "post",
        data: post_data,
        dataType: "json",
        url: ajax_url
      });
    },

    hide_link_show_more_reviews: function() {
      $(".link-show-more-reviews-js")
        .removeClass("d-flex")
        .addClass("d-none");
    },

    set_default_link_review: function() {
      window.prodigyProduct.get_count_review().done(function(res) {
        if (res.data === 0) {
          window.prodigyProduct.hide_link_show_more_reviews();
        }
      });
    },

    init_show_more: function() {
      $("body").on(
        "click",
        ".prodigy-product__description-show-more",
        function() {
          var $container = $(this).parent();
          $container
            .find(".prodigy-product__description-container")
            .removeClass("prodigy-product__description-container--truncated");
          $container.find(".prodigy-product__description-fade").remove();
          $(this).remove();
        }
      );
    }
  };

  window.prodigyProduct = prodigy_product;
})(jQuery, window);

jQuery(function($) {
  window.prodigyProduct.init();
  window.prodigyRecaptchaCallback = window.prodigyProduct.captcha_callback.bind(
    window.prodigyProduct
  );
});
